/*  asteroid_client.c   */
/*  Jure Skvarc, January 1999  */
/*  A program which connects to the asteroid server and sends user  */
/*  queries to it.  It displays the results on the standard output  */
/*==================================================================*/
#include <stdio.h>
#include <malloc.h>

#include <sys/time.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>
#include <limits.h>
#include <netdb.h>
#include <arpa/inet.h>



#define ASTEROID_PORT 58972
#define RESULT_LENGTH 81

/* Converts ascii text to in_addr struct.  NULL is returned if the address
   can not be found. */

struct in_addr *
atoaddr(char *address)

{
  struct hostent *host;
  static struct in_addr saddr;

  /* First try it as aaa.bbb.ccc.ddd. */
  saddr.s_addr = inet_addr(address);
  if (saddr.s_addr != -1) {
    return &saddr;
  }
  host = gethostbyname(address);
  if (host != NULL) {
    return (struct in_addr *) *host->h_addr_list;
  }
  return NULL;
}


int
make_connection(char *netaddress, const char **p)

{
  int port = ASTEROID_PORT;
  int type = SOCK_STREAM;
  struct in_addr *addr;
  int sock, connected;
  struct sockaddr_in address;

  addr = atoaddr(netaddress);
  if (addr == NULL) {
    *p = "make_connection:  Invalid network address!\n";
    return -1;
  }
 
  memset((char *) &address, 0, sizeof(address));
  address.sin_family = AF_INET;
  address.sin_port = htons(port);
  address.sin_addr.s_addr = addr->s_addr;

  sock = socket(AF_INET, type, 0);

  connected = connect(sock, (struct sockaddr *) &address, sizeof(address));
  if (connected < 0) {
    *p = "make_connection:  Connect failed!\n";
    return -1;
  }
  return sock;
}

/*======================================*/
/*  Read a string from the server       */
/*======================================*/
int 
read_from_server(int sockfd, char *str, size_t count)

{
  int n;
  int i = 0;

  do {
    n = read(sockfd, str + i, 1);
    if (n <= 0) {
      /* The other side may have closed unexpectedly */
      return -1; 
    }
  } while (str[i++] != '\0' && i < count);
  str[i] = '\0';
  return i;
}


/*==========================================*/
/*  Wait for data from the server           */
/*  Return the result of the select function*/
/*  0 - timeout                             */
/*  1 - something to read                   */
/*  -1 - error                              */
/*  0 - timeout                             */
/*  timeout is given in milliseconds        */
/*==========================================*/
int
wait_for_data(int sc, int timeout)

{
  fd_set set;
  struct timeval tout;
  int w;

  FD_ZERO(&set);
  FD_SET(sc, &set);  /*  Set the descriptor we want to check  */
  tout.tv_sec = timeout / 1000;
  tout.tv_usec = timeout % 1000;
  w = select(sc + 1, &set, NULL, NULL, &tout);
  return w;
}


/*===========================*/
/*  Send data to server      */
/*===========================*/
int
sock_write(int sockfd, char *buf, size_t count)

{
  size_t bytes_sent = 0;
  int this_write;

  count += 1;
  while (bytes_sent < count) {
    do
      this_write = write(sockfd, buf, count - bytes_sent);
    while ( (this_write < 0) && (errno == EINTR) );
    if (this_write <= 0)
      return this_write;
    bytes_sent += this_write;
    buf += this_write;
  }
  return count;
}
