/*  asteroids.c                    */
/*  Part of the fitsblink program  */
/*  Jure Skvarc, february 1999     */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "../functs.h"

extern STATE state;
extern BLINK_FRAME *frame;


/*  Transform ASCII data in the string line into the structure asteroid  */
int
read_fitsline(char *line, ASTEROID *asteroid, int n)

{
  char ra[12], dec[12];
  char *p, *p1;

  /*  First check if this is a WARNING or ERROR line  */
  if (strstr(line, "ERROR") || strstr(line, "WARNING"))
    return 1;
  p1 = strdup(line);
  p = strtok(p1, " ");
  if (p) sscanf(p, "%d", &asteroid[n].num);
  else return 1;
  if (strlen(line) < 25) return 1;
  p1[24] = '\0';
  strcpy(asteroid[n].name, p1 + 6);
  p = strtok(p1 + 25, " ");
  if (p) {
    sscanf(p, "%s", ra);
    asteroid[n].ra = parse_ra_c(ra);
  }
  else return 1;
  p = strtok(NULL, " ");
  if (p) {
    sscanf(p, "%s", dec);
    asteroid[n].dec = parse_dec_c(dec);
  }
  else return 1;
  p = strtok(NULL, " ");
  if (p) sscanf(p, "%lf", &asteroid[n].x);
  else return 1;
  p = strtok(NULL, " ");
  if (p) sscanf(p, "%lf", &asteroid[n].y);
  else return 1;
  p = strtok(NULL, " ");
  if (p) sscanf(p, "%f", &asteroid[n].magnitude);
  else return 1;

  free(p1);
  return 0;
}

/*  This function connects to the asteroid server, and gets pixel coordinates of  */
/*  all of the asteroids in the given screen  */
int
get_asteroids(BLINK_FRAME *frame, int m, char *address, char *id, float maxmag)

{
  char *p;
  char result[RESULT_LENGTH];
  char oneline[RESULT_LENGTH];
  char command[256];
  int sc, len;
  int i, n;
  ASTEROID asteroid[REZMAXNUM];

  /*  Try to establish connection with the server  */
  /*************************************************/
  if (state.interactive) {
    open_message("Establishing connection");
  } 
  sc = make_connection(address, &p);
  if (sc < 0) {
    close_message();
    if (state.interactive) {
      fl_show_alert(p, "", "", 1);
    } 
    return -1;
  }
  sock_write(sc, id, strlen(id));
  if (state.interactive) {
    write_message("Waiting for server response");
  } 
  /*  Wait for intro string for 30 seconds  */
  /******************************************/
  do {
    if (wait_for_data(sc, 30000) <= 0) {
      close_message();
      if (state.interactive) {
	fl_show_alert("Server timeout or error", "", "", 1);
      } 
      return -1;
    }  
    result[0] = '\0';
    len = read_from_server(sc, result, RESULT_LENGTH);
  } while (!(len < RESULT_LENGTH));
  if (state.interactive) {
    write_message("Sending command");
  } 
  /*  Prepare a command  */
  sprintf(command, "fits %.8f %.6f %.6f %.6f %.6f %e %e %e %d %d %.2f\n", frame[m].fitsfile.jd, 
	  frame[m].fitsfile.crval1, frame[m].fitsfile.crval2,
	  frame[m].fitsfile.crpix1, frame[m].fitsfile.crpix2,
	  frame[m].fitsfile.cdelt1, frame[m].fitsfile.cdelt2,
	  frame[m].fitsfile.crot, 
	  (int) frame[m].fitsfile.nax[0], (int) frame[m].fitsfile.nax[1], maxmag);
  sock_write(sc,  command, strlen(command));
  if (state.interactive) {
    write_message("Waiting for response");
  } 
  oneline[0] = '\0';
  n = 0;
  do {
    if (wait_for_data(sc, 30000) <= 0) {
      close_message();
      if (state.interactive) {
	fl_show_alert("Server timeout or error", "", "", 1);
      } 
      return -1;
    }  
    /*  Read at most RESULT_LENGTH bytes from the server  */
    len = read_from_server(sc, result, RESULT_LENGTH);
    if (len > 0) {
      if (len < RESULT_LENGTH) {
	/*  This is the last line of the result  */
	/*  Append it to whatever is in the online string  */
	p = oneline + strlen(oneline);
	for (i = 0; i < strlen(result); i++) {
	  if (result[i] != '\n') {
	    *p++ = result[i];
	  }
	  else {
	    i++;
	    break;
	  }
	}
	*p = '\0';
	if (read_fitsline(oneline, asteroid, n) == 0)
	  n++;
	do {
	  /*  Now copy the rest of the result line to the beginning of the */
	  /*  oneline string  */
	  p = oneline;
	  while(i < RESULT_LENGTH && result[i] != '\n') {
	    *p++ = result[i++];
	  }
	  *p = '\0';
	  if (i < strlen(result)) {
	    i++;
	    if (read_fitsline(oneline, asteroid, n) == 0)
	      n++;
	  }
	} while (i < strlen(result));
      }
      else {
	/*  Result line is exactly RESULT_LENGTH long  */
	/*  so it must contain a \n                    */
	/*  Find it and append to the oneline string    */
	p = oneline + strlen(oneline);
	for (i = 0; i < RESULT_LENGTH; i++) {
	  if (result[i] != '\n') {
	    *p++ = result[i];
	  }
	  else {
	    *p = '\0';
	    i++;
	    break;
	  }
	}
	if (read_fitsline(oneline, asteroid, n) == 0)
	  n++;
	do {
	  /*  Now copy the rest of the result line to the beginning of the */
	  /*  oneline string  */
	  p = oneline;
	  while(i < RESULT_LENGTH && result[i] != '\n') {
	    *p++ = result[i++];
	  }
	  *p = '\0';
	  if (i < RESULT_LENGTH) {
	    i++;
	    if (read_fitsline(oneline, asteroid, n) == 0)
	      n++;
	  }
	} while (i < RESULT_LENGTH);
      }
    }
  } while (!(len < RESULT_LENGTH));
  close_message();
  close(sc);
  if (n > 0) {
    if (state.interactive) {
      remove_all_markers(frame, m);
      if (frame[m].imagelist.s) free(frame[m].imagelist.s);
    }
    reserve_marker_space(frame, m, n);
    frame[m].imagelist.s = (STAR *) myalloc(sizeof(STAR) * n, "get_asteroids", 
					  "frame[m].obj");
    for (i = 0; i < n; i++) {
      char s[20];
      if (asteroid[i].num != 0) {
	sprintf(s, "%d", asteroid[i].num);
      }
      else {
	strcpy(s, strdup(asteroid[i].name));
      }
      append_marker(frame, m, asteroid[i].x, asteroid[i].y, 5, 
		    strdup(s), FL_YELLOW, OBJ_BOX);
      frame[m].imagelist.s[i].n = i;
      frame[m].imagelist.s[i].x = asteroid[i].x;
      frame[m].imagelist.s[i].y = asteroid[i].y;
      frame[m].imagelist.s[i].ra = asteroid[i].ra;
      frame[m].imagelist.s[i].dec = asteroid[i].dec;
      frame[m].imagelist.s[i].m = asteroid[i].magnitude;
      frame[m].imagelist.s[i].catnum = 0;
    }
    frame[m].imagelist.n = n;
  }
  if (state.interactive) {
    set_image(frame, m);
    draw_image(frame, m);
  }
  return n;
}
		


/*  Transform ASCII data in the string line into the structure asteroid  */
int
read_checkline(char *line, ASTEROID *asteroid, int n)

{
  char ra[12], dec[12];
  char *p, *p1;

  p1 = strdup(line);
  p = strtok(p1, " ");
  if (p) sscanf(p, "%d", &asteroid[n].num);
  else return 1;
  if (strlen(line) < 25) return 1;
  p1[24] = '\0';
  strcpy(asteroid[n].name, p1 + 6);
  p = strtok(p1 + 25, " ");
  if (p) {
    sscanf(p, "%s", ra);
    asteroid[n].ra = parse_ra_c(ra);
  }
  else return 1;
  p = strtok(NULL, " ");
  if (p) {
    sscanf(p, "%s", dec);
    asteroid[n].dec = parse_dec_c(dec);
  }
  else return 1;
  p = strtok(NULL, " ");
  if (p) sscanf(p, "%f", &asteroid[n].magnitude);
  else return 1;
  p = strtok(NULL, " ");
  if (p) sscanf(p, "%f", &asteroid[n].err);
  else return 1;
  free(p1);
  return 0;
}


/*  Make a connection to the asteroid server at address and get a list of  */
/*  objects that are at time within dist arc-minutes to the coordinate     */
/*  ra, dec  */ 

int
check_asteroid(ASTEROID *asteroid, double time, double ra, double dec, double dist, char *address, char *id, float maxmag)

{
  char *p;
  char result[RESULT_LENGTH];
  char oneline[RESULT_LENGTH];
  char command[128];
  int sc, len;
  int i, n;

  if (id == NULL || strlen(id) == 0) return -1;
  /*  Try to establish connection with the server  */
  /*************************************************/
  if (state.interactive) {
    open_message("Establishing connection");
  } 
  sc = make_connection(address, &p);
  if (sc < 0) {
    if (state.interactive) {
      fl_show_alert(p, "", "", 1);
    } 
    close_message();
    return -1;
  }
  sock_write(sc, id, strlen(id));
  if (state.interactive) {
    write_message("Waiting for server response");
  } 
  /*  Wait for intro string for 30 seconds  */
  /******************************************/
  do {
    if (wait_for_data(sc, 30000) <= 0) {
      if (state.interactive) {
	fl_show_alert("Server timeout or error", "", "", 1);
      } 
      close_message();
      return -1;
    }  
    result[0] = '\0';
    len = read_from_server(sc, result, RESULT_LENGTH);
  } while (!(len < RESULT_LENGTH));
  if (state.interactive) {
    write_message("Sending command");
  } 
  /*  Prepare a command  */
  sprintf(command, "check jd%.8f %s %s %f %.2f\n", time,
	  format_ra(ra), format_dec(dec), dist, maxmag);
  sock_write(sc,  command, strlen(command));
  if (state.interactive) {
    write_message("Waiting for response");
  } 
  oneline[0] = '\0';
  n = 0;
  do {
    if (wait_for_data(sc, 30000) <= 0) {
      if (state.interactive) {
	fl_show_alert("Server timeout or error", "", "", 1);
      } 
      close_message();
      return -1;
    }  
    /*  Read at most RESULT_LENGTH bytes from the server  */
    len = read_from_server(sc, result, RESULT_LENGTH);
    if (len > 0) {
      if (len < RESULT_LENGTH) {
	/*  This is the last line of the result  */
	/*  Append it to whatever is in the online string  */
	p = oneline + strlen(oneline);
	for (i = 0; i < strlen(result); i++) {
	  if (result[i] != '\n') {
	    *p++ = result[i];
	  }
	  else {
	    i++;
	    break;
	  }
	}
	*p = '\0';
	if (read_checkline(oneline, asteroid, n) == 0)
	  n++;
	do {
	  /*  Now copy the rest of the result line to the beginning of the */
	  /*  oneline string  */
	  p = oneline;
	  while(i < RESULT_LENGTH && result[i] != '\n') {
	    *p++ = result[i++];
	  }
	  *p = '\0';
	  if (i < strlen(result)) {
	    i++;
	    if (read_checkline(oneline, asteroid, n) == 0)
	      n++;
	  }
	} while (i < strlen(result));
      }
      else {
	/*  Result line is exactly RESULT_LENGTH long  */
	/*  so it must contain a \n                    */
	/*  Find it and append to the oneline string    */
	p = oneline + strlen(oneline);
	for (i = 0; i < RESULT_LENGTH; i++) {
	  if (result[i] != '\n') {
	    *p++ = result[i];
	  }
	  else {
	    *p = '\0';
	    i++;
	    break;
	  }
	}
	if (read_checkline(oneline, asteroid, n) == 0)
	  n++;
	do {
	  /*  Now copy the rest of the result line to the beginning of the */
	  /*  oneline string  */
	  p = oneline;
	  while(i < RESULT_LENGTH && result[i] != '\n') {
	    *p++ = result[i++];
	  }
	  *p = '\0';
	  if (i < RESULT_LENGTH) {
	    i++;
	    if (read_checkline(oneline, asteroid, n) == 0)
	      n++;
	  }
	} while (i < RESULT_LENGTH);
      }
    }
  } while (!(len < RESULT_LENGTH));
  close_message();
  close(sc);
  return n;
}




/*  This function is executed when the user presses the Asteroid button  */
/*  It finds all asteroids which are present on the image in marks their */
/*  positions     */
void
asteroidsC(FL_OBJECT *obj, long val)

{
  int n = state.control[state.control_num];

  if (state.frame_used > 0) {
    if (!frame[n].fitsfile.matched && !frame[n].matched) {
      if (!fl_show_question("There is no indication that the WCS values can be trusted.\n
Would you like to continue anyway?", 1)) {
	return;
      }
    }
    if (state.asteroid_server == NULL || strlen(state.asteroid_server) == 0 ||
	state.asteroid_id == NULL || strlen(state.asteroid_id) == 0) {
      fl_show_alert("Server address not specified!", "", "", 1);
    }
    else {
      frame[n].objnum = get_asteroids(frame, n, state.asteroid_server, state.asteroid_id, state.asteroid_maxmag);
      if (frame[n].objnum > 0) {
	draw_objects(frame, n);
	draw_image(frame, n);
      }
    }
  }
  else {
    fl_show_alert("No image loaded!", "", "", 1);
  }

}

