/*  astrometry.c  */
/*  part of the fitsblink program  */
/*  routines for calling of astrometry related functions */
/*  Jure Skvarc                              */
#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <forms.h>

#include "formblink.h"
#include "functs.h"

extern STATE state;
extern BLINK_FRAME *frame;

/*  Convert the designation from long to packed format  */
void
designation_to_packed(char *name, char *packed)

{
  /*  Convert from long to compact format, if possible */
  char a[19], *p;
  int year, num, cent;

  strcpy(a, name);
  p = strtok(a, " ");
  if (strstr(name, "P-L") || strstr(name, "T-1") || strstr(name, "T-2") || strstr(name, "T-3")) {
    sprintf(packed, "%c%cS%s", name[5], name[7], p);
  }
  else if (p) {
    year = strtol(p, NULL, 10);
    /*  Does it look like a year?  */
    if (year > 1900 && year < 2100) {
      cent = year / 100 - 19 + 'J';
      p = strtok(NULL, " ");
      if (p) {
	num = strtol(p + 2, NULL, 10);
	sprintf(packed, "%c%02d%c%c%c%c", cent, year % 100, p[0], 
		num < 100 ? num / 10 + '0' : num / 10 - 10 + 'A',
		num % 10 + '0', p[1]);
      }
    }
  }
}

void 
astrometryC(FL_OBJECT *obj, long val) 

{
  int m;
  int x, y, z;

  m = fl_get_menu(obj);
  switch (m) {
  case EDIT_PARAMETERS:
    edit_parameters(obj, m);
    break;
  case BLUR_IMAGE:
    blur(&frame[state.control[state.control_num]].fits);
    set_image(frame, state.control[state.control_num]);
    draw_image(frame, state.control[state.control_num]);
    
    break;
  case DETECT_STARS:
    detect_stars(frame, state.control[state.control_num]);
    break;
  case MATCH_STARS:
    match_stars(frame, state.control[state.control_num]);
    break;
  case SAVE_STAR_LIST:
    star_list(frame, state.control[state.control_num]);
    break;
  case LOAD_STAR_LIST:
    load_star_list(frame, state.control[state.control_num]);
    break;
  case DO_ASTROMETRY:
    state.doastr = 1;
    fl_get_mouse(&x, &y, &z);
    fl_set_mouse(0, 0);
    fl_set_mouse(x, y);
    break;
  case OPTIONS:
    edit_options(obj, m);
    break;
  case CATALOGS:
    edit_catalogs(obj, m);
    break;
  case SAVE_SETTINGS:
    show_settings(obj, m);
    break;
  case SEND_MAIL:
    edit_mail(obj, m);
    break;
  case SHOW_REPORT:
    show_report();
    break;
  }
}

