/*  blink.c  for fitsblink */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <sys/types.h> 
#include <sys/time.h>
#include <unistd.h>
#include <getopt.h>
#include <time.h>

#include "functs.h"
#include "consts.h"

STATE state;
BLINK_FRAME *frame;
CATALOG catalog[NUMBER_OF_CATALOGS];
OBSERVATORY observatory;
MAIL mail;
FITS_IMAGE count_options;
MATCH_OPTIONS match_options;
TELESCOPE telescope;
GAUSSF gaussf;
extern BLINK_FRAME defvals;
extern FITS_TABLE table;
/*int
asprintf(char **niz, const char *format, ...)

{
  *niz = myalloc(500, "asprintf", "niz");
  return sprintf(*niz, format, __VALIST[0], __VALIST[1], __VALIST[2], __VALIST[3]);
}
*/
/*****************/
/*  MYALLOC      */
/*****************/
void *
myalloc(unsigned int size, char *function, char *variable)
     
{
  void *p;
  if (size == 0) {
    fprintf(stderr, "Zero size, function %s, variable %s.\n",
           function, variable);
  }
  p = malloc(size);
  if (p == NULL) {
    fprintf(stderr, "Out of memory, function %s, variable %s.\n",
           function, variable);
    exit(1);
  }
  return(p);
}


/*      DELAY                            */
/*      Wait for  t hundreths of second */
void
delay(int t)
     
{
  fd_set set;
  struct timeval timeout;
  
  FD_ZERO (&set);
  /* Initialize the timeout data structure.  */
  timeout.tv_sec = t / 100;
  timeout.tv_usec = (t % 100) * 10000;
  select(FD_SETSIZE, &set, NULL, NULL, &timeout);
}

/*  Transform pixel value according to values in arrays gamx and gamy  */
int
correct(BLINK_FRAME *frame, int n, PIXEL x)

{
  int i;
  float *gamx = frame[n].gamx;
  float *gamy = frame[n].gamy;
  for (i = 1; i < frame[n].ngam; i++) {
    if (x <= gamx[i] && x >= gamx[i - 1]) {
      return (int) (gamy[i - 1] + (x - gamx[i - 1]) * frame[n].kg[i]);
    }
  }
  return 0;
}

/*  Draw objects which belong to the image */
void
draw_objects(BLINK_FRAME *frame, int n)

{
  int j;
  Pixmap pixmap = frame[n].pixmap;
  Display *disp = state.disp;
  GC gc = fl_state[fl_get_form_vclass(state.blinker->blinker)].gc[0];
  OBJECT *obj = frame[n].obj;
  float dx = frame[n].dx, dy = frame[n].dy;
  int x0, y0, w0, h0;
  int xmax, ymax;
  float kx, ky;
  float ta, tb, tc;
  XPoint xpoints[10];
  float ox, oy;

  if (!frame[n].objshow) return;
  xmax = frame[n].fits.x2 - frame[n].fits.x1 + 3;
  ymax = frame[n].fits.y2 - frame[n].fits.y1 + 3;
  kx = (float) (xmax - 1) / (state.blinker->blinkW->w - 1);
  ky = (float) (ymax - 1) / (state.blinker->blinkW->h - 1);
  if (kx < ky) kx = ky;
  w0 = (xmax - 1) / kx + 0.5;
  h0 = (ymax - 1) / kx + 0.5;
  x0 = (state.blinker->blinkW->w - w0) / 2 + 0.5;
  y0 = (state.blinker->blinkW->h - h0) / 2 + 0.5;
  /*  Draw additonal objects  */
  for (j = 0; j < frame[n].objnum; j++) {
    if (obj[j].x < frame[n].fits.x1 - 1 || obj[j].x > frame[n].fits.x2 + 1 ||
	obj[j].y < frame[n].fits.y1 - 1 || obj[j].y > frame[n].fits.y2 + 1) continue;
    XSetState(disp, gc, fl_get_pixel(obj[j].color), 0, GXcopy, AllPlanes);
    ox = (obj[j].x - frame[n].fits.x1 + 1 - dx) / kx;
    oy = (obj[j].y - frame[n].fits.y1 + 1 - dy) / kx;
    switch (obj[j].shape) {
    case OBJ_CIRCLE:
      XDrawArc(disp, pixmap, gc, 
	       x0 + (int) (ox - obj[j].r + 0.5), 
	       y0 + (int) (oy - obj[j].r + 0.5), 
	       (int) (2 * obj[j].r), (int) (2 * obj[j].r), 0, 23040);
      XDrawArc(disp, pixmap, gc, 
	       x0 + (int) (ox - obj[j].r - 0.5), 
	       y0 + (int) (oy - obj[j].r - 0.5), 
	       (int) (2 * obj[j].r + 2), (int) (2 * obj[j].r + 2), 0, 23040);
      break;
    case OBJ_BOX:
      XDrawRectangle(disp, pixmap, gc, 
		     x0 + (int) (ox - obj[j].r + 0.5), 
		     y0 + (int) (oy - obj[j].r + 0.5), 
		     (int) (2 * obj[j].r), (int) (2 * obj[j].r));
      XDrawRectangle(disp, pixmap, gc, 
		     x0 + (int) (ox - obj[j].r - 0.5), 
		     y0 + (int) (oy - obj[j].r - 0.5), 
		     (int) (2 * obj[j].r + 2), (int) (2 * obj[j].r + 2));
      break;
    case OBJ_DIAMOND:
      xpoints[0].x = x0 + (int) (ox + 0.5);
      xpoints[0].y = y0 + (int) (oy - obj[j].r - 0.5);
      xpoints[1].x = x0 + (int) (ox + obj[j].r + 0.5);
      xpoints[1].y = y0 + (int) (oy + 0.5);
      xpoints[2].x = xpoints[0].x;
      xpoints[2].y = y0 + (int) (oy + obj[j].r + 0.5);
      xpoints[3].x = x0 + (int) (ox - obj[j].r - 0.5);
      xpoints[3].y = xpoints[1].y;
      xpoints[4].x = xpoints[0].x;
      xpoints[4].y = xpoints[0].y;
      xpoints[5].x = xpoints[0].x;
      xpoints[5].y = xpoints[0].y - 1;
      xpoints[6].x = xpoints[1].x + 1;
      xpoints[6].y = xpoints[1].y;
      xpoints[7].x = xpoints[2].x;
      xpoints[7].y = xpoints[2].y + 1;
      xpoints[8].x = xpoints[3].x - 1;
      xpoints[8].y = xpoints[3].y;
      xpoints[9].x = xpoints[5].x;
      xpoints[9].y = xpoints[5].y;
      XDrawLines(disp, pixmap, gc, xpoints, 10, CoordModeOrigin);
      break;
      case OBJ_UPTRIANGLE:
	tb = 0.5773502 * obj[j].r;
	tc = 0.5 * tb;
	ta = 0.5 * obj[j].r;
	xpoints[0].x = x0 + (int) (ox + 0.5);
	xpoints[0].y = y0 + (int) (oy - tb - 0.5);
	xpoints[1].x = x0 + (int) (ox + ta + 0.5);
	xpoints[1].y = y0 + (int) (oy + tc + 0.5);
	xpoints[2].x = x0 + (int) (ox - ta - 0.5);
	xpoints[2].y = xpoints[1].y;
	xpoints[3].x = xpoints[0].x;
	xpoints[3].y = xpoints[0].y;
	xpoints[4].x = xpoints[0].x;
	xpoints[4].y = xpoints[0].y - 1;
	xpoints[5].x = xpoints[1].x + 1;
	xpoints[5].y = xpoints[1].y + 1;
	xpoints[6].x = xpoints[2].x - 1;
	xpoints[6].y = xpoints[5].y;
	xpoints[7].x = xpoints[4].x;
	xpoints[7].y = xpoints[4].y;
	XDrawLines(disp, pixmap, gc, xpoints, 8, CoordModeOrigin);
	break;
      case OBJ_DOWNTRIANGLE:
	tb = 0.5773502 * obj[j].r;
	tc = 0.5 * tb;
	ta = 0.5 * obj[j].r;
	xpoints[0].x = x0 + (int) (ox + 0.5);
	xpoints[0].y = y0 + (int) (oy + tb - 0.5);
	xpoints[1].x = x0 + (int) (ox + ta + 0.5);
	xpoints[1].y = y0 + (int) (oy - tb + 0.5);
	xpoints[2].x = x0 + (int) (ox - ta - 0.5);
	xpoints[2].y = xpoints[1].y;
	xpoints[3].x = xpoints[0].x;
	xpoints[3].y = xpoints[0].y;
	xpoints[4].x = xpoints[0].x;
	xpoints[4].y = xpoints[0].y + 1;
	xpoints[5].x = xpoints[1].x + 1;
	xpoints[5].y = xpoints[1].y + 1;
	xpoints[6].x = xpoints[2].x - 1;
	xpoints[6].y = xpoints[2].y - 1;
	xpoints[7].x = xpoints[4].x;
	xpoints[7].y = xpoints[4].y;
	XDrawLines(disp, pixmap, gc, xpoints, 8, CoordModeOrigin);
	break;
    }
  }
  XSetState(disp, gc, state.pixel[0], 0, GXcopy, AllPlanes);
}

/*  Determine the size of pixmap  */
int
pixmap_size(int width, int height, int depth)

{
  int count;
  int bits;
  XPixmapFormatValues *pfv;

  pfv = XListPixmapFormats(state.disp,  &count);
  for (count--; count > 0; count--) {
    if (pfv[count].depth != depth) continue;
    bits = width * pfv[count].bits_per_pixel;
    if (bits % pfv[count].scanline_pad) {
      bits -= bits % pfv[count].scanline_pad;
      bits += pfv[count].scanline_pad;
    }
    XFree(pfv);
    return (bits / 8) * height;
  }
  return 0;
}


/*  Draw the image with current "gamma" settings to the memory  */
void
set_image(BLINK_FRAME *frame, int n)

{
  float kx, ky;
  int w, h;
  int x, y, x0, y0, w0, h0, xp, yp;
  int ys, xi, yi;
  float dx, dy;
  GC gc;
  int xmax, ymax;
  int xlow, ylow, xhigh, yhigh;

  Window win = FL_ObjWin(state.blinker->blinkW);
  XImage *xim;
  XWindowAttributes xwa;
  unsigned char *data = NULL;


  Pixmap pixmap = frame[n].pixmap;
  Display *disp = state.disp;
  PIXEL *image = frame[n].fits.image;

  fl_set_oneliner_font(0, 20); 
  fl_show_oneliner("Setting the image", 100, 100);
  XFlush(state.disp);
  xmax = frame[n].fits.x2 - frame[n].fits.x1 + 3;
  ymax = frame[n].fits.y2 - frame[n].fits.y1 + 3;
  if (xmax == 0 || ymax == 0) return;
  dx = frame[n].dx + frame[n].fits.x1 - 1;
  dy = frame[n].dy + frame[n].fits.y1 - 1; 
  gc = fl_state[fl_get_form_vclass(state.blinker->blinker)].gc[0];
  w = state.blinker->blinkW->w;
  h = state.blinker->blinkW->h;

  kx = (float) (xmax - 1) / (w - 1);
  ky = (float) (ymax - 1) / (h - 1);
  if (kx < ky) kx = ky;
  w0 = (xmax - 1) / kx + 0.5;
  h0 = (ymax - 1) / kx + 0.5;
  x0 = (w - w0) / 2 + 0.5;
  y0 = (h - h0) / 2 + 0.5;
  XSetState(disp, gc, state.pixel[0], 0, GXcopy, AllPlanes);
  XFillRectangle(disp, pixmap, gc, 0, 0, w, h); 


  XGetWindowAttributes(state.disp, win, &xwa);
  data = myalloc(pixmap_size(w0, h0, xwa.depth), "set_image", "data");
  xim = XCreateImage(state.disp, xwa.visual, xwa.depth, ZPixmap, 0, data, w0, h0, 8, 0);

  /*  Draw the image  */
  xlow = 0;
  xhigh = frame[n].fits.width;
  ylow = 0;
  yhigh = frame[n].fits.height;
  yp = y0;
  for (y = 0; y < h0; y++, yp++) {
    yi = (int) (kx * y + dy + 0.5);
    if (yi >= ylow && yi < yhigh) {
      ys = yi * frame[n].fits.width;
      xp = x0;
      for (x = 0; x < w0; x++, xp++) {
	xi = (int) (kx * x + dx + 0.5);
	if (xi >= xlow && xi < xhigh) {
	  XPutPixel(xim, x, y, state.pixel[correct(frame, n, image[xi + ys])]);
	}
      }
    }
  }
  XPutImage(state.disp, pixmap, gc, xim, 0, 0, x0, y0, w0, h0);

  XDestroyImage(xim);
  draw_objects(frame, n);
  fl_hide_oneliner();
  fl_winset(FL_ObjWin(state.blinker->blinkW));
}


/*  Erase the currently selected filename from the list and release all
    the memory occupied by the image  */
void
erase_image(int n)

{
  FL_OBJECT *ch1, *ch2;
  int i, m;

  if (n >= state.frame_used) return;
  ch1 = state.gui->firstnameW;
  ch2 = state.gui->secondnameW;
  m = n + 1;
  state.freeze = 0;
  fl_hide_object(state.blinker->lockedW);
  fl_hide_object(state.blinker->upW);
  fl_hide_object(state.blinker->downW);
  fl_hide_object(state.blinker->rightW);
  fl_hide_object(state.blinker->leftW);
  fl_hide_object(state.blinker->circleW);
  /*  Erase the filename from both choices  */
  if (fl_get_choice_maxitems(ch1) > 0) {
    fl_delete_choice(ch1, m);
    fl_delete_choice(ch2, m);
    /*  Remove the image from the memory  */
    remove_frame(frame, n);
    if (state.blinknum == n) state.blinknum--;
    if (state.frame_used < 2 && state.blink) {
      /*  Stop blinking  */
      state.blink = 0;
      /* Switch off the Blink button */
      fl_set_button(state.blinker->playW, 0);
    }
    if (fl_get_choice_maxitems(ch1) > 0) {
      state.control[0] = fl_get_choice(ch1) - 1;
      state.control[1] = fl_get_choice(ch2) - 1;
      show_frame_info(frame, state.control[0], 0);
      show_frame_info(frame, state.control[1], 1);
    }
    else {
      /*  The last image was erased */
      /*  Erase histogram, grey level correction curve and images  */
      fl_set_xyplot_data(state.gui->firstgreyW, NULL, NULL, 0, "", "", "");
      fl_set_xyplot_data(state.gui->secondgreyW, NULL, NULL, 0, "", "", "");
      fl_set_xyplot_data(state.gui->firstgammaW, NULL, NULL, 0, "", "", "");
      fl_set_xyplot_data(state.gui->secondgammaW, NULL, NULL, 0, "", "", "");
      fl_winset(FL_ObjWin(state.blinker->blinkW));
      fl_set_form_title(state.blinker->blinker, "(no image)");
      fl_rectf(state.blinker->blinkW->x, state.blinker->blinkW->y,
	       state.blinker->blinkW->w, state.blinker->blinkW->h, 
	       state.blinker->blinkW->col1);
      fl_rectf(state.blinker->zoom1W->x, state.blinker->zoom1W->y,
	       state.blinker->zoom1W->w, state.blinker->zoom1W->h, 
	       state.blinker->zoom1W->col1);
      fl_redraw_object(state.blinker->zoom1W);
      fl_rectf(state.blinker->zoom2W->x, state.blinker->zoom2W->y,
	       state.blinker->zoom2W->w, state.blinker->zoom2W->h, 
	       state.blinker->zoom2W->col1);
      fl_redraw_object(state.blinker->zoom2W);
      /*  Disable Astrometry options  */
      for (i = 1; i < EMPTY_ITEM1; i++) {
	fl_set_menu_item_mode(state.blinker->astrometryW, i, FL_PUP_GREY);
      }
      fl_set_menu_item_mode(state.blinker->astrometryW, OPTIONS, FL_PUP_GREY);
      /*  If the Set parameters window is open, close it  */
      if (state.parameters && fl_form_is_visible(state.parameters->Parameters)) {
	fl_hide_form(state.parameters->Parameters);
      }
    }
  }
}

/*  Calculate values for linear grey level correction  */
void
set_lin(BLINK_FRAME *frame, int n)

{
  float dg, dx;
  int i;

  dx = frame[n].high - frame[n].low;
  dg = dx / (frame[n].ngam - 1.0);
  /*  Set tables for brightness correction  */
  for (i = 0; i < frame[n].ngam; i++) {
    frame[n].gamx[i] = frame[n].low + i * dg;
    frame[n].gamy[i] = (state.grey_max * i) / (frame[n].ngam - 1.0);
    if (i > 0) {
      frame[n].kg[i] = (frame[n].gamy[i] - frame[n].gamy[i - 1]) / 
	(frame[n].gamx[i] - frame[n].gamx[i - 1]);
    }
  }
}


int
load_frame(BLINK_FRAME *frame, int n)

{
  int xmax, ymax;
  PIXEL *image;
  int i, j, low, high, g;
  float dg;
  int wh;
  char *errors[ERROR_MAX];
  int errnum;

  if ((errnum = read_fits(frame[n].name, &image, &xmax, &ymax, 
			  &(frame[n].fitsfile), errors)) > 0) {
    if (state.interactive) {
      show_errors(errnum, errors);
      return 1;
    }
    else {
      for (i = 0; i < errnum; i++) {
	fprintf(stderr, errors[i]);
      }
      exit(1);
    }
  }
  frame[n].fits.name = frame[n].name;
  frame[n].imagelist.s = NULL;
  frame[n].imagelist.n = 0;
  frame[n].cataloglist.s = NULL;
  frame[n].cataloglist.n = 0;
  frame[n].detect = 0;
  frame[n].matched = frame[n].fitsfile.matched;
  frame[n].match.magfac = 0;
  if (frame[n].fitsfile.filter[0] != '*') 
    strcpy(frame[n].match.filter, frame[n].fitsfile.filter);
  else 
    strcpy(frame[n].match.filter, defvals.match.filter);
  if (frame[n].fitsfile.is_gain) 
    frame[n].fits.gain = frame[n].fitsfile.gain;
  else 
    frame[n].fits.gain = DEFAULT_GAIN;
  if (frame[n].fitsfile.is_rdnoise) 
    frame[n].fits.rdnoise = frame[n].fitsfile.rdnoise;
  else 
    frame[n].fits.rdnoise = DEFAULT_RDNOISE;
  frame[n].map.minmag = DEFAULT_MINMAG;
  frame[n].map.maxmag = DEFAULT_MAXMAG;
  frame[n].map.extarea = MAP_EXTAREA;
  /*  If WCS coordinates are present, calculate coordinates of the image center */
  /*  Enable the Set parameters option in the astrometry menu  */
  if (state.interactive) {
    fl_set_menu_item_mode(state.blinker->astrometryW, EDIT_PARAMETERS, FL_PUP_NONE);
    fl_set_menu_item_mode(state.blinker->astrometryW, BLUR_IMAGE, FL_PUP_NONE);
    fl_set_menu_item_mode(state.blinker->astrometryW, LOAD_STAR_LIST, FL_PUP_NONE);
  }
  if (frame[n].fitsfile.wcs) {
    int status = 0;
    fits_pix_to_world(0.5 * xmax, 0.5 * ymax,
	     frame[n].fitsfile.crval1, frame[n].fitsfile.crval2,
	     frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2,
	     frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2,
	     frame[n].fitsfile.crot,
	     frame[n].fitsfile.ctype,
	     &(frame[n].ra), &(frame[n].dec), &status);
    if (frame[n].ra >= 360.0) {
      frame[n].ra -= 360.0;
    }
    frame[n].fi = frame[n].fitsfile.crot;
    frame[n].xsize = fabs(frame[n].fitsfile.cdelt1 * 3600.0);
    frame[n].ysize = fabs(frame[n].fitsfile.cdelt2 * 3600.0);
  }
  else {
    frame[n].ra = 0.0;
    frame[n].dec = 0.0;
    frame[n].fi = 0.0;
    frame[n].xsize = 1.92;
    frame[n].ysize = 1.92;
  }
  if (state.interactive) {
    fl_set_menu_item_mode(state.blinker->astrometryW, DETECT_STARS, FL_PUP_NONE);
    fl_set_menu_item_mode(state.blinker->astrometryW, OPTIONS, FL_PUP_NONE);
  }
  /*  If possible, calculate time and date of mid-exposure  */
  calculate_midexposure(&frame[n].fitsfile, &frame[n].year, &frame[n].month,
			&frame[n].day, &frame[n].hour, 
			&frame[n].minute, &frame[n].second);
  /*  Also calculate the Julian day  */
  frame[n].fitsfile.jd = julian_day(frame[n].year, frame[n].month, frame[n].day + 
				(frame[n].hour + 
				 (frame[n].minute + frame[n].second / 60.0) 
				 / 60.0) / 24.0);
  /*  Set default options for star matching  */
  set_match_defaults(frame, n);
  set_image_parameters(&frame[n].fits, xmax, ymax, image);
  /* awx = state.blinker->zoom1W.x / state.zoom_width;
     awy = state.blinker->zoom1W.y / state.zoom_width;
     frame[n].astrwin.image = (PIXEL *) myalloc(sizeof(PIXEL) * awx * awy, 
     "load_frame", "astrwin");
     set_image_parameters(&frame[n].astrwin, awx, awy,
     state.blinker->zoom1W.y / state.zoom_width, );  */
  /*  Translation and rotation of the image  */
  frame[n].dx = frame[n].dy = frame[n].alfa = 0.0;
  if (state.interactive) {
    /*  Create Pixmap for image  */
    frame[n].pixmap = 
      XCreatePixmap(state.disp, 
		    FL_ObjWin(state.blinker->blinkW), 
		    state.bw, state.bh, 
		    fl_state[fl_get_form_vclass(state.blinker->blinker)].depth);

    /*  Find the lowest and the highest grey level  */
    low = 65536;
    high = -65536;
    for (i = 0; i < xmax * ymax; i++) {
      g = image[i];
      if (low > g) low = g;
      if (high < g) high = g;
    }
    frame[n].xmin = frame[n].low = low;
    frame[n].xmax = frame[n].high = high;
    /*  Set number of points for grey level correction  */
    frame[n].ngam = NGAMMA;
    frame[n].gamx = myalloc(frame[n].ngam * sizeof(float), 
			    "load_frame", "frame[n].gamx");
    frame[n].gamy = myalloc(frame[n].ngam * sizeof(float), 
			    "load_frame", "frame[n].gamy");
    frame[n].kg = myalloc(frame[n].ngam * sizeof(float), 
			  "load_frame", "frame[n].kg");
    /*  Width of the histogram box  */
    frame[n].nhisto = wh = HISTOGRAM_WIDTH;
    /* reserve space for histogram  */
    frame[n].histx = myalloc(wh * sizeof(float), 
			     "load_frame", "frame[n].histx");
    frame[n].histy = myalloc(wh * sizeof(float), 
			     "load_frame", "frame[n].histy");
    for (i = 0; i < wh; i++) {
      frame[n].histx[i] = i / (double) (wh - 1) * (high - low);
      frame[n].histy[i] = 0;
    }
    /*  Calculate histogram  */
    dg = (high - low) / (wh - 1.0);
    for (i = 0; i < xmax * ymax; i++) {
      j = (image[i] - low) / dg;
      frame[n].histy[j]++;
    }
    for (i = 0; i < wh; i++) {
      frame[n].histy[i] = log(frame[n].histy[i] + 1);
    }
    /*  Number of markers  */
    frame[n].objnum = 0;
    frame[n].objshow = 1;
    /*  Make automatic grey level correction  */
    autolevels(n);
  }
  return 0;
}

/*  Remove all the data belonging to frame n from the memory  */
int
remove_frame(BLINK_FRAME *frame, int n)

{
  int i;

  if (state.interactive) {
    free(frame[n].histy);
    free(frame[n].histx);
    free(frame[n].kg);
    free(frame[n].gamy);
    free(frame[n].gamx);
    /*  Free the Pixmap  */
    XFreePixmap(state.disp, frame[n].pixmap);
  }
  /* Free space occupied by the header  */
  for (i = frame[n].fitsfile.nkeys - 1; i >= 0; i--) {
    if (frame[n].fitsfile.key[i]) free(frame[n].fitsfile.key[i]);
  }
  free(frame[n].fitsfile.key);

  /*  Free space for the image  */
  free(frame[n].fits.image);
  if (frame[n].imagelist.s) free(frame[n].imagelist.s);
  if (frame[n].cataloglist.s) free(frame[n].cataloglist.s);
  if (frame[n].name) free(frame[n].name);
  remove_all_markers(frame, n);
  /* Move data about other images so that they fill the hole which appeared */
  (state.frame_used)--;
  for (i = n; i < state.frame_used; i++) frame[i] = frame[i + 1];
  /*  Set pointers to NULL */
  frame[i].gamx = NULL;
  frame[i].gamy = NULL;
  frame[i].kg = NULL;
  frame[i].histx = NULL;
  frame[i].histy = NULL;
  frame[i].name = NULL;
  frame[i].fits.image = NULL;
  frame[i].cataloglist.s = NULL;
  frame[i].imagelist.s = NULL;
  return 0;
}



int
load_file(char *pname)

{
  char line[80], *line1, *p, name[80], *fname;
  FILE *fp;
  int i, i1, j, n;
  float dx, dy;
  float x, y, r;
  char color[81], shape[81], s[81];

  if ((fp = fopen(pname, "r")) == NULL) {
    fprintf(stderr, "Can not open file %s\n", pname);
    exit(1);
  }
  i = 0; /*  Number of currently loaded images  */
  i1 = 0;
  while(!feof(fp)) {
    n = 0;
    /*  Read a line from file  */
    fgets(line, 80, fp);
    if (feof(fp)) break;
    line1 = (char *) strdup(line);
    do {
      p = strtok(line1, " \t");
      if (p != NULL) n++;
      line1 = NULL;
    } while (p != NULL);
    /*  n contains a number of items in the current line  */
    switch (n) {
    case 3:
      /*    case 7: */
      if (n == 3) {
	  /*  Three items: get file name and offsets in x and y direction */
	  /****************************************************************/
	  sscanf(line, "%s %f %f", name, &dx, &dy);
	  frame[i].a = 0.0;
	  frame[i].b = 0.0;
	  frame[i].d = 0.0;
	  frame[i].e = 0.0;
      }
      else {
	/*  Seven items: */
	  /****************************************************************/
	/*	  sscanf(line, "%s %f %f %f %f %f %f %f", name, &dx, &dy,
		 &frame[i].a, &frame[i].b, &frame[i].d, &frame[i].e);  */
      }

      frame[i].name = (char *)strdup(name);
      next_image();
      /*  Load the image into the structure */
      if (load_frame(frame, i)) {
	(state.frame_used)--;
	break;
      }

      frame[i].dx = dx;
      frame[i].dy = dy;
      /*frame[i].fits.x1 = max(1 + dx, 1);
      frame[i].fits.x2 = min(frame[i].fits.x2 - 2 + dx, frame[i].fits.x2);
      frame[i].fits.y1 = max(1 + dy, 1);
      frame[i].fits.y2 = min(frame[i].fits.y2 - 2 + dy, frame[i].fits.y2); */

      /*  Reserve space for OBJECT_NUM markers and assign OBJECT_NUM  */
      /*  to frame[i].objmax  */
      reserve_marker_space(frame, i, OBJECT_NUM);
      frame[i].imagelist.s = (STAR *) myalloc(sizeof(STAR) * frame[i].objmax, 
					      "load_file", "imagelist.s");
      set_image(frame, i);
      /*  Add image name to choices  */
      fname = strrchr(name, '/');
      if (!fname) fname = name;
      else fname++;
      fl_addto_choice(state.gui->firstnameW, fname);
      fl_addto_choice(state.gui->secondnameW, fname);
      if (i == 0 || i == 1) {
	state.control[state.control_num] = i;
	/*  Write the file name into the input field,  */
	/*  draw histogram, grey level correction curve, translation and rotation  */
	show_frame_info(frame, i, 0);
	if (state.frame_used == 1) {
	  state.control[1] = 0;
	  show_frame_info(frame, i, 1);
	}
      }
      i++;
      i1 = i - 1;
      break;
    case 4:
    case 5:
    case 6:
      /*  Info about an additional object  */
      if (state.frame_used == 0) {
	fprintf(stderr, "No image loaded yet: ignoring line %s", line);
	break;
      }
      j = frame[i1].objnum;
      color[0] = '\0';
      shape[0] = '\0';
      if (n == 4) {
	if (sscanf(line, "%f %f %f %s", &x, &y, &r, s) != 4) break;
      }
      else if (n == 5) {
	if (sscanf(line, "%f %f %f %s %s", &x, &y, &r, s, color) != 5) break;
      }
      else if (n == 6) {
	if (sscanf(line, "%f %f %f %s %s %s", &x, &y, &r, s, color, shape) != 6) break;
      }
      append_marker(frame, i1, x, y, r, s, 
		    find_marker_color(color), find_marker_shape(shape));
      frame[i1].imagelist.s[j].x = x;
      frame[i1].imagelist.s[j].y = y;
      frame[i1].imagelist.s[j].mag = r;
      frame[i1].imagelist.n = j + 1;
      frame[i1].imagelist.s[j].catnum = 0;
      frame[i1].imagelist.s[j].catalog = '0';
      break;
    }
  }
  fclose(fp);
  /*  Draw all objects  */
  for (i1 = 0; i1 < i; i1++) 
    draw_objects(frame, i1);

  if (state.frame_used > 0) {
    show_frame_info(frame, state.control[state.control_num], state.control_num);
  }
  return 0;
}


int
main(int argc, char *argv[])

{
  return main0(argc, argv);
}


int
main0(int argc, char *argv[])

{
  int i;
  char *np;

  /*  reserve space for FRAME_MAX descriptions of images  */
  frame = (BLINK_FRAME *) myalloc(sizeof(BLINK_FRAME) * FRAME_MAX,
				  "main", "frame");
  state.frame_max = FRAME_MAX;
  for (i = 0; i < state.frame_max; i++) {
    frame[i].gamx = NULL;
    frame[i].gamy = NULL;
    frame[i].kg = NULL;
    frame[i].histx = NULL;
    frame[i].histy = NULL;
    frame[i].name = NULL;
    frame[i].fits.image = NULL;
  }
  /*===============================================*/
  /*  Set defaults for the program state variable  */
  /*===============================================*/
  state.browser[0] = NULL;
  state.browser[1] = NULL;
  state.b[0] = 0;
  state.b[1] = 0; 
  state.error = NULL; 
  state.readme = NULL;
  state.parameters = NULL;
  state.report = NULL;
  state.options = NULL;
  state.catalogs = NULL;
  state.mail = NULL;
  state.newwcs = NULL;
  state.settings = NULL;
  state.set_catalogs = NULL;
  state.set_observatory = NULL;
  state.set_display = NULL;
  state.set_mail = NULL;
  state.set_commands = NULL;
  state.set_ccd = NULL;
  state.set_count = NULL;
  state.set_match = NULL;
  state.set_asteroid = NULL;
  state.showcomet = NULL;
  state.telescope = NULL;
  state.message = NULL;
  state.matchedobjects = NULL;
  state.blinkwidth = 1000;
  state.blinkheight = 910;
  state.frame_used = 0;
  state.control_num = 0;
  state.zoom_width = 4;
  state.freeze = 0;
  state.blink = 0;
  state.blinknum = 0;
  state.delay = DEFAULT_DELAY;
  state.autolevel_high = DEFAULT_AUTOLEVEL_HIGH;
  state.logfile = NULL;
  state.logfp = stdout;
  state.interactive = 1;
  state.verbose = 0;
  state.script = 0;
  state.doastr = 0;
  state.color = 0;
  state.inverse = 0;
  state.wwwbrowser = strdup(DEFAULT_BROWSER);
  state.helpfile = strdup(DEFAULT_MANUAL_DIR);
  state.asteroid_server = NULL;
  state.asteroid_id = NULL;
  state.asteroid_maxmag = 30;
  state.starlistformat = FITS_STARLIST;
  state.dss_big = DSS_BIG;
  state.dss_server = strdup(DSS_SERVER);
  strcpy(state.desig_prefix, DEFAULT_DESIG_PREFIX);
  state.desig = DEFAULT_DESIG_NUMBER;
  /*======================================================*/
  /*  Set default values for star counting and matching   */
  /*======================================================*/
  set_defvals();
  /*====================================*/
  /*  Mark count_options variables      */
  /*====================================*/
  count_options.maxsize = -999;
  count_options.sg_num = -999;
  count_options.fitmeasure = -999;
  count_options.border = -999;
  count_options.satlevel = -999;
  count_options.minbright = -999;
  count_options.minstar = -999;
  count_options.inner = -999;
  count_options.outer = -999;
  count_options.aperture = -999;
  count_options.resolve = -999;
  count_options.gain = -999;
  count_options.rdnoise = -999;
  /*====================================*/
  /*  Mark match_options variables      */
  /*====================================*/
  match_options.firstbright = -999;
  match_options.nstar = -999;
  match_options.min = -999;
  match_options.mindist = -999;
  match_options.poserrmax = -999;
  match_options.magerrmax = -999;
  match_options.maxres = -999;
  /*=============================*/
  /*  Define supported catalogs  */
  /*=============================*/
  define_catalogs();
  /*=================================*/
  /*  Define observatory parameters  */
  /*=================================*/
  strcpy(observatory.code, "");
  observatory.code[3] = '\0';
  observatory.observer[0] = '\0';
  observatory.telescope[0] = '\0';
  observatory.instrument[0] = '\0';

  /*=================================*/
  /*  Telescope control defaults     */
  /*=================================*/
  telescope.ra = 0;
  telescope.dec = 0;
  telescope.exptime = 1;
  telescope.dark = 0;
  telescope.temp = 0;
  telescope.filename = NULL;

  /*========================*/
  /*  Table structure       */
  /*========================*/
  table.name = NULL;
  /*  Mail structure  */
  if ((mail.to = (char *) malloc(1)) == NULL) {
    fprintf(stderr, "Out of memory, mail.to string.\n");
    exit(1);
  }
  mail.to[0] = '\0';
  if ((mail.cc = (char *) malloc(1)) == NULL) {
    fprintf(stderr, "Out of memory, mail.cc string.\n");
    exit(1);
  }
  mail.cc[0] = '\0';
  if ((mail.title = (char *) malloc(1)) == NULL) {
    fprintf(stderr, "Out of memory, mail.title string.\n");
    exit(1);
  }
  mail.title[0] = '\0';
  if ((mail.file = (char *) malloc(1)) == NULL) {
    fprintf(stderr, "Out of memory, mail.file string.\n");
    exit(1);
  }
  mail.file[0] = '\0';

  /*  Find out name of the program  */
  /*================================*/
  np = strrchr(argv[0], '/');  /* check first if there is any path  */
  if (np == NULL) np = argv[0];  /*  No path, just name  */
  else np++;
  if (strcmp(np, "starcount") == 0) {
    state.interactive = 0;
    starcount(argc, argv);
  } 
  else if (strcmp(np, "catalog") == 0) {
    state.interactive = 0;
    make_catalog(argc, argv);
  } else {
    tgui(argc, argv);
  }
  return 0;
}
