/*  blinkgui.c for fitsblink */
/*  Jure Skvarc, May 1998  */
/* User interface */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include <forms.h>
#include <fitsio.h>

#include "formblink.h"
#include "functs.h"
#include "consts.h"

extern STATE state;
extern BLINK_FRAME *frame;
extern OBSERVATORY observatory;

extern char *optarg;
extern int optind, opterr, optopt;
/*  Draw given image to the screen  */
void
draw_image(BLINK_FRAME *frame, int n)

{
  GC gc = fl_state[fl_get_form_vclass(state.blinker->blinker)].gc[0];
  Window win = FL_ObjWin(state.blinker->blinkW);

  XSetFunction(state.disp, gc, GXcopy);
  XCopyArea(state.disp, frame[n].pixmap, win, gc, 0, 0, 
	    state.blinker->blinkW->w, state.blinker->blinkW->h,
	    state.blinker->blinkW->x, state.blinker->blinkW->y);
  state.doastr = 0;
  fl_set_form_title(state.blinker->blinker, frame[n].name);
  fl_set_button(state.blinker->markersW, frame[n].objshow);
}


/*  Draw profiles in x and y directions around point x, y */
void
draw_profiles(BLINK_FRAME *frame, int n, int x0, int y0)

{
  int w, h, w2, h2;
  int i, i1;
  static int m = 0;
  static float *x = NULL, *y = NULL;
  float ymax, ymin;
  /*  Width and height of the zoom window  */
  w = state.blinker->zoom1W->w / state.zoom_width;
  h = state.blinker->zoom1W->h / state.zoom_width;
  /*  Reserve space for data points, n is number of points  */
  if (w > m || h > m) {
    if (w > h) m = w;
    else m = h;
    if (x) free(x);
    if (y) free(y);
    x = (float *) myalloc(sizeof(float) * m, "draw_profiles", "x");
    y = (float *) myalloc(sizeof(float) * m, "draw_profiles", "y");
  }
  w2 = w / 2;
  h2 = h / 2;
  ymax = 0;
  ymin = 65536;
  for (i = 0; i < w; i++) {
    x[i] = i + x0 - w2;
    y[i] = video_point(&frame[n].fits, x[i], y0);
    if (y[i] > ymax) ymax = y[i];
    if (y[i] < ymin) ymin = y[i];
  }
  fl_set_xyplot_ybounds(state.blinker->xplotW, ymin, ymax);
  fl_set_xyplot_data(state.blinker->xplotW, x, y, w, "", "", "");

  ymax = 0;
  ymin = 65536;
  for (i = 0; i < h; i++) {
    i1 = h - 1 - i;
    y[i] = i + y0 - h2;
    x[i1] = video_point(&frame[n].fits, x0, y[i]);
    if (x[i1] > ymax) ymax = x[i1];
    if (x[i1] < ymin) ymin = x[i1];
  }
  fl_set_xyplot_xbounds(state.blinker->yplotW, ymin, ymax);
  fl_set_xyplot_data(state.blinker->yplotW, x, y, h, "", "", "");
}


/*  Draw a magnified image around the x, y position to a zoom window */
void
draw_zoom(BLINK_FRAME *frame, int n, FL_OBJECT *o, int x, int y)

{
  int xmin, xmax, ymin, ymax;
  int w, h;
  int xq, yq, xt, yt;
  int z;
  int i, j;
  PIXEL v;
  FITS_IMAGE *im;
  Display *disp;

  GC gc = fl_state[fl_get_form_vclass(state.blinker->blinker)].gc[0];
  Window win = FL_ObjWin(o);
  disp = state.disp;

  im = &(frame[n].fits);
  if (x >= im->x1 - 1 && x <= im->x2 + 1 &&
      y >= im->y1 - 1 && y <= im->y2 + 1) {  /*  Draw only if the pointer is inside image */
    
    /*  Determine number of pixels in x and y direction  */
    z = state.zoom_width;
    w = o->w / z;
    h = o->h / z;
    /*  Limiting coordinates of the area shown  */
    xmin = max(0, x - w);
    xmax = min(x + w, im->width - 1);
    ymin = max(0, y - h);
    ymax = min(y + h, im->height - 1);
    /*  Do the drawing  */
    yq = o->y;
    yt = y - h / 2;
    for (j = 0; j < h; j++) {
      xq = o->x;
      xt = x - w / 2; 
      for (i = 0; i < w; i++) {
	v = video_point(im, xt++, yt);
	XSetForeground(disp, gc, state.pixel[correct(frame, n, v)]);
	XFillRectangle(disp, win, gc, xq, yq, z, z);
	xq += z;
      }
      yq += z;
      yt++;
    }
    /*  Draw a cross  */
    xt = o->x + w / 2 * z;
    yt = o->y + h / 2 * z;
    w = o->w / 4;
    h = o->h / 4;
    fl_rectf(xt, yt - h, z, h, FL_RED);
    fl_rectf(xt, yt + z, z, h, FL_RED);
    fl_rectf(xt - h, yt, h, z, FL_RED);
    fl_rectf(xt + z, yt, h, z, FL_RED);
  }
}


/*  Increment image counter and, if necessary, reserve */
/*  space for more images  */
void
next_image(void)

{

  (state.frame_used)++;
  if (state.frame_used == state.frame_max) {
    int i;
    state.frame_max += FRAME_MAX;
    if ((frame = (BLINK_FRAME *) 
	 realloc(frame, sizeof(BLINK_FRAME) * state.frame_max)) == NULL) {
      fprintf(stderr, "Out of memory, function next_image, variable frame\n");
      exit(1);
    }
    for (i = state.frame_used; i < state.frame_max; i++) {
      frame[i].gamx = NULL;
      frame[i].gamy = NULL;
      frame[i].kg = NULL;
      frame[i].histx = NULL;
      frame[i].histy = NULL;
      frame[i].name = NULL;
      frame[i].fits.image = NULL;
    }
  }
}


void
add_to_choices(char *name, int n, int val)

{
  char *fname;

  fname = strrchr(name, '/');
  if (!fname) fname = name;
  else fname++;
  fl_addto_choice(state.gui->firstnameW, fname);
  fl_addto_choice(state.gui->secondnameW, fname);
  /*  Write the file name into the input field,  */
  /*  draw histogram, grey level correction curve, translation and rotation  */
  state.control[val] = n;
  show_frame_info(frame, n, val);
  if (state.frame_used == 1) {
    show_frame_info(frame, n, 1 - val);
  }
}

/*  Load the image with name "name" */
/*  This function takes care of allocation of additional memory if  */
/*  necessary and of initialization of all variables belonging to one */
/*  image.  It is called by other functions which load images, including */
/*  from scripting programs.  It calls function load_frame */
/*  Returns index of the image or -1 on failure */
int
load_image(char *name, int val) 

{
  int n = state.frame_used;

  if (val > 1) return -1;
  if (!state.interactive) {
    val = 0;
  }
  frame[n].name = strdup(name);
  next_image();
  if (!load_frame(frame, n)) {
    set_image(frame, n);
    reserve_marker_space(frame, n, OBJECT_NUM);
    /*  Add image name to choices  */
    add_to_choices(name, n, val);
    return state.frame_used - 1;
  }
  else {
    state.frame_used--;
    return -1;
  }
}



/**************************************/
/*  Callbacks for the blinking window */
/**************************************/
void
do_blink(void)

{
  float d;

  draw_image(frame, state.blinknum);
  show_pan(frame, state.blinknum);
  d = fl_get_counter_value(state.blinker->deltatW);
  delay((int) (100.0 * d));
  state.blinknum++;
  if (state.blinknum == state.frame_used) state.blinknum = 0;
}

/*  Set the blinking on (b!=0) or off (b==0) */
/*  Return -1 if blinking could not start  */
int
blink(int b)

{
  if (state.frame_used > 1) {
    b = (b) ? 1 : 0;
    state.blink = b;
    state.blinknum = 0;
    fl_set_button(state.blinker->playW, b);
    return 0;
  }
  return -1;
}

/*  Set delay time between two images  */
int
set_delay(float d)

{
  if (d <= 0) d = 0.1;
  else if (d > 10) d = 10;
  fl_set_counter_value(state.blinker->deltatW, d);
  return 0;
}



/*  Should remain empty  */
void
deltatC(FL_OBJECT *obj, long val)

{
}

void
blinkC(FL_OBJECT *obj, long val)

{
}

/*  Move the right image one pixel up */
void
upC(FL_OBJECT *obj, long val)

{
  if (state.freeze) {
    /*  Get index of the right image  */
    int n = state.control[1];
    /*  Shift it up for one pixel  */
    frame[n].dy++;
    state.yf++;
    draw_zoom(frame, n, state.blinker->zoom2W, state.xf, state.yf);
    show_right_offsets(frame, n);
    if (state.control[0] == state.control[1]) {
      show_left_offsets(frame, n);
      draw_zoom(frame, n, state.blinker->zoom1W, state.xf, state.yf);
    }
  }
}

/*  Move the right image one pixel down */
void
downC(FL_OBJECT *obj, long val)

{
  if (state.freeze) {
    /*  Get index of the right image  */
    int n = state.control[1];
    /*  Shift it down for one pixel  */
    frame[n].dy--;
    state.yf--;
    draw_zoom(frame, n, state.blinker->zoom2W, state.xf, state.yf);
    show_right_offsets(frame, n);
    if (state.control[0] == state.control[1]) {
      show_left_offsets(frame, n);
      draw_zoom(frame, n, state.blinker->zoom1W, state.xf, state.yf);
    }
  }
}

/*  Move the right image one pixel right */
void
rightC(FL_OBJECT *obj, long val)

{
  if (state.freeze) {
    /*  Get index of the right image  */
    int n = state.control[1];
    /*  Shift it right for one pixel  */
    frame[n].dx--;
    state.xf--;
    draw_zoom(frame, n, state.blinker->zoom2W, state.xf, state.yf);
    show_right_offsets(frame, n);
    if (state.control[0] == state.control[1]) {
      show_left_offsets(frame, n);
      draw_zoom(frame, n, state.blinker->zoom1W, state.xf, state.yf);
    }
  }
}

/*  Move the right image one pixel left */
void
leftC(FL_OBJECT *obj, long val)

{
  if (state.freeze) {
    /*  Get index of the right image  */
    int n = state.control[1];
    /*  Shift it left for one pixel  */
    frame[n].dx++;
    state.xf++;
    draw_zoom(frame, n, state.blinker->zoom2W, state.xf, state.yf);
    show_right_offsets(frame, n);
    if (state.control[0] == state.control[1]) {
      show_left_offsets(frame, n);
      draw_zoom(frame, n, state.blinker->zoom1W, state.xf, state.yf);
    }
  }
}

/*  Switch off image shifting mode  */
void
circleC(FL_OBJECT *obj, long val)

{
  state.freeze = 0;
  /*  Hide cursors and LOCKED ! sign  */
  fl_hide_object(state.blinker->lockedW);
  fl_hide_object(state.blinker->upW);
  fl_hide_object(state.blinker->downW);
  fl_hide_object(state.blinker->rightW);
  fl_hide_object(state.blinker->leftW);
  fl_hide_object(state.blinker->circleW);
  set_image(frame, state.control[1]);
  if (!fl_get_button(state.blinker->playW) &&
      state.control_num == 1) {
    draw_image(frame, state.control[1]);
  }
}

/*=================================*/
/*  Open telescope control window  */
/*=================================*/
void
telescope_controlC(FL_OBJECT *obj, long val)

{
  show_telescope_control();
}


void
show_pan(BLINK_FRAME *frame, int n)

{
  int x, y, w, h;
  FL_OBJECT *d = state.blinker->panW;
  Window win;

  win = FL_ObjWin(d);
  fl_winset(win);
  fl_drw_box(FL_DOWN_BOX, d->x, d->y, d->w, d->h, FL_GRAY35, 2);
  if (state.frame_used > 0) {
    /*  Draw a box inside the object  */
    w = (frame[n].fits.x2 - frame[n].fits.x1 + 3) / (float) frame[n].fits.width * (d->w - 4);
    if (w < 5) w = 5;
    h = (frame[n].fits.y2 - frame[n].fits.y1 + 3) / (float) frame[n].fits.height * (d->h - 4);
    if (h < 5) h = 5;
    if (w == d->w) w -= 2;
    if (h == d->h) h -= 2;
    x = d->x + 2 + frame[n].fits.x1 / (float) frame[n].fits.width * (d->w - 4);
    y = d->y + 2 + frame[n].fits.y1 / (float) frame[n].fits.height * (d->h - 4);
    fl_drw_box(FL_UP_BOX, x, y, w, h, FL_GRAY63, 2);
  }
}


#if SWIG
/*====================================*/
/*  Set the zoom level for i-th image */
/*====================================*/
int
resize_image(int i, int x, int y, int w, int h)

{
  int n = state.control[state.control_num];
  if (i < state.frame_used && x >= 0 && y >= 0 && w >= 0 && h >= 0 && 
      x + w < frame[i].fits.width && y + h < frame[i].fits.height) {
    frame[i].fits.x1 = x + 1;
    frame[i].fits.y1 = y + 1;
    frame[i].fits.x2 = x + w;
    frame[i].fits.y2 = y + h;
    set_image(frame, i);
    if (state.frame_used > 0) {
      draw_image(frame, n);
      show_pan(frame, n);
    }
    return 0;
  }
  else return -1;
}
#endif

int
full_image(int i)

{
  if (i < state.frame_used) {
    frame[i].fits.x1 = 1;
    frame[i].fits.y1 = 1;
    frame[i].fits.x2 = frame[i].fits.width - 2;
    frame[i].fits.y2 = frame[i].fits.height - 2;
    set_image(frame, i);
    return 0;
  }
  else return -1;
}

/*===============================================*/
/*  Change zoom of all images to see full images */
/*===============================================*/
void
fullC(FL_OBJECT *obj, long val)

{
  int n = state.control[state.control_num];
  int i;

  for (i = 0; i < state.frame_used; i++) {
    full_image(i);
  }
  if (state.frame_used > 0) {
    draw_image(frame, n);
    show_pan(frame, n);
  }
}



/*   Handler for the pan window  */
int
freeobj_panW_handle(FL_OBJECT *d, int event, FL_Coord mx, FL_Coord my, int key, void *xev)
     
{
  static int move = 0;
  int n = state.control[state.control_num];
  int i;
  static int x, y;
  static int xr, yr, wr, hr, mxr, myr;
  
  switch(event) {
  case FL_STEP:
    break;
  case FL_DRAW:
    show_pan(frame, n);
    break;
  case FL_MOTION:
    break;
  case FL_DBLCLICK:
    break;
  case FL_PUSH:
    if (key == 1) {  
      wr = (frame[n].fits.x2 - frame[n].fits.x1 + 3) / (float) frame[n].fits.width * (d->w - 4);
      if (wr < 5) wr = 5;
      hr = (frame[n].fits.y2 - frame[n].fits.y1 + 3) / (float) frame[n].fits.height * (d->h - 4);
      if (hr < 5) hr = 5;
      xr = d->x + 2 + frame[n].fits.x1 / (float) frame[n].fits.width * (d->w - 4);
      yr = d->y + 2 + frame[n].fits.y1 / (float) frame[n].fits.height * (d->h - 4);
      /*  Check if the keypress was inside the box  */
      if (mx >= xr && mx <= xr + wr &&
	  my >= yr && my <= yr + hr) {
	mxr = mx;
	myr = my;
	move = 1;
      }
    }
    break;
  case FL_RELEASE:
    if (key == 1 && move) {
      x = (x - d->x - 2) / (float) (d->w - 4) * frame[n].fits.width;
      y = (y - d->y - 2) / (float) (d->h - 4) * frame[n].fits.height;
      if (x >= 0 && y >= 0) {
	for (i = 0; i < state.frame_used; i++) {
	  if (frame[i].fits.width == frame[n].fits.width && 
	      frame[i].fits.height == frame[n].fits.height) {
	    frame[i].fits.x2 += (x - frame[i].fits.x1 + 1);
	    if (frame[i].fits.x2 >= frame[i].fits.width) frame[i].fits.x2 = frame[i].fits.width - 2;
	    frame[i].fits.y2 += (y - frame[i].fits.y1 + 1);
	    if (frame[i].fits.y2 >= frame[i].fits.height) frame[i].fits.y2 = frame[i].fits.height - 2;
	    frame[i].fits.x1 = x + 1;
	    frame[i].fits.y1 = y + 1;
	    set_image(frame, i); 
	  }
	}
	draw_image(frame, n); 
	show_pan(frame, n);
      }
      move = 0;
    }
    break;
  case FL_MOUSE:  
    if (move) {
      fl_drw_box(FL_DOWN_BOX, d->x, d->y, d->w, d->h, FL_GRAY35, 2);
      x = xr + mx - mxr;
      y = yr + my - myr;
      if (x < d->x + 2) x = d->x + 2;
      if (y < d->y + 2) y = d->y + 2;
      if (x + wr + 2 > d->x + d->w - 2) x = d->x + d->w - wr - 2;
      if (y + hr + 2 > d->y + d->h - 2) y = d->y + d->h - hr - 2;
      fl_drw_box(FL_UP_BOX, x, y, wr, hr, FL_GRAY63, 2);
    }
    break;
  case FL_KEYBOARD:
    break;
  case FL_ENTER:
    break;
  case FL_LEAVE:
    break;
  }
  return 0;
}

/*  Handler for the blink window  */
int
freeobj_blinkW_handle(FL_OBJECT *d, int event, FL_Coord mx, FL_Coord my, int key, void *xev)

{
  GC gc;
  Window win;
  int w, h, w0, h0, x0, y0;
  float kx, ky;
  static int xp = 0, yp = 0;
  int xmax, ymax;
  int n;
  static int out = 0;
  static int force = 0;
  char temp[20];
  double r, rmin;
  int i, imin;
  XEvent *ev;
  int xr1, xr2, yr1, yr2, t;
  static int move = 0;
  static int  rbx, rby;
  static int xr, yr, wr, hr;
  int n0 = state.control[0], n1 = state.control[1];

  if ((state.frame_used == 0 && event != FL_DRAW) || out) return 1;
  gc = fl_state[fl_get_form_vclass(state.blinker->blinker)].gc[0];
  win = FL_ObjWin(d);
  fl_winset(win);
  n = state.control[state.control_num];
  /*  Convert mouse coordinates to image coordinates  */
  w = d->w;
  h = d->h;
  xmax = (frame[n].fits.x2 - frame[n].fits.x1 + 3);
  ymax = (frame[n].fits.y2 - frame[n].fits.y1 + 3);
  kx = (float) (xmax - 1) / (w - 1);
  ky = (float) (ymax - 1) / (h - 1);
  if (kx < ky) kx = ky;
  w0 = (xmax - 1) / kx + 0.5;
  h0 = (ymax - 1) / kx + 0.5;
  x0 = d->x + (w - w0) / 2 + 0.5;
  y0 = d->y + (h - h0) / 2 + 0.5;

  switch(event) {
  case FL_STEP:
    break;
  case FL_DRAW:
    if (state.bw != w || state.bh != h) {
      int i;
      out = 1;
      fl_addto_form(state.blinker->blinker);
      /*  Delete free object  */
      fl_delete_object(state.blinker->blinkW);
      /*  Add free object again  */
      fl_get_winsize(FL_ObjWin(state.blinker->zoom1W), &w, &h);
      h = state.blinker->zoom1W->y;
      state.blinker->blinkW = 
	fl_add_free(FL_NORMAL_FREE,10,
		    state.blinker->racoorW->y + state.blinker->racoorW->h + 5, 
		    w - 20, 
		    h - (state.blinker->racoorW->y + state.blinker->racoorW->h) - 10,
		    "",freeobj_blinkW_handle);
      fl_set_object_boxtype(state.blinker->blinkW, FL_FRAME_BOX);
      fl_end_form();
      state.bw = state.blinker->blinkW->w;
      state.bh = state.blinker->blinkW->h;
      fl_get_winsize(win, &state.blinkwidth, &state.blinkheight);
      /*  Free all of the existing pixmaps  */
      for (i = state.frame_used - 1; i >= 0; i--) {
	XFreePixmap(state.disp, frame[i].pixmap); 
      }
      /*  Create pixmaps with new dimensions and draw images into them  */
      for (i = 0; i < state.frame_used; i++) {
	frame[i].pixmap = 
	  XCreatePixmap(state.disp, 
			FL_ObjWin(state.blinker->blinkW), 
			state.bw, state.bh, 
			fl_state[fl_get_form_vclass(state.blinker->blinker)].depth);
	set_image(frame, i); 
      }
      out = 0;
    }
    
    /*  redraw the form  */
    fl_redraw_object(state.blinker->bgroupW);
    if (state.frame_used > 0) {
      if (!state.freeze) {
	float xt = xp - frame[n].dx; 
	float yt = yp - frame[n].dy; 
	draw_zoom(frame, n0, state.blinker->zoom1W, 
		  xt + frame[n0].dx, yt + frame[n0].dy);
	draw_profiles(frame, n, 
		      (int) (xt + frame[n].dx), (int) (yt + frame[n].dy));
	draw_zoom(frame, n1, state.blinker->zoom2W, 
		  xt + frame[n1].dx - frame[n0].fits.x1 + frame[n1].fits.x1,  
		  yt + frame[n1].dy - frame[n0].fits.y1 + frame[n1].fits.y1);
      }
      draw_image(frame, n);
      show_pan(frame, n);
    }
    break;
  case FL_MOTION:
    if (force) {
      force--;
    }
    else {
    /*  Determine image coordinates from the screen coordinates of the mouse  */
      xp = (mx - x0) * kx + frame[n].dx + frame[n].fits.x1 - 0.5;
      yp = (my - y0) * kx + frame[n].dy + frame[n].fits.y1 - 0.5;
    }
    if (!state.freeze) {
      int n0 = state.control[0], n1 = state.control[1];
      float xt = xp - frame[n].dx; 
      float yt = yp - frame[n].dy; 
      /*  Draw magnified images around mouse position in the window  */
      draw_zoom(frame, n0, state.blinker->zoom1W, 
		xt + frame[n0].dx, yt + frame[n0].dy);
      draw_profiles(frame, n, 
		    (int) (xt + frame[n].dx), (int) (yt + frame[n].dy));
      draw_zoom(frame, n1, state.blinker->zoom2W, 
		xt + frame[n1].dx - frame[n0].fits.x1 + frame[n1].fits.x1, 
		yt + frame[n1].dy - frame[n0].fits.y1 + frame[n1].fits.y1);
    }
    fl_drawmode(GXcopy);
    XSetForeground(state.disp, gc, fl_get_pixel(state.blinker->xcoorW->col1));
    sprintf(temp, "%d", xp);
    fl_rectf(state.blinker->xcoorW->x, state.blinker->xcoorW->y,
	   state.blinker->xcoorW->w, state.blinker->xcoorW->h,
	     state.blinker->xcoorW->col1);
    fl_set_input(state.blinker->xcoorW, temp);
    sprintf(temp, "%d", yp);
    fl_rectf(state.blinker->ycoorW->x, state.blinker->ycoorW->y,
	   state.blinker->ycoorW->w, state.blinker->ycoorW->h,
	     state.blinker->ycoorW->col1);
    fl_set_input(state.blinker->ycoorW, temp);
    sprintf(temp, "%u", video_point(&(frame[n].fits), xp, yp));
    fl_rectf(state.blinker->valueW->x, state.blinker->valueW->y,
	   state.blinker->valueW->w, state.blinker->valueW->h,
	     state.blinker->valueW->col1);
    fl_set_input(state.blinker->valueW, temp);

    if (frame[n].fitsfile.wcs) {
      double ra, dec;
      int status = 0;
      fits_pix_to_world(xp + 1, yp + 1, 
	       frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
	       frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2, 
	       frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2, 
	       frame[n].fitsfile.crot,
	       frame[n].fitsfile.ctype,
	       &ra, &dec, &status);
      if (ra >= 360.0) {
	ra -= 360.0;
      }
      if (!status) {
	int rah, ram, dech, decm, c;
	double ral, decl;
	ra /= 15.0;
	rah = (int) ra;
	ram = (int) ((ra - rah) * 60);
	ral = 60.0 * (60.0 * (ra - rah) - ram);
	if (dec < 0) {
	  c = '-';
	  dec = -dec;
	}
	else {
	  c = '+';
	}
	dech = (int) dec;
	decm = (int) ((dec - abs(dech)) * 60);
	decl = 60.0 * (60.0 * (dec - abs(dech)) - decm);
	sprintf(temp, "%2d:%02d:%06.3f", rah, ram, ral);
	fl_rectf(state.blinker->racoorW->x, state.blinker->racoorW->y,
	       state.blinker->racoorW->w, state.blinker->racoorW->h,
	       state.blinker->racoorW->col1);
	fl_set_input(state.blinker->racoorW, temp);
	sprintf(temp, "%c%2d:%02d:%05.2f", c, dech, decm, decl);
	fl_rectf(state.blinker->deccoorW->x, state.blinker->deccoorW->y,
	       state.blinker->deccoorW->w, state.blinker->deccoorW->h,
	       state.blinker->deccoorW->col1);
	fl_set_input(state.blinker->deccoorW, temp);
      }
    }
    break;
  case FL_DBLCLICK:
    /*  Double click inside the image freezes images in zoom windows  */
    state.freeze = !state.freeze;
    if (!state.freeze) {
      circleC(0, 0);
    }      
    else {
      /*  Store coordinates  */
      state.xf = (mx - x0) * kx + frame[state.control[1]].dx +
	frame[n1].fits.x1 - 0.5;
      state.yf = (my - y0) * kx + frame[state.control[1]].dy + 
	frame[n1].fits.y1 - 0.5;
      /*  Show cursors and LOCKED ! sign  */
      fl_show_object(state.blinker->lockedW);
      fl_show_object(state.blinker->upW);
      fl_show_object(state.blinker->downW);
      fl_show_object(state.blinker->rightW);
      fl_show_object(state.blinker->leftW);
      fl_show_object(state.blinker->circleW);
    }      
    break;
  case FL_PUSH:
    if (state.doastr && key == 1) {   /* If in astrometry mode, find the nearest star  */
      /*  Determine image coordinates from the screen coordinates of the mouse  */
      xp = (mx - x0) * kx + frame[n].dx + frame[n].fits.x1 - 0.5;
      yp = (my - y0) * kx + frame[n].dy + frame[n].fits.y1 - 0.5;
      /* Find the closest star from the star list  */
      rmin = 1e20;
      imin = -1;
      for (i = 0; i < frame[n].imagelist.n; i++) {
	r = fsqr(frame[n].imagelist.s[i].x - xp) + 
	  fsqr(frame[n].imagelist.s[i].y - yp);
	if (r < rmin) {
	  rmin = r;
	  imin = i;
	}
      }
      if (rmin > 25) {
	fl_show_alert("No detected object within circle of r = 5 pixels!", "Try again!", "", 1);
      }
      else {
	char name[30];
	if (frame[n].imagelist.s[imin].catnum != 0) {
	  name[0] = '\0';
	  make_starname(frame[n].imagelist.s[imin].catalog, 
			frame[n].imagelist.s[imin].catnum, name);
	}
	else {
	  strncpy(name, frame[n].obj[imin].s, 29);
	  name[29] = '\0';
	}
	if ((showcomet(frame, n, &frame[n].imagelist.s[imin], 0, name, 
		       frame[n].fits.aperture))) {
	  char a[FLEN_VALUE];
	  sprintf(a, "%-13s %c%4d %02d %08.5f %11s %11s          %4.1f %s      %3s", 
		  name, 'C', frame[n].year, frame[n].month,
		  frame[n].day +
		  (frame[n].hour + 
		   (frame[n].minute + frame[n].second / 60.0) / 60.0) / 24.0,
		  format_ra_mpc(frame[n].imagelist.s[imin].ra),
		  format_dec_mpc(frame[n].imagelist.s[imin].dec), 
		  2.5 * (frame[n].match.magfac - log10(frame[n].imagelist.s[imin].intensity)), 
		  frame[n].match.filter, observatory.code);


#if 0	  
	  if (strlen(name) > 7) 
	    name[8] = '\0';
	  if (strlen(name) > 0 && 
	      isalpha(name[0])) {
	    sprintf(a, "     %-8s %c%4d %02d %08.5f %11s %11s          %4.1f %s      %3s", 
		    name, 'C', frame[n].year, frame[n].month,
		    frame[n].day +
		    (frame[n].hour + 
		     (frame[n].minute + frame[n].second / 60.0) / 60.0) / 24.0,
		    format_ra_mpc(frame[n].imagelist.s[imin].ra),
		    format_dec_mpc(frame[n].imagelist.s[imin].dec), 
		    2.5 * (frame[n].match.magfac - log10(frame[n].imagelist.s[imin].intensity)), 
		    frame[n].match.filter, observatory.code);
	  }
	  else {
	    int number = strtol(name, NULL, 10);
	    sprintf(a, "%05d         %c%4d %02d %08.5f %11s %11s          %4.1f %s      %3s", 
		    number, 'C', frame[n].year, frame[n].month,
		    frame[n].day + 
		    (frame[n].hour + 
		     (frame[n].minute + frame[n].second / 60.0) / 60.0) / 24.0,
		    format_ra_mpc(frame[n].imagelist.s[imin].ra),
		    format_dec_mpc(frame[n].imagelist.s[imin].dec), 
		    2.5 * (frame[n].match.magfac - log10(frame[n].imagelist.s[imin].intensity)), 
		    frame[n].match.filter, observatory.code);
	  }
#endif
	  edit_mail(state.blinker->astrometryW, SEND_MAIL);
	  fl_addto_browser(state.mail->messageW, a);
	}
      }
    }
    else if (key == 2) {
      /*  Middle button is pressed, do a selection of a subarea  */
      int lw = fl_get_linewidth();  /* Save current linestyle and linewidth  */
      int ls = fl_get_linestyle();
      int dm = fl_get_drawmode();
      move = 1;
      fl_linewidth(2);
      fl_linestyle(FL_LONGDASH);
      fl_drawmode(GXxor);
      rbx = mx;
      rby = my;
      xr = rbx;
      yr = rby;
      wr = 5;
      hr = 5;
      fl_rect(xr, yr, wr, hr, FL_WHITE);
      fl_drawmode(dm);
      fl_linewidth(lw);
      fl_linestyle(ls);
    }
    if (key == 3) {   /* Find the nearest star  */
      /*  Determine image coordinates from the screen coordinates of the mouse  */
      xp = (mx - x0) * kx + frame[n].dx + frame[n].fits.x1 - 0.5;
      yp = (my - y0) * kx + frame[n].dy + frame[n].fits.y1 - 0.5;
      /* Find the closest star from the star list  */
      rmin = 1e20;
      imin = -1;
      for (i = 0; i < frame[n].imagelist.n; i++) {
	r = fsqr(frame[n].imagelist.s[i].x - xp) + 
	  fsqr(frame[n].imagelist.s[i].y - yp);
	if (r < rmin) {
	  rmin = r;
	  imin = i;
	}
      }
      if (rmin > 25) {
      }
      else {
	if (frame[n].imagelist.s[imin].catnum > 0) {
	  if (frame[n].obj[imin].s) free(frame[n].obj[imin].s);
	  frame[n].obj[imin].s = myalloc(100, "", "");
	  sprintf(frame[n].obj[imin].s, "%s %s", 
		   (frame[n].imagelist.s[imin].catalog == 'g') ?
		   "GSC" : (frame[n].imagelist.s[imin].catalog == 'u' ? "SA" : 
			    (frame[n].imagelist.s[imin].catalog == 'a' ? "ACT" : "")),
		   frame[n].imagelist.s[imin].catid);
	}
	showcomet(frame, n, &frame[n].imagelist.s[imin], 1, frame[n].obj[imin].s, frame[n].fits.aperture);
      }
    }
    break;
  case FL_RELEASE:
    if (key == 2 && move) {
      int lw = fl_get_linewidth();  /* Save current linestyle and linewidth  */
      int ls = fl_get_linestyle();
      int dm = fl_get_drawmode();
      int x1, x2, y1, y2;

      fl_linewidth(2);
      fl_linestyle(FL_LONGDASH);
      fl_drawmode(GXxor);
      /*  Erase previous rectangle */
      fl_rect(xr, yr, wr, hr, FL_WHITE);
      fl_drawmode(dm);
      fl_linewidth(lw);
      fl_linestyle(ls);
      /*  Calculate new active area for the current frame  */
      x1 = frame[n].fits.x1 + (xr - x0) * kx + /* frame[n].dx + */ 0.5;
      y1 = frame[n].fits.y1 + (yr - y0) * kx + /* frame[n].dy + */0.5;
      x2 = frame[n].fits.x1 + (xr + wr - x0) * kx + /* frame[n].dx + */0.5;
      y2 = frame[n].fits.y1 + (yr + hr - y0) * kx + /* frame[n].dy + */0.5;
      if (x1 > x2) {
	int t = x1;
	x1 = x2;
	x2 = t;
      }
      if (y1 > y2) {
	int t = y1;
	y1 = y2;
	y2 = t;
      }
      if (x1 < 0) x1 = 0;
      if (y1 < 0) y1 = 0;
      move = 0;
      for (i = 0; i < state.frame_used; i++) {
	if (frame[i].fits.width == frame[n].fits.width && 
	    frame[i].fits.height == frame[n].fits.height) {
	  frame[i].fits.x1 = x1 + 1;
	  frame[i].fits.y1 = y1 + 1;
	  frame[i].fits.x2 = x2 - 1;
	  frame[i].fits.y2 = y2 - 1;
	  if (frame[i].fits.x2 >= frame[i].fits.width) 
	    frame[i].fits.x2 = frame[i].fits.width - 1;
	  if (frame[i].fits.y2 >= frame[i].fits.height) 
	    frame[i].fits.y2 = frame[i].fits.height - 1;
	  set_image(frame, i); 
	}
      }
      draw_image(frame, n); 
      show_pan(frame, n);
    }
    break;
  case FL_MOUSE:  
    if (move) {
      int lw = fl_get_linewidth();  /* Save current linestyle and linewidth  */
      int ls = fl_get_linestyle();
      int dm = fl_get_drawmode();
      fl_linewidth(2);
      fl_linestyle(FL_LONGDASH);
      fl_drawmode(GXxor);
      /*  Erase previous rectangle */
      fl_rect(xr, yr, wr, hr, FL_WHITE);
      xr = rbx;
      yr = rby;
      wr = mx - rbx + 1;
      hr = my - rby + 1;
      /*  Draw a new rectangle */
      fl_rect(xr, yr, wr, hr, FL_WHITE);
      fl_drawmode(dm);
      fl_linewidth(lw);
      fl_linestyle(ls);
    }
    break;
  case FL_KEYBOARD:
    ev = (XEvent *) xev;
    /*  Determine image coordinates from the screen coordinates of the mouse  */
    switch (ev->xkey.keycode) {
    case 98:
      yp--;
      break;
    case 100:
      xp--;
      break;
    case 102:
      xp++;
      break;
    case 104:
      yp++;
      break;
    }
    /*  Calculate back mouse coordinates  */
    mx = x0 - (frame[n].dx + frame[n].fits.x1 - 1 - xp) / kx;
    my = y0 - (frame[n].dy + frame[n].fits.y1 - 1 - yp) / kx;
    fl_get_win_mouse(win, &xr1, &yr1,  &t);
    fl_get_mouse(&xr2, &yr2,  &t);
    fl_set_mouse(mx + xr2 - xr1, my + yr2 - yr1);
    force = 4;
    break;
  case FL_ENTER:
    fl_set_cursor(win, (state.doastr) ? XC_cross : XC_dotbox);
    fl_set_object_label(state.blinker->hintW, "Use arrow keys to move pixel by pixel");
    break;
  case FL_LEAVE:
    fl_reset_cursor(win);
    XSetForeground(state.disp, gc, fl_get_pixel(state.blinker->xcoorW->col1));
    fl_rectf(state.blinker->xcoorW->x, state.blinker->xcoorW->y,
	   state.blinker->xcoorW->w, state.blinker->xcoorW->h,
	     state.blinker->xcoorW->col1);
    fl_set_input(state.blinker->xcoorW, "");
    fl_rectf(state.blinker->ycoorW->x, state.blinker->ycoorW->y,
	   state.blinker->ycoorW->w, state.blinker->ycoorW->h,
	     state.blinker->ycoorW->col1);
    fl_set_input(state.blinker->ycoorW, "");
    fl_rectf(state.blinker->racoorW->x, state.blinker->racoorW->y,
	   state.blinker->racoorW->w, state.blinker->racoorW->h,
	     state.blinker->racoorW->col1);
    fl_set_input(state.blinker->racoorW, "");
    fl_rectf(state.blinker->deccoorW->x, state.blinker->deccoorW->y,
	   state.blinker->deccoorW->w, state.blinker->deccoorW->h,
	     state.blinker->deccoorW->col1);
    fl_set_input(state.blinker->deccoorW, "");
    fl_rectf(state.blinker->valueW->x, state.blinker->valueW->y,
	   state.blinker->valueW->w, state.blinker->valueW->h,
	     state.blinker->valueW->col1);
    fl_set_input(state.blinker->valueW, "");
    fl_set_object_label(state.blinker->hintW, "");

    if (move) {
      int lw = fl_get_linewidth();  /* Save current linestyle and linewidth  */
      int ls = fl_get_linestyle();
      int dm = fl_get_drawmode();

      fl_linewidth(2);
      fl_linestyle(FL_LONGDASH);
      fl_drawmode(GXxor);
      /*  Erase previous rectangle */
      fl_rect(xr, yr, wr, hr, FL_WHITE);
      fl_drawmode(dm);
      fl_linewidth(lw);
      fl_linestyle(ls);
      move = 0;
    }

    break;
  }
  return 0;
}


void
colormap(long val)

{
  switch (val) {
  case 1:
    define_greymap();
    fl_set_button(state.blinker->greybutW, 1);
    fl_set_button(state.blinker->colorbutW, 0);
    break;
  case 2:
    define_colormap();
    fl_set_button(state.blinker->greybutW, 0);
    fl_set_button(state.blinker->colorbutW, 1);
    break;
  }
}

/*  Change from color image to grey level and vice versa */
void
colormapC(FL_OBJECT *obj, long val)

{
  colormap(val);
}


void
do_markers(void) 

{
  int n = state.control[state.control_num];

  if (state.frame_used > 0) {
    if (!frame[n].objshow && frame[n].objnum > 0) {
      set_image(frame, n);
    }
    else {
      /*  Draw objects to the currently displayed image  */
      draw_objects(frame, n);
    }
    draw_image(frame, n);
  }
}

void
markers(int m) 

{
  int n = state.control[state.control_num];
  if (state.frame_used > 0) {
    frame[n].objshow = m;
    do_markers();
  }
}

/**********************************/
/* Switch display of markers      */
/**********************************/
void
markersC(FL_OBJECT *obj, long val)

{
  int n = state.control[state.control_num];
  if (state.frame_used > 0) {
    frame[n].objshow = fl_get_button(obj);
    do_markers();
  }
}

/*  Define the greymap  */
int
define_greymap(void)

{
  Visual *visual;
  int i, j;
  float k;

  visual = DefaultVisual(state.disp, DefaultScreen((state.disp)));
  /*  Get number of grey levels  */
  state.grey_max = min(visual->map_entries - FL_BUILT_IN_COLS, GREY_WANTED);
  state.grey_max--;
  for (i = 0; i <= state.grey_max; i++) 
    state.pixel[i] = i + FL_FREE_COL1;
  fl_free_pixels(state.pixel, state.grey_max + 1);
  k = 255.0 / state.grey_max;
  for (i = 0; i <= state.grey_max; i++) {
    j = (int) (i * k);
    fl_mapcolor(i + FL_FREE_COL1, j, j, j);
    state.pixel[i] = fl_get_pixel(state.pixel[i]);
  }
  for (i = 0; i < state.frame_used; i++) 
    set_image(frame, i);
  if (state.frame_used > 0)
    draw_image(frame, state.control[state.control_num]);
  state.color = 0;
  return 0;
}

/*  Define the colormap  */
int
define_colormap(void)

{
  Visual *visual;
  int i, j, step;
  static int rgb[][3] = {{0,8,16}, {0,12,24}, {0,16,28}, {0,20,32},
		     {4,20,36}, {4,24,40}, {4,28,40}, {4,28,44},
		     {4,32,48}, {4,32,52}, {4,36,52}, {4,36,56},
		     {4,40,56}, {4,40,60}, {4,40,60}, {4,44,64},
		     {4,44,64}, {4,48,68}, {8,48,68}, {8,52,72},
		     {8,52,72}, {8,56,76}, {8,56,76}, {8,60,76},
		     {8,60,80}, {8,60,80}, {8,64,84}, {8,64,84},
		     {12,68,84}, {12,68,88}, {12,72,88}, {12,72,88},
		     {12,76,92}, {12,76,92}, {12,80,92}, {16,80,96},
		     {16,80,96}, {16,84,96}, {16,84,100}, {16,88,100},
		     {16,88,100}, {20,92,104}, {20,92,104}, {20,96,104},
		     {20,96,108}, {20,100,108}, {24,100,108}, {24,104,108},
		     {24,104,112}, {24,108,112}, {28,108,112}, {28,112,116},
		     {28,112,116}, {28,116,116}, {32,116,116}, {32,120,120},
		     {32,120,120}, {32,120,120}, {36,120,120}, {36,124,120},
		     {36,124,116}, {36,124,116}, {36,124,116}, {40,128,116},
		     {40,128,116}, {40,128,116}, {44,128,116}, {44,132,116},
		     {44,132,116}, {44,132,116}, {48,132,116}, {48,136,116},
		     {48,136,116}, {52,136,116}, {52,136,116}, {52,140,116},
		     {52,140,116}, {56,140,116}, {56,140,116}, {56,140,116},
		     {60,144,116}, {60,144,116}, {64,144,116}, {64,144,116},
		     {64,148,116}, {68,148,116}, {68,148,116}, {68,148,116},
		     {72,148,116}, {72,152,116}, {76,152,116}, {76,152,116},
		     {76,152,116}, {80,152,116}, {80,156,116}, {84,156,116},
		     {84,156,112}, {88,156,112}, {88,156,112}, {88,160,112},
		     {92,160,112}, {92,160,112}, {96,160,112}, {96,160,112},
		     {100,164,112}, {100,164,112}, {104,164,112}, {104,164,112},
		     {108,164,112}, {108,168,116}, {112,168,116}, {112,168,116},
		     {116,168,116}, {116,168,116}, {120,172,116}, {120,172,116},
		     {124,172,116}, {124,172,116}, {128,172,116}, {128,172,116},
		     {132,176,116}, {136,176,116}, {136,176,116}, {140,176,120},
		     {140,176,120}, {144,180,120}, {144,180,120}, {148,180,120},
		     {148,180,120}, {152,180,120}, {152,180,120}, {156,184,124},
		     {156,184,124}, {160,184,124}, {160,184,124}, {164,184,124},
		     {164,184,128}, {168,188,128}, {168,188,128}, {172,188,128},
		     {172,188,132}, {176,188,132}, {176,188,132}, {180,192,132},
		     {180,192,132}, {180,192,136}, {184,192,136}, {184,192,136},
		     {188,192,140}, {188,196,140}, {188,196,140}, {192,196,140},
		     {192,196,144}, {192,196,144}, {196,196,144}, {196,200,148},
		     {196,200,148}, {200,200,148}, {200,200,152}, {200,200,152},
		     {200,200,152}, {204,200,152}, {204,200,156}, {204,200,156},
		     {204,200,156}, {204,200,156}, {204,204,160}, {204,204,160},
		     {208,204,160}, {208,204,160}, {208,204,164}, {208,204,164},
		     {208,204,164}, {208,204,168}, {208,204,168}, {212,208,168},
		     {212,208,168}, {212,208,172}, {212,208,172}, {212,208,172},
		     {212,208,176}, {216,212,176}, {216,212,176}, {216,212,180},
		     {216,212,180}, {216,212,180}, {216,216,184}, {216,216,184},
		     {220,216,184}, {220,216,188}, {220,216,188}, {220,220,188},
		     {220,220,192}, {220,220,192}, {220,220,192}, {224,220,196},
		     {224,224,196}, {224,224,196}, {224,224,200}, {224,224,200},
		     {224,224,200}, {224,224,200}, {224,228,204}, {224,228,204},
		     {224,228,204}, {224,228,204}, {228,228,208}, {228,228,208},
		     {228,228,208}, {228,232,208}, {228,232,208}, {228,232,212},
		     {228,232,212}, {228,232,212}, {228,232,212}, {228,232,216},
		     {228,232,216}, {228,236,216}, {228,236,216}, {228,236,216},
		     {228,236,220}, {228,236,220}, {228,236,220}, {228,236,220},
		     {232,240,220}, {232,240,224}, {232,240,224}, {232,240,224},
		     {232,240,224}, {232,240,224}, {232,240,224}, {232,240,228},
		     {232,244,228}, {232,244,228}, {232,244,228}, {232,244,228},
		     {232,244,232}, {232,244,232}, {232,244,232}, {232,244,232},
		     {236,248,232}, {236,248,232}, {236,248,236}, {236,248,236},
		     {236,248,236}, {236,248,236}, {236,248,236}, {236,248,236},
		     {236,252,240}, {236,252,240}, {236,252,240}, {236,252,240},
		     {236,252,240}, {236,252,240}, {236,252,240}};
  
  visual = DefaultVisual(state.disp, DefaultScreen((state.disp)));
  /*  Get number of colors levels  */
  state.grey_max = min(visual->map_entries - FL_BUILT_IN_COLS, GREY_WANTED);
  step = 256 / state.grey_max;
  state.grey_max--;
  for (i = 0; i <= state.grey_max; i++) 
    state.pixel[i] = i + FL_FREE_COL1;
  fl_free_pixels(state.pixel, state.grey_max + 1);
  j = 0;
  for (i = 0; i <= state.grey_max; i++) {
    fl_mapcolor(state.pixel[i], rgb[j][0], rgb[j][1], rgb[j][2]);
    state.pixel[i] = fl_get_pixel(state.pixel[i]);
    j += step;
  }
  for (i = 0; i < state.frame_used; i++) 
    set_image(frame, i);
  if (state.frame_used > 0)
    draw_image(frame, state.control[state.control_num]);
  state.color = 1;
  return 0;
}


#ifdef SWIG
/*  Resize the blinker window  */
int
resize_blinker(int x, int y, int w, int h)

{
  if (x >= 0 && y >= 0 && w > 400 && h > 400 && w <= 1280 && h <= 1024) {
    fl_set_form_position(state.blinker->blinker, x, y);
    fl_set_form_size(state.blinker->blinker, w, h);
    return 0;
  }
  else return -1;
}
#endif

void
loop(void)

{
  do {
    FL_OBJECT *o;
    XEvent event;
    o = fl_check_forms();
    if (o == FL_EVENT) {
      fl_XNextEvent(&event);
    }
    else if (o == state.blinker->playW) {
      if (state.frame_used > 1 && fl_get_button(o)) {
	state.blink = 1;
	state.blinknum = 0;
      }
      else {
	fl_set_button(o, 0);
	state.blink = 0;
	if (state.frame_used > 0) {
	  draw_image(frame, state.control[state.control_num]);
	  show_pan(frame, state.control[state.control_num]);
	}
      }
    } else if (state.b[0] && o == state.browser[0]->closeeditW) {
      fl_clear_browser(state.browser[0]->browseW);
      fl_hide_form(state.browser[0]->Edit);
      fl_activate_object(state.gui->firstfitsW);
      state.b[0] = 0;
    } else if (state.b[1] && o == state.browser[1]->closeeditW) {
      fl_clear_browser(state.browser[1]->browseW);
      fl_hide_form(state.browser[1]->Edit);
      fl_activate_object(state.gui->secondfitsW);
      state.b[1] = 0;
    } else if (state.readme && o == state.readme->closehelpW) {
      fl_hide_form(state.readme->help);
      fl_set_menu_item_mode(state.gui->helpW, 2, FL_PUP_NONE);
    } else if (state.parameters && o == state.parameters->parokW) {
      fl_hide_form(state.parameters->Parameters);
      fl_set_menu_item_mode(state.blinker->astrometryW, EDIT_PARAMETERS, FL_PUP_NONE);
      fl_set_menu_item_mode(state.blinker->astrometryW, BLUR_IMAGE, FL_PUP_NONE);
      fl_set_menu_item_mode(state.blinker->astrometryW, DETECT_STARS, FL_PUP_NONE);
    } else if (state.options && o == state.options->acceptoptionsW) {
      fl_hide_form(state.options->Options);
      fl_set_menu_item_mode(state.blinker->astrometryW, OPTIONS, FL_PUP_NONE);
    } else if (state.catalogs && o == state.catalogs->okcatalogsW) {
      fl_hide_form(state.catalogs->Catalog);
      fl_set_menu_item_mode(state.blinker->astrometryW, CATALOGS, FL_PUP_NONE);
    } else if (state.mail && o == state.mail->closemailW) {
      fl_hide_form(state.mail->Mail);
      fl_set_menu_item_mode(state.blinker->astrometryW, SEND_MAIL, FL_PUP_NONE);
    } else if (state.report && o == state.report->closereportW) {
      hide_report();
    } else if (state.telescope && o == state.telescope->telcloseW) {
      fl_remove_timeout(state.timeout);
      fl_hide_form(state.telescope->Telescope);
    }
    if (state.blink) do_blink();
  } while(!state.script);
}



void
tgui(int argc, char *argv[])

{
  FL_IOPT opt;
  int op, i;

  opt.sharedColormap = 1;
  fl_set_defaults(FL_PDSharedMap, &opt);
  state.disp = fl_initialize(&argc, argv, "Blink", 0, 0);
  /*  Load the settings  */
  load_settings();
  check_catalogs();
  /*  Create the main form  */
  state.gui = create_form_setting();
  /*  Create the blinker  */
  state.blinker = create_form_blinker();
   /*  Disable ticks in grey level histogram and correction graphs */
  fl_set_xyplot_xtics(state.gui->firstgreyW, -1.0, -1.0);
  fl_set_xyplot_ytics(state.gui->firstgreyW, -1.0, -1.0);
  fl_set_xyplot_xtics(state.gui->firstgammaW, -1.0, -1.0);
  fl_set_xyplot_ytics(state.gui->firstgammaW, -1.0, -1.0);
  fl_set_xyplot_xtics(state.gui->secondgreyW, -1.0, -1.0);
  fl_set_xyplot_ytics(state.gui->secondgreyW, -1.0, -1.0);
  fl_set_xyplot_xtics(state.gui->secondgammaW, -1.0, -1.0);
  fl_set_xyplot_ytics(state.gui->secondgammaW, -1.0, -1.0);

  /*  "Press" the button of the first image  */
  fl_set_button(state.gui->firstimageW, 1); 
  fl_set_button(state.gui->secondimageW, 0);
  /* Set the upper and lower limit for interval between images  */
  fl_set_counter_bounds(state.blinker->deltatW, 0.0, 5.0);
  fl_set_counter_value(state.blinker->deltatW, state.delay);
  fl_set_counter_step(state.blinker->deltatW, 0.1, 0.1);
  /*  Hide the LOCKED ! sign and cursors */
  fl_hide_object(state.blinker->lockedW);
  fl_hide_object(state.blinker->upW);
  fl_hide_object(state.blinker->downW);
  fl_hide_object(state.blinker->rightW);
  fl_hide_object(state.blinker->leftW);
  fl_hide_object(state.blinker->circleW);
  /*  Dectivate input fields with coordinates and pixel value  */
  fl_deactivate_object(state.blinker->xcoorW);
  fl_deactivate_object(state.blinker->ycoorW);
  fl_deactivate_object(state.blinker->racoorW);
  fl_deactivate_object(state.blinker->deccoorW);
  fl_deactivate_object(state.blinker->valueW);
  /*=====================*/
  /*  Set the File menu  */
  /*=====================*/
  fl_set_menu(state.gui->fileW, " Load to left control | Load to right "
	      "control | Load instruction file | "
	      "Save currently displayed image | Browse FITS table ");
  /*=====================*/
  /*  Set the help menu  */
  /*=====================*/
  fl_set_menu(state.gui->helpW, " About | Manual ");
  /*===========================*/
  /*  Set the astrometry menu  */
  /*===========================*/
  fl_set_menu(state.blinker->astrometryW, " Edit parameters | Blur image | "
	      "Detect stars | Match stars | Save star list | Load star list | "
	      "Do astrometry | | Options | Catalogs | Settings | "
	      "Mail results | Show report");
  /*  Disable menu options  */
  fl_set_menu_item_mode(state.blinker->astrometryW, EDIT_PARAMETERS, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, BLUR_IMAGE, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, DETECT_STARS, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, MATCH_STARS, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, SAVE_STAR_LIST, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, LOAD_STAR_LIST, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, DO_ASTROMETRY, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, EMPTY_ITEM1, FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, OPTIONS, FL_PUP_GREY);
  /*  Show the forms  */
  fl_show_form(state.gui->setting, FL_PLACE_MOUSE | FL_FIX_SIZE, FL_FULLBORDER, "Fitsblink");
  fl_set_form_minsize(state.blinker->blinker, 
    state.blinker->blinker->w / 2, state.blinker->blinker->h / 2);
  fl_set_form_size(state.blinker->blinker, state.blinkwidth, state.blinkheight);
  fl_show_form(state.blinker->blinker, FL_PLACE_FREE_CENTER, FL_FULLBORDER, "");
  fl_redraw_form(state.blinker->blinker);
  fl_set_form_title(state.blinker->blinker, "(no image)");
  /*  Store dimensions of the blinking area  */
  state.bw = state.blinker->blinkW->w;
  state.bh = state.blinker->blinkW->h;
  state.blinker->blinkW->wantkey = FL_KEY_ALL;

  /*  Settings for the profile  */
  fl_set_xyplot_xtics(state.blinker->xplotW, -1, -1);
  fl_set_xyplot_ytics(state.blinker->xplotW, -1, -1);
  fl_set_xyplot_xtics(state.blinker->yplotW, -1, -1);
  fl_set_xyplot_ytics(state.blinker->yplotW, -1, -1);
  if (state.color) { 
    define_colormap();
    fl_set_button(state.blinker->greybutW, 0);
    fl_set_button(state.blinker->colorbutW, 1);
  }
  else {
    define_greymap();
    fl_set_button(state.blinker->greybutW, 1);
    fl_set_button(state.blinker->colorbutW, 0);
  }


  while ((op = getopt(argc, argv, "i:vs")) != -1) {
    switch(op) {
    case 'i':
      /*  load  a .inp file  */
      load_file(optarg);
      break;
    case 'v':
      state.verbose = 1;
      break;
    case 's':
      state.script = 1;  /*  Fitsblink was called from a script */
      break;
    case '?':
      switch(optopt) {
      default:
	fprintf(stderr, "Unknown option -%c ignored!\n", optopt);
	break;
      }
      break;
    }
  }

  /* Load the images  */
  for (i = optind; i < argc; i++) {
    int j = i - optind;

    if (load_image(argv[i], 0) < 0) {
      continue;
    }
    if (j == 0 || j == 1) {
      state.control[state.control_num] = j;
      /*  Write the file name into the input field,  */
      /*  draw histogram, grey level correction curve, translation and rotation  */
      show_frame_info(frame, j, 0);
      if (state.frame_used == 1) {
	state.control[1] = 0;
	show_frame_info(frame, j, 1);
      }
    }
  }
  loop();
}
