/*  catalogs.c  */
/*  part of the fitsblink program  */
/*  routines for setting different star catalogs  */
/*  Jure Skvarc                                   */
#include <stdio.h>
#include <stdlib.h>
 
#include <forms.h>
#include <string.h>

#include "formblink.h"
#include "functs.h"

extern STATE state;
extern BLINK_FRAME *frame;
extern CATALOG catalog[];

void
checkusnoC(FL_OBJECT *obj, long val)
{
  catalog[CAT_USNO].use = fl_get_button(obj);
}

void
checkactC(FL_OBJECT *obj, long val)
{
  catalog[CAT_ACT].use = fl_get_button(obj);
}

void
checktycho2C(FL_OBJECT *obj, long val)
{
  catalog[CAT_TYCHO2].use = fl_get_button(obj);
}

void
checkguideC(FL_OBJECT *obj, long val)
{
  if (val == 0) {
    catalog[CAT_GUIDE].use = fl_get_button(obj);
  }
  else {
    catalog[CAT_GUIDE_SOUTH].use = fl_get_button(obj);
  }
}

void
usnodirC(FL_OBJECT *obj, long val)
{
  int l = strlen(fl_get_input(obj)) + 1;

  if (l > catalog[CAT_USNO].lpath) {
    if ((catalog[CAT_USNO].path = (char *) realloc(catalog[CAT_USNO].path, l)) == NULL) {
      fprintf(stderr, "Out of memory, reallocation of space for USNO catalog path,\n");
      exit(1);
    }
    catalog[CAT_USNO].lpath = l;
  }    
  strcpy(catalog[CAT_USNO].path, fl_get_input(obj));
}

void
actdirC(FL_OBJECT *obj, long val)
{
  int l = strlen(fl_get_input(obj)) + 1;

  if (l > catalog[CAT_ACT].lpath) {
    if ((catalog[CAT_ACT].path = (char *) realloc(catalog[CAT_ACT].path, l)) == NULL) {
      fprintf(stderr, "Out of memory, reallocation of space for ACT catalog path,\n");
      exit(1);
    }
    catalog[CAT_ACT].lpath = l;
  }    
  strcpy(catalog[CAT_ACT].path, fl_get_input(obj));
}


void
tycho2dirC(FL_OBJECT *obj, long val)
{
  int l = strlen(fl_get_input(obj)) + 1;

  if (l > catalog[CAT_TYCHO2].lpath) {
    if ((catalog[CAT_TYCHO2].path = (char *) realloc(catalog[CAT_TYCHO2].path, l)) == NULL) {
      fprintf(stderr, "Out of memory, reallocation of space for TYCHO2 catalog path,\n");
      exit(1);
    }
    catalog[CAT_TYCHO2].lpath = l;
  }    
  strcpy(catalog[CAT_TYCHO2].path, fl_get_input(obj));
}


void
guidedirC(FL_OBJECT *obj, long val)
{
  int l = strlen(fl_get_input(obj)) + 1;

  if (l > catalog[CAT_GUIDE].lpath) {
    if ((catalog[CAT_GUIDE].path = (char *) realloc(catalog[CAT_GUIDE].path, l)) == NULL) {
      fprintf(stderr, "Out of memory, reallocation of space for GUIDE catalog path,\n");
      exit(1);
    }
    catalog[CAT_GUIDE].lpath = l;
  }    
  strcpy(catalog[CAT_GUIDE].path, fl_get_input(obj));
}


void
guidedirsouthC(FL_OBJECT *obj, long val)
{
  int l = strlen(fl_get_input(obj)) + 1;

  if (l > catalog[CAT_GUIDE_SOUTH].lpath) {
    if ((catalog[CAT_GUIDE_SOUTH].path = (char *) realloc(catalog[CAT_GUIDE_SOUTH].path, l)) == NULL) {
      fprintf(stderr, "Out of memory, reallocation of space for GUIDE catalog path,\n");
      exit(1);
    }
    catalog[CAT_GUIDE_SOUTH].lpath = l;
  }    
  strcpy(catalog[CAT_GUIDE_SOUTH].path, fl_get_input(obj));
}


/*  Check if the catalogs are found at their default paths  */
void
check_catalogs(void)

{
  char *file;
  FILE *fp;

  /*  Check for ACT  */
  file = myalloc(sizeof(char) * (strlen(catalog[CAT_ACT].path) + 20), "check_catalogs", "file");
  sprintf(file, "%s/act0000fts.bin", catalog[CAT_ACT].path);
  /*  Can I open catalog file? */
  if ((catalog[CAT_ACT].use = (((fp = fopen(file, "r")) != NULL)))) {
    fclose(fp);
  }
  free(file);


  /*  Check for USNO SA 1.0  */
  file = myalloc(sizeof(char) * (strlen(catalog[CAT_USNO].path) + 15), "check_catalogs", "file");
  sprintf(file, "%s/zone%04d.cat", catalog[CAT_USNO].path, 0);
  /*  Can I open catalog file? */
  if ((catalog[CAT_USNO].use = (((fp = fopen(file, "r")) != NULL)))) {
    fclose(fp);
  }
  free(file);
  /*  Check for GSC north */
  file = myalloc(sizeof(char) * (strlen(catalog[CAT_GUIDE].path) + 
				strlen("/gsc/n0000/0001.gsc") + 1),
		"check_catalogs", "file");
  sprintf(file, "%s/gsc/n0000/0001.gsc", catalog[CAT_GUIDE].path);
  /*  Can I open catalog file? */
  if ((catalog[CAT_GUIDE].use = (((fp = fopen(file, "r")) != NULL)))) {
    fclose(fp);
  }
  free(file);

  if (!catalog[CAT_GUIDE].use) {
    /*  Check for GSC north again  */
    file = myalloc(sizeof(char) * (strlen(catalog[CAT_GUIDE].path) + 
				  strlen("/gsc/n0000/0001.gsc.gz") + 1),
		  "check_catalogs", "file");
    sprintf(file, "%s/gsc/n0000/0001.gsc.gz", catalog[CAT_GUIDE].path);
    /*  Can I open catalog file? */
    if ((catalog[CAT_GUIDE].use = (((fp = fopen(file, "r")) != NULL)))) {
      fclose(fp);
    }
    free(file);
  }
  /*  Check for GSC south */
  file = myalloc(sizeof(char) * (strlen(catalog[CAT_GUIDE_SOUTH].path) + 
		  strlen("/gsc/s7500/9346.gsc") + 1), "check_catalogs", "file");
  sprintf(file, "%s/gsc/s7500/9346.gsc", catalog[CAT_GUIDE_SOUTH].path);
  /*  Can I open catalog file? */
  if ((catalog[CAT_GUIDE_SOUTH].use = (((fp = fopen(file, "r")) != NULL)))) {
    fclose(fp);
  }
  free(file);
  if (!catalog[CAT_GUIDE_SOUTH].use) {
    /*  Check for GSC south again*/
    file = myalloc(sizeof(char) * (strlen(catalog[CAT_GUIDE_SOUTH].path) + 
				   strlen("/gsc/s7500/9346.gsc.gz") + 1), 
		   "check_catalogs", "file");
    sprintf(file, "%s/gsc/s7500/9346.gsc.gz", catalog[CAT_GUIDE_SOUTH].path);
    /*  Can I open catalog file? */
    if ((catalog[CAT_GUIDE_SOUTH].use = (((fp = fopen(file, "r")) != NULL)))) {
      fclose(fp);
    }
    free(file);
  }
  /*  Check for TYCHO-2  */
  file = myalloc(sizeof(char) * (strlen(catalog[CAT_TYCHO2].path) + 20), "check_catalogs", "file");
  sprintf(file, "%s/catalog.dat", catalog[CAT_TYCHO2].path);
  /*  Can I open catalog file? */
  if ((catalog[CAT_TYCHO2].use = (((fp = fopen(file, "r")) != NULL)))) {
    fclose(fp);
  }
  free(file);

}

/*===================================================================*/
/*  Define supported catalogs - must be called before they are used  */
/*===================================================================*/
void
define_catalogs(void)

{
  /*================*/
  /*  USNO          */
  /*================*/
  catalog[CAT_USNO].use = 0;
  catalog[CAT_USNO].name = "USNO A2.0";
  catalog[CAT_USNO].lpath = PATHLENGTH;
  if ((catalog[CAT_USNO].path = (char *) malloc(catalog[CAT_USNO].lpath)) == NULL) {
    fprintf(stderr, "Out of memory, path for USNO catalog.\n");
    exit(1);
  }
  catalog[CAT_USNO].path[0] = '\0';
  catalog[CAT_USNO].passband = USNO_RED;
  catalog[CAT_USNO].filter[USNO_BLUE] = "B";
  catalog[CAT_USNO].filter[USNO_RED] = "R";

  /*================*/
  /*     ACT        */
  /*================*/
  catalog[CAT_ACT].use = 0;
  catalog[CAT_ACT].name = "ACT";
  catalog[CAT_ACT].lpath = PATHLENGTH;
  if ((catalog[CAT_ACT].path = (char *) malloc(catalog[CAT_ACT].lpath)) == NULL) {
    fprintf(stderr, "Out of memory, path for ACT catalog.\n");
    exit(1);
  }
  catalog[CAT_ACT].path[0] = '\0';
  catalog[CAT_ACT].passband = ACT_BLUE;
  catalog[CAT_ACT].filter[ACT_BLUE] = "B";
  catalog[CAT_ACT].filter[ACT_RED] = "R";


  /*================*/
  /*     GSC        */
  /*================*/
  catalog[CAT_GUIDE].use = 0;
  catalog[CAT_GUIDE].name = "GSC 1.1";
  catalog[CAT_GUIDE].lpath = PATHLENGTH;
  if ((catalog[CAT_GUIDE].path = (char *) malloc(catalog[CAT_GUIDE].lpath)) == NULL) {
    fprintf(stderr, "Out of memory, path for GUIDE NORTH catalog.\n");
    exit(1);
  }
  catalog[CAT_GUIDE].path[0] = '\0';
  catalog[CAT_GUIDE_SOUTH].use = 0;
  catalog[CAT_GUIDE_SOUTH].name = "GSC 1.1";
  catalog[CAT_GUIDE_SOUTH].lpath = PATHLENGTH;
  if ((catalog[CAT_GUIDE_SOUTH].path = (char *) malloc(catalog[CAT_GUIDE_SOUTH].lpath)) == NULL) {
    fprintf(stderr, "Out of memory, path for GUIDE SOUTH catalog.\n");
    exit(1);
  }
  catalog[CAT_GUIDE_SOUTH].path[0] = '\0';

  /*================*/
  /*     TYCHO2     */
  /*================*/
  catalog[CAT_TYCHO2].use = 0;
  catalog[CAT_TYCHO2].name = "TYCHO-2";
  catalog[CAT_TYCHO2].lpath = PATHLENGTH;
  if ((catalog[CAT_TYCHO2].path = (char *) malloc(catalog[CAT_TYCHO2].lpath)) == NULL) {
    fprintf(stderr, "Out of memory, path for TYCHO-2 catalog.\n");
    exit(1);
  }
  catalog[CAT_TYCHO2].path[0] = '\0';
  catalog[CAT_TYCHO2].passband = TYCHO_V;
  catalog[CAT_TYCHO2].filter[TYCHO_V] = "V";
  catalog[CAT_TYCHO2].filter[TYCHO_B] = "B";

}

/*  Edit catalogs */
void
edit_catalogs(FL_OBJECT *obj, int m)

{
  int n;

  /*  The window in not created yet, do it now  */
  if (state.catalogs == NULL) {
    state.catalogs = create_form_Catalog();
    fl_set_form_minsize(state.catalogs->Catalog, 
		  state.catalogs->Catalog->w, state.catalogs->Catalog->h);
  }
  /*  Deactivate CATALOGS item in the menu */
  fl_set_menu_item_mode(obj, m, FL_PUP_GREY);
  fl_show_form(state.catalogs->Catalog,  FL_FIX_SIZE, FL_FULLBORDER | FL_PLACE_FREE_CENTER, "Star catalogs");
  /*  Write current parameter values into the form */
  n = state.control[state.control_num];

  fl_set_button(state.catalogs->checkusnoW, catalog[CAT_USNO].use);
  fl_set_input(state.catalogs->usnodirW, catalog[CAT_USNO].path);
  fl_set_button(state.catalogs->checkactW, catalog[CAT_ACT].use);
  fl_set_input(state.catalogs->actdirW, catalog[CAT_ACT].path);

  fl_set_button(state.catalogs->checkguideW, catalog[CAT_GUIDE].use);
  fl_set_button(state.catalogs->checkguidesouthW, catalog[CAT_GUIDE_SOUTH].use);
  fl_set_input(state.catalogs->guidedirW, catalog[CAT_GUIDE].path);
  fl_set_input(state.catalogs->guidedirsouthW, catalog[CAT_GUIDE_SOUTH].path);
  fl_set_button(state.catalogs->checktycho2W, catalog[CAT_TYCHO2].use);
  fl_set_input(state.catalogs->tycho2dirW, catalog[CAT_TYCHO2].path);
}

