/*  consts.h  for Fitsblink  */
#ifndef _blink_consts
#define _blink_consts
#include "version.h"
#define NGAMMA 5
#define FALSE 0
#define TRUE 1

/*  Conversion factor from degrees to radians  */
#define DEGRAD 1.74532925199e-2
#define RADDEG 57.2957795132
/*  Initial number of frames  */
#define FRAME_MAX 8
/*  Number of bins in a grey level histogram  */
#define HISTOGRAM_WIDTH 64
#define ERROR_MAX 6
/*  A maximal number of objects attached to an image  */
#define OBJECT_NUM 1000
/*  SHAPE indexes  */
#define OBJ_CIRCLE 0
#define OBJ_BOX 1
#define OBJ_DIAMOND 2
#define OBJ_UPTRIANGLE 3
#define OBJ_DOWNTRIANGLE 4
#define OBJ_MAXSHAPE 5
#define OBJ_MAXCOLOR 6

/*  Wanted number of grey levels  */
#define GREY_WANTED 128
#ifndef min
#define min(a,b) ((a) > (b)) ? (b) : (a)
#endif
#ifndef max
#define max(a,b) ((a) > (b)) ? (a) : (b) 
#endif
#define DEFAULT_DELAY 0,2
#define DEFAULT_AUTOLEVEL_HIGH 0.01

/*  Menu item numbers from the Astrometry menu  */
/*==============================================*/
#define EDIT_PARAMETERS 1
#define BLUR_IMAGE 2
#define DETECT_STARS 3
#define MATCH_STARS 4
#define SAVE_STAR_LIST 5
#define LOAD_STAR_LIST 6
#define DO_ASTROMETRY 7
#define EMPTY_ITEM1 8
#define OPTIONS 9
#define CATALOGS 10
#define SAVE_SETTINGS 11
#define SEND_MAIL 12
#define SHOW_REPORT 13

/* Menu item numbers for the File menu  */
#define LOAD_TO_LEFT 1
#define LOAD_TO_RIGHT 2
#define LOAD_INSTRUCTION_FILE 3
#define SAVE_DISPLAYED 4
#define BROWSE_FITS_TABLE 5

/* Maximum size of the line in a browser  */
#define MAXBROWSERLENGTH 2048

/*  Star list format related constants  */
/*  Output file (star list format)  */
/* Old, original format  */
#define OLD_ASCII_LIST 0
#define FITS_STARLIST 1
#define FITS_NFIELDS 17

/*  Help system related constants  */
#define DEFAULT_BROWSER "netscape"
#define DEFAULT_MANUAL_DIR "/usr/doc/fitsblink-3.2/fitsblink/fitsblink.html"

/*  Designation  related constants */
#define DEFAULT_DESIG_PREFIX "AA"
#define DEFAULT_DESIG_NUMBER 0

/*  Number of filters  */
#define FILTERNUM 3
/*  Constant definitions for functions which access USNO catalog */
/*===============================================================*/
/*  Width of the declination zone for USNO catalog  */
#define USNO_DECWIDTH 7.5
/*  Distance between two accelerator lines in decimal hours  */
#define USNO_RADIST 0.25
/*  Record length in bytes  */
#define USNO_RECORDLENGTH 12
/*  Maximal allowed number od stars */
#define MAXSTARNUM 33546
/*  buffer size  */
#define BUFSIZE 1000
#define USNO_BLUE 0
#define USNO_RED 1

/*  Constant definitions for functions which access GSC  catalog */
/*===============================================================*/
/*  Width of the declination zone for GUIDE catalog  */
#define GUIDE_DECWIDTH 7.5
#define NUM_GSC_PLATES 9537

/*  Constants for functions which access the ACT catalog  */
/*========================================================*/
#define ACT_RAWIDTH 7.5
/*  Record length in bytes  */
#define ACT_RECORDLENGTH 44
#define ACT_BLUE 0
#define ACT_RED 1

/*  Constants for functions which access the TYCHO2 catalog  */
/*===========================================================*/
#define TYCHO2_RECORDLEN 207
#define TYCHO_V 0
#define TYCHO_B 1

/*  Default values  */
#define DEFAULT_RAWIDTH   1.92
#define DEFAULT_DECHEIGHT 1.92
#define DEFAULT_WIDTH     512.0
#define DEFAULT_HEIGHT    512.0
#define DEFAULT_ROTATION  0.0
#define DEFAULT_MINMAG    -2.0
#define DEFAULT_MAXMAG    18.0

/*  FITSIO related constants  */
/*============================*/
#define CARDLENGTH 81

/*  General constants about catalogs  */
/*====================================*/
/*  Total number of catalogs  */
#define NUMBER_OF_CATALOGS 5

/*  Catalog numbers  */
#define PATHLENGTH 256
#define CAT_USNO 0
#define CAT_ACT 1
#define CAT_GUIDE 2
#define CAT_GUIDE_SOUTH 3
#define CAT_TYCHO2 4


/*  Digital Sky Survey access  */
#define DSS_BIG 1200.0
#define DSS_SERVER "archive.eso.org/dss/dss/image"
/*  Constants for the starcount functions  */
/*=========================================*/

#define PI 3.1415927 
/* stack size for star counting */
#define STACKMAX 40000
/* initial object array size */
#define NSTAR 2500;
/* assumed maximum star area  */
#define DEFAULT_STAR_AREA 50
/* Default size of a subimage where average background is calculated  */
#define DEFAULT_MAXSIZE 30
/* Default sensitivity threshold in background variation (sigma) units  */
#define DEFAULT_SGNUM 5.0
/*  Minimum object brightness above the background  */
#define DEFAULT_MINBRIGHT 50
/*  Minimum star size in pixels  */
#define DEFAULT_MINSTAR 2
/*  Size of the exclusion zone on the image edge where no stars are detected */
#define DEFAULT_BORDER 0
/*  Saturation level */
#define DEFAULT_SATLEVEL 65536
/*  Coefficient about resolving close lying stars  */
#define DEFAULT_RESOLVE 0.2
/*  Do we use aperture photometry? */
#define DEFAULT_APERTURE 1
/*  Radius of the inner circle for the aperture photometry */
#define DEFAULT_INNER 3.0
/*  Radius of the outer circle for the aperture photometry */
#define DEFAULT_OUTER 10.0
/*  Default camera gain [e-/ADU] */
#define DEFAULT_GAIN 3.0
/*  Default camera read out noise [e-] */
#define DEFAULT_RDNOISE 10.0
/*  Flag which says whether to find a peak inside the inner circle  */
#define FIND_PEAK TRUE

/*  Constants for the star matching functions  */
/*=============================================*/
/*  Default number of bright stars taken into account in initial matching  */
#define FIRSTBRIGHT 40
/*  Minimum acceptable number of stars for the initial matching */
#define MINFIRSTBRIGHT 6
/*  Maximum number of candidates for a constellation star  */
#define CANDIDATE_MAX 100
/*  Maximum allowed positional error for stars in constellation  */
#define POSERRMAX 6.0
/*  Maximum allowed magnitude error for stars in constellation  */
#define MAGERRMAX 100
/*  Minimum value of constellation to be accepted  */
#define CONSTMIN 15.0
/*  Minimum distance of stars in the initial set of stars used for matching  */
#define MINDIST 20.0
/*  Number of stars in constellation  */
#define CONSTSTAR 12
/*  Maximal allowed residual (in arcseconds)  */
#define MAXRES 1.0
/*  Extended region on each side of the image  */
#define MAP_EXTAREA 0.3
/* Order of plate constants  */
#define FITORDER 1
/* Default filter  */
#define DEFAULT_FILTER " "

/*   asteroid program related constants  */
#define RESULT_LENGTH  81
#define REZMAXNUM 200
#endif

