/*  dss.c  */
/*  Part of fitsblink  */
/*  Jure Skvarc        */
/*=====================*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/wait.h>
#include <glob.h>
#include "functs.h"

extern STATE state;
extern BLINK_FRAME *frame;
FITSFILE dssimage;
char *dsssources[] = {"DSS1", "DSS2", "DSS2"};

/* Retrieve a dss image from internet and store it to a file with the */
/* given name   */
int
get_dss_image(char *name, char *object, double ra, double dec,
	      double width, double height, char *dsssource)

{
  char url[1000];
  sprintf(url, "http://%s?-c.ra=%s&-c.dec=%s&-c.bm=%f/%f&-source=%s&-mime=image/x-fits(%s)", 
	   state.dss_server, 
	   format_ra(ra), format_dec(dec), 
	   width, height, dsssource, name);
  unlink(name);
  open_message("Downloading image");
  write_message("Downloading image");
  if (load_image(url, 1) >= 0) {
    erase_image(state.frame_used - 1);
    close_message();
    return 0;
  }
  else {
    close_message();
    return -1;
  }
}


/*  Using the currently displayed (actually, n-th) image as a template, */
/*  and download a DSS image. Then read the image and rescale it so that */
/*  it matches the image used as a template  */

int
dssload(BLINK_FRAME *frame, int n)

{
  float dx, dy;
  float sfi, cfi;
  float xproj, yproj;
  double xc, yc;
  double ra, dec;
  int err, nlen;
  int ni;  /* Index of the newly loaded image  */
  int nn;  /* Index of the created image  */
  char *name, *p, *p1;

  if (state.frame_used > 0) {
    if (!frame[n].fitsfile.matched && !frame[n].matched) {
      if (!fl_show_question("There is no indication that the WCS values can be trusted.\n
Would you like to continue anyway?", 1)) {
	return -1;
      }
    }
    /*  Calculate the width and height of the image n  */
    /*  ============================================== */
    dx = (frame[n].fits.x2 - frame[n].fits.x1 + 1) * frame[n].xsize;
    dy = (frame[n].fits.y2 - frame[n].fits.y1 + 1) * frame[n].ysize;
    sfi = sin(frame[n].fi * DEGRAD);
    cfi = cos(frame[n].fi * DEGRAD);
    xproj = fabs(dx * cfi) + fabs(dy * sfi);
    yproj = fabs(dx * sfi) + fabs(dy * cfi);
    if (xproj > state.dss_big || yproj > state.dss_big) {
      if (!fl_show_question("This image may take a lot of time to download.\nI suggest to choose a smaller region\n
Press Yes if you want to proceed anyway.", 1)) {
	return -1;
      }
    }
    /*  Calculate the center coordinates  */
    /*  ================================= */
    xc = 0.5 * (frame[n].fits.x2 + frame[n].fits.x1);
    yc = 0.5 * (frame[n].fits.y2 + frame[n].fits.y1);
    err = 0;
    fits_pix_to_world(xc + 1, yc + 1, 
		      frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
		      frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2,
		      frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2,
		      frame[n].fitsfile.crot, "-TAN",
		      &ra, &dec, &err);


    /*  Create file name by adding -dss to the template image */
    /* =======================================================*/
    nlen = strlen(frame[n].name) + 9;
    name = myalloc(sizeof(char) * nlen, "dss_load", "name");
    strcpy(name, frame[n].name);
    /*  Find last / in the file name  */
    p = strrchr(name, '/');
    if (!p) p = name;
    else p++;
    /*  Find first . in the file name  */
    p1 = strchr(p, '.');
    if (p1) *p1 = '\0';
    strcat(p, "-dss.fts");
    if (!get_dss_image(p, "", ra, dec, xproj / 60.0, yproj / 60.0, 
		       dsssources[dssimage.matched])) {
      ni = load_image(p, 1);
      show_frame_info(frame, ni, state.control_num);
      nn = create_image_from_template(frame, n, ni, p);
      add_to_choices(p, nn, 0);
      /*  remove the temporary image  */
      erase_image(ni);
    }
    free(name);
  }
  else {
    fl_show_alert("No image loaded!", "", "", 1);
  }
  return 0;
}



void
dssC(FL_OBJECT *obj, long val)

{
  int n = state.control[state.control_num];

  dssload(frame, n);
}


/*  Set RA of the image center  */
void
dssraC(FL_OBJECT *obj, long val)

{
  char *r;
  double ra;

  r = (char *) fl_get_input(obj);
  ra = parse_ra(r);
  dssimage.crval1 = ra;
  set_rainput(obj, ra);
}



/*  Set DEC of the image center  */
void
dssdecC(FL_OBJECT *obj, long val)

{
  char *r;
  double dec;

  r = (char *) fl_get_input(obj);
  dec = parse_dec(r);
  dssimage.crval2 = dec;
  set_decinput(obj, dec);
}

/*  Object name  */
void
dssobjectC(FL_OBJECT *obj, long val)

{
  strcpy(dssimage.object, fl_get_input(obj));
}


/*  Width of the image in arcminutes  */
void
dsswidthC(FL_OBJECT *obj, long val)

{
  dssimage.sizex = strtod(fl_get_input(obj), NULL);
  set_doubleval(obj, "%.3f", dssimage.sizex);
}


/* Height of the image in arcminutes  */
void
dssheightC(FL_OBJECT *obj, long val)

{
  dssimage.sizey  = strtod(fl_get_input(obj), NULL);
  set_doubleval(obj, "%.3f", dssimage.sizey);
}

void
dssnameC(FL_OBJECT *obj, long val)

{
  if (dssimage.name) free(dssimage.name);
  dssimage.name = strdup(fl_get_input(obj));
  trim_spaces(dssimage.name);
  fl_set_input(obj, dssimage.name);
}

void
dsssurveyC(FL_OBJECT *obj, long val)

{
  dssimage.matched = val;
}


void
dss_inputC(FL_OBJECT *obj, long val)

{
  int out;

  /*  The window in not created yet, do it now  */
  if (state.dss == NULL) {
    state.dss = create_form_Dss();
    fl_set_form_minsize(state.dss->Dss, 
			state.dss->Dss->w, state.dss->Dss->h);
    dssimage.crval1 = 0;
    dssimage.crval2 = 0;
    dssimage.sizex = 10;
    dssimage.sizey = 10;
    dssimage.name = strdup("dssimg");
    dssimage.wcs = 0;
    set_rainput(state.dss->dssraW, dssimage.crval1);
    set_decinput(state.dss->dssdecW, dssimage.crval2);
    set_doubleval(state.dss->dsswidthW, "%.3f", dssimage.sizex);
    set_doubleval(state.dss->dssheightW, "%.3f", dssimage.sizey);
    fl_set_input(state.dss->dssnameW, dssimage.name);
    fl_set_input(state.dss->dssobjectW, dssimage.object);
    fl_set_button(state.dss->dss1W, 1);
    fl_set_button(state.dss->dss2blueW, 0);
    fl_set_button(state.dss->dss2redW, 0);
  }
  fl_show_form(state.dss->Dss, FL_FIX_SIZE, FL_FULLBORDER | FL_PLACE_FREE_CENTER, "Load DSS field");
  out = 0;
  do {
    FL_OBJECT *o = fl_check_forms();
    XEvent event;
    if (o == FL_EVENT) {
      fl_XNextEvent(&event);
    }
    else if (o == state.dss->dssgetW) {
      /*  Download image from internet to a file  */
      get_dss_image(dssimage.name, dssimage.object, dssimage.crval1, 
		    dssimage.crval2, 
		    dssimage.sizex, dssimage.sizey,
		    dsssources[dssimage.matched]);
      load_image(dssimage.name, 1);
      out = 1;
    }
    else if (o == state.dss->dsscancelW) {
      out = 1;
    }
  } while(!out);
  fl_hide_form(state.dss->Dss);
}

