/* fb2pg.c  */
/* Read FITS star lists created by fitsblink and insert them into a  */
/* PostgreSQL database or print them to the standard output  */
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <string.h>
#include <getopt.h>
#include <time.h>
#include <unistd.h>

#include <fitsio.h>
#include <libpq-fe.h>
#include "structs.h"
#include "functs.h"


int
insert_stars(FILE *fptemp, char *fn, int to_database)

{

    char comment[81];
    int status  = 0;
    fitsfile *fptr;
    /*  Open the file  */
    fits_open_file(&fptr, fn, READONLY, &status);
    if (status) {
      if (state.interactive) {
	fl_show_alert("Can not open file", fn, "for reading!", 0);
	free(name);
	return 1;
      }
      else {
	fprintf(stderr, "Can not open file %s for reading!\n", fn);
	free(name);
	return 1;
      }	
    }
    /*  Move to the table extension  */
    fits_movnam_hdu(fptr, BINARY_TBL, "Objects", 0, &status);
    fits_report_error(stderr, status);
    /*  Read the number of stars in the array  */
    fits_get_num_rows(fptr, &nstar, &status);
    fits_report_error(stderr, status);
    /*  reserve space for stars  */
    frame[n].imagelist.s = (STAR *) myalloc(sizeof(STAR) * nstar, "load_star_list", "frame[n].imagelist.s");
    /*  Set star number  */
    frame[n].imagelist.n = nstar;
    
    /*  Read the magnitude factor  */
    fits_read_key_flt(fptr, "MAGFAC", &frame[n].match.magfac, 
		      comment, &status);

    status = 0;
    /*====================================================*/
    /*  Read counting parameters used for star detection  */
    /*====================================================*/
    fits_read_key(fptr, TINT, "X1", &frame[n].last_count_options.x1, 
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "Y1", &frame[n].last_count_options.y1, 
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "X2", &frame[n].last_count_options.x2,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "Y2", &frame[n].last_count_options.y2,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "MINBRITE", &frame[n].last_count_options.minbright,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "MINSTAR", &frame[n].last_count_options.minstar,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "BCKSIZE", &frame[n].last_count_options.maxsize,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "SG_NUM", &frame[n].last_count_options.sg_num,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TSHORT, "BORDER", &frame[n].last_count_options.border,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "SATLEVEL", &frame[n].last_count_options.satlevel,
		      comment, &status);
    status = 0; 
    fits_read_key_log(fptr, "APERTURE", &frame[n].last_count_options.aperture,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "AP_INNER", &frame[n].last_count_options.inner,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "AP_OUTER", &frame[n].last_count_options.outer,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "RESOLVE", &frame[n].last_count_options.resolve,
		      comment, &status);
    status = 0; 
    /*====================================================*/
    /*  Read matching parameters used for star detection  */
    /*====================================================*/
    fits_read_key_flt(fptr, "MINCONST", &frame[n].last_match_options.min,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT,  "FIRSTBRT", 
		  &frame[n].last_match_options.firstbright, comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "POSERRMX", &frame[n].last_match_options.poserrmax,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "MAXRES", &frame[n].last_match_options.maxres,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "MINDIST", &frame[n].last_match_options.mindist,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT,  "PLTORDER", 
		  &frame[n].last_match_options.fitorder_used, comment, &status);
    status = 0; 
    fits_read_key_str(fptr, "FILTER", 
		      frame[n].last_match_options.filter, comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "NUMSTARS", 
		  &frame[n].last_match_options.nmatched, comment, &status);
    if (status == 0) {
      frame[n].last_match_options.matched = 1;
    }
    status = 0; 

    /*  Read the original image name  */
    frame[n].name1 = (char *) myalloc(sizeof(char) * 1024, "load_star_list",
				     "frame[n].name1");
    fits_read_key_str(fptr, "IMGNAME", frame[n].name1, comment, &status);
    frame[n].matched = (frame[n].match.magfac > 0);

    /* Find out column numbers  */
    fits_get_colnum(fptr, CASESEN, "ID", &colid, &status);
    fits_get_colnum(fptr, CASESEN, "XT", &colxt, &status);
    fits_get_colnum(fptr, CASESEN, "YT", &colyt, &status);
    fits_get_colnum(fptr, CASESEN, "SXT", &colsxt, &status);
    fits_get_colnum(fptr, CASESEN, "SYT", &colsyt, &status);
    fits_get_colnum(fptr, CASESEN, "AREA", &colarea, &status);
    fits_get_colnum(fptr, CASESEN, "INTENSITY", &colintensity, &status);
    fits_get_colnum(fptr, CASESEN, "SATURATE", &colsaturate, &status);
    fits_get_colnum(fptr, CASESEN, "BACKGRND", &colbackgrnd, &status);
    fits_get_colnum(fptr, CASESEN, "RA", &colra, &status);
    fits_get_colnum(fptr, CASESEN, "DEC", &coldec, &status);
    fits_get_colnum(fptr, CASESEN, "SRA", &colsra, &status);
    fits_get_colnum(fptr, CASESEN, "SDEC", &colsdec, &status);
    fits_get_colnum(fptr, CASESEN, "MAG", &colmag, &status);
    fits_get_colnum(fptr, CASESEN, "SMAG", &colsmag, &status);
    fits_get_colnum(fptr, CASESEN, "CATNAME", &colcatname, &status);
    fits_get_colnum(fptr, CASESEN, "CATID", &colcatid, &status);
    fits_get_colnum(fptr, CASESEN, "COLOR", &colcolor, &status);
    fits_get_colnum(fptr, CASESEN, "SHAPE", &colshape, &status);
    fits_get_colnum(fptr, CASESEN, "SYMMETRY", &colsymmetry, &status);

    
    /*  Read all stars  */
    for (i = 0; i < nstar; i++) {
      int j = i + 1;
      int anynul, i1;
      float x, y;
      char *catid[1];
      float x, y, sxt, syt, bg, sra, sdec, mag, smag, sym;
      int area, intensity;
      short saturated, catalog;
      double ra, dec;
      fits_read_col(fptr, TINT, colid, j, 1, 1, NULL, &i1, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colxt, j, 1, 1, NULL, &x, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colyt, j, 1, 1, NULL, &y, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsxt, j, 1, 1, NULL, &sxt, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsyt, j, 1, 1, NULL, &syt, &anynul, &status);
      fits_read_col(fptr, TINT, colarea, j, 1, 1, NULL, &area, &anynul, &status);
      fits_read_col(fptr, TINT, colintensity, j, 1, 1, NULL, 
		     &intensity, &anynul, &status);
      fits_read_col(fptr, TSHORT, colsaturate, j, 1, 1, NULL, 
		     &saturated, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colbackgrnd, j, 1, 1, NULL, 
		     &bg, &anynul, &status);
      fits_read_col(fptr, TDOUBLE, colra, j, 1, 1, NULL, 
		     &ra, &anynul, &status);
      fits_read_col(fptr, TDOUBLE, coldec, j, 1, 1, NULL, 
		     &dec, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsra, j, 1, 1, NULL, 
		     &sra, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsdec, j, 1, 1, NULL, 
		     &sdec, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colmag, j, 1, 1, NULL, 
		     &mag, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsmag, j, 1, 1, NULL, 
		     &smag, &anynul, &status);
      fits_read_col(fptr, TSHORT, colcatname, j, 1, 1, NULL, 
		     &catalog, &anynul, &status);
      fits_read_col(fptr, TSTRING, colcatid, j, 1, 1, NULL, catid, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsymmetry, j, 1, 1, NULL, &sym, &anynul, &status);
      sprintf(a, "INSERT INTO stars VALUES (nextval('stars_id_sq'), %d,\
%.3f, %.3f, %f, %f, %d, %10f, '%c', %.3f, %.8f, %.8f, %f, %f, %.4f, %f, %f,\
%d, '%s', %.7f)\n", imageid, xt, yt, sxt, syt, area, (double) intensity, saturated, bg, ra, dec, sra, sdec, mag, smag, sym, catalog, catid, jd);
      /*  Write to a temporary file  */

      if (to_database) {
	fprintf(fptemp, "%d, %d, %.3f, %.3f, %f, %f, %d, %10f, '%c', %.3f, \
%.8f, %.8f, %f, %f, %.4f, %f, %f, %d, '%s', %.7f", imageid, xt, yt, sxt, syt, area, (double) intensity, saturated, bg, ra, dec, sra, sdec, mag, smag, sym, catalog, catid, jd);
      }
    }


}







PGresult *
do_query(PGconn *pgconn, char a)

{
  PGresult *pgres;

  pgres = PQexec(pgconn, a);
  if (PQresultStatus(pgres) != PGRES_COMMAND_OK) {
    sprintf(stderr, "Error executing query %s\n", a);
    return NULL;
  }
}









int
main(int argc, char *argv[])
     
{
  int opt;
  FITSFILE f;

  int status = 0;
  int nfound;
  char error[81], comment[81], temp[81];
  int errnum = 0;
  int i, j;
  char a[2000];
  int year, month, day, hour, minute;
  float second;
  time_t t;
  char *insert_date;
  double xc, yc, ac, dc;
  char tname[11];

  PGconn* pgconn;
  PGresult* pgres;

  state.server = NULL;
  state.database = "tass";
  state.port = NULL;
  state.logfile = stdout;
  state.logname = NULL;
  state.version = "1.0";
  state.verbose = 0;
  /*  Check for command line options  */
  while ((opt = getopt (argc, argv, "l:s:d:p:v")) != -1) {
    switch(opt) {
    case 'l':
      /*  Name of the log file  */
      state.logname = strdup(optarg);
      if ((state.logfile = fopen(state.logname,"a") )== NULL) {
	fprintf(stderr, "Can not open log file!\n");
	exit(1);
      }
      break;
    case 's':
      /*  Internet address of the server  */
      state.server = strdup(optarg);
      break;
    case 'd':
      /*  Name of the database  */
      state.database = strdup(optarg);
      break;
    case 'p':
      /*  Port number of the server  */
      state.port = strdup(optarg);
      break;
    case 'v':
      /*  Switch on verbose mode  */
      state.verbose = 1;
      break;
    case '?':
      switch(optopt) {
      default:
	fprintf(stderr, "Unknown option -%c ignored!\n", optopt);
	break;
      }
      break;
    }
  }
  if (argc < optind + 1) {
    printf("fb2pg %s     Jure Skvarc, May 2001\n", state.version);
    printf("Usage:: %s [options] fitsfile\n", argv[0]);
    printf("Options:\n");
    printf("-l filename    name of the log file (default stdout)\n");
    printf("-s server      Internet address of the server\n");
    printf("-d database    database name\n");
    printf("-p port        port number\n");
    printf("-v             verbose mode\n");
    printf("\n");
    exit(1);
  }
  
  /*===================================*/
  /* make a connection to the database */
  /*===================================*/
  pgconn = PQsetdb(state.server, state.port, NULL, NULL, state.database);


  /*  Drop indices of the stars table */
  sprintf(a, "DROP INDEX star_ra");
  pgres = do_query(a);
  if (pgres) PQclear(pgres);
  sprintf(a, "DROP INDEX star_dec");
  pgres = do_query(a);
  if (pgres) PQclear(pgres);
  sprintf(a, "DROP INDEX star_mag");
  pgres = do_query(a);
  if (pgres) PQclear(pgres);
  sprintf(a, "DROP INDEX star_jd");
  pgres = do_query(a);
  if (pgres) PQclear(pgres);
  sprintf(a, "DROP INDEX star_id");
  pgres = do_query(a);
  if (pgres) PQclear(pgres);

  /*  Open temporary file for appending  */
  strcpy(tname, "fb2pgXXXXXX");
  mkstemp(tname);
}
