/*  FIELD_COMPARE  */
/*  Poiscem ujemajoce zvezde iz dveh seznamov */
/*  (c) Jure Skvarc, oktober 1996  */
#define _POSIX_C_SOURCE  2
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <time.h>

#ifdef DO_HIGHER_ORDER
#include <f2c.h>
#endif

#include "../functs.h"

extern STATE state;
extern BLINK_FRAME *frame;
TRANSROT trglob;


#ifdef DO_HIGHER_ORDER
int dgelss_(integer *, integer *, integer *, 
	    doublereal *, integer *, doublereal *, integer *, doublereal *, 
	    doublereal *, integer *, doublereal *, integer *, integer *);

/*  Do a higher order fitting between the catalog and image positions  */
/*  nmatch   number of matched stars   */
/*  list1    list of stars from the image  */
/*  list2    list of stars from the catalog  */
/*  tr       linear transformation between the lists  */
/*  match_options  are what they say  */
int
do_higher_order(int nmatched, STARS *list1, STARS *list2, TRANSROT *tr, MATCH_OPTIONS *match_options)

{
  int i, k, mm;
  int m;
  int i1, i2, j, k1, k2;
  int u, v;
  int n = state.control[state.control_num];
  double x, y, xy, xx, yy;
  doublereal a[21 * 21], b[42], s[21], work[200];
  doublereal rcond = -1;
  integer size = 21, nrhs = 2, rank, info, lwork = 200;
  double w, h, w2, h2;

  w = frame[n].fits.width;
  h = frame[n].fits.height;
  w2 = 0.5 * w;
  h2 = 0.5 * h;
  /* Set all matrix and right hand elements to zero  */
  for (i = 0; i < 21 * 21; i++) a[i] = 0;
  for (i = 0; i < 2 * 21; i++) b[i] = 0;
  frame[n].match.fitorder_used = frame[n].match.fitorder;
  /*  Determine which order is still allowed  */
  /*  Assume that you request two times more data points as constants */
  if (frame[n].match.fitorder >= 2 && nmatched < 12) {
    frame[n].match.fitorder_used = 1;
  }
  else if (frame[n].match.fitorder >= 3 && nmatched < 20) {
    frame[n].match.fitorder_used = 2;
  }
  else if (frame[n].match.fitorder >= 4 && nmatched < 30) {
    frame[n].match.fitorder_used = 3;
  }
  else if (frame[n].match.fitorder == 5 && nmatched < 42) {
    frame[n].match.fitorder_used = 4;
  }
  /*  If the fitorder has fallen back to 1, exit!  */
  /*  if (frame[n].match.fitorder == 1) return 1;*/

  m = frame[n].match.fitorder_used;
  size = 0;
  for (i = 0; i <= m; i++) size += (i + 1);
  /*  Calculate residuals for the linear transformation  */
  i1 = i2 = 0;
  for (k = 0; k < list1->n; k++) {
    mm = list1->s[k].m;
    if (mm != -1) {
      /*  From image coordinates calculate coordinates in the catalog system
	  and subtract catalog coordinates  */
      xx = list2->s[mm].x;
      yy = list2->s[mm].y;
      /* Normalize coordinates  */
      x = (list1->s[k].x - w2) / w;
      y = (list1->s[k].y - h2) / h;
      /*  Now prepare the array and the right side  */
      i1 = 0;  /*  Array index  */
      for (k1 = 0; k1 <= m; k1++) {
	for (j = 0; j <= k1; j++) {
	  i = k1 - j;
	  /* i and j are the x and y powers  */
	  i2 = 0;
	  for (k2 = 0; k2 <= m; k2++) {
	    for (v = 0; v <= k2; v++) {
	      u = k2 - v;
	      a[i1 * size + i2] += pow(x, (double) (i + u)) * 
		pow(y, (double) (j + v));
	      i2++;
	    }
	  }
	  xy = pow(x, i) * pow(y, j);
	  b[i1] += xx * xy;
	  b[i1 + size] += yy * xy; 
	  i1++;
	}
      }
    }
  }
  /*  Solve the system of equations */
  dgelss_(&size, &size, &nrhs, a, &size, 
	  b, &size, s, &rcond, &rank, 
	  work, &lwork, &info);
  if (info != 0) {
    if (state.interactive) {
      fl_show_alert("High order fitting failed!", "", "", 0);
      return 1;
    }
    else {
      fprintf(stderr, "High order fitting failed!!\n");
      return 1;
    }
  }
  /* Copy the coefficients to the array  */
  for (i = 0; i < size; i++) {
    match_options->xcoef[i] = b[i];
    match_options->ycoef[i] = b[i + size];
  }
    
  return 0;
}
#endif


/*  Make a match between two star lists. list1 is from detected stars  */
/*  list 2 is from catalogs  */
int
field_compare(STARS *list1, STARS *list2, MATCH_OPTIONS match_options, 
	      BLINK_FRAME *frame, int n)

{
  int i, j, k;
  time_t t;
  int s1sizetemp, s2sizetemp;
  TRANSROT tr, tr1;
  STAR_PAIR *gs;
  int nmatched;
  double xx, yy, x, y;
  double w, h, w2, h2;
  double d, det;
  float kf;
  double xinc, yinc, xc, yc;
  int mm;
  double x1, y1;
  int err;
  int refit, frefit;
  float res;
  STARS tlist1, tlist2;
  int indmed;

  list1->sp = 0;
  for (i = 0; i < list1->n; i++) {
    list1->sp += log(list1->s[i].intensity);
  }
  list1->sp = 0.9 * exp(list1->sp / i); 
  /*  Constants for fits_pixel_to_world routine  */
  xinc = frame[n].map.width / frame[n].map.w;
  yinc = frame[n].map.height / frame[n].map.h;
  xc = 0.5 * frame[n].map.w;
  yc = 0.5 * frame[n].map.h;

  t = time(NULL);
  fprintf(state.logfp, "==================================================\n");
  fprintf(state.logfp, "Fitsblink %s field_compare %s", VERSION, asctime(gmtime(&t)));
  fprintf(state.logfp, "  - START\n");
  fprintf(state.logfp, "    minimal value to accept constellation: %f\n", match_options.min);
  fprintf(state.logfp, "    approximate ra of image center: %.5f degrees\n", frame[n].ra);
  fprintf(state.logfp, "    approximate dec of image center: %.5f degrees\n", frame[n].dec);
  fprintf(state.logfp, "    Assumed x pixel size: %.3f arcseconds\n", xinc * 3600);
  fprintf(state.logfp, "    Assumed y pixel size: %.3f arcseconds\n", yinc * 3600);
  fprintf(state.logfp, "    number of neighbouring stars in a constellation: %d\n", match_options.nstar);
  fprintf(state.logfp, "    minimal distance between stars in a constellation: %.1f\n", match_options.mindist);
  fprintf(state.logfp, "    Largest allowed residual after matching: %.2f arcseconds\n", match_options.maxres);
  
  fprintf(state.logfp, "    number of stars used for initial matching: %d\n", match_options.firstbright);
  fprintf(state.logfp, "    maximum allowed positional error for a star in constellation: %.1f\n", match_options.poserrmax);
  fprintf(state.logfp, "    maximum allowed magnitude error for a star in constellation: %.1f\n", match_options.magerrmax);
  fprintf(state.logfp, "    extended area: %.2f\n", frame[n].map.extarea);
  fprintf(state.logfp, "    brightest allowed stars from the catalogue: %.1f\n", frame[n].map.minmag);
  fprintf(state.logfp, "    faintest allowed stars from the catalogue: %.1f\n", frame[n].map.maxmag);
  fprintf(state.logfp, "    Number of stars in the first list:%d\n", list1->n);
  fprintf(state.logfp, "    Number of stars in the second list:%d\n", list2->n);
  
  /*  Sort first list on intensities  */
  qsort((void *) list1->s, (size_t) list1->n, sizeof(STAR), compare_starmag);
  s1sizetemp  =  min(match_options.firstbright, list1->n);

  /*  Copy first s1sizetemp items into a temporary array  */
  /*  Reserve space for the temporary array  */
  tlist1.s = (STAR *) myalloc(sizeof(STAR) * s1sizetemp, 
			    "constellations", "tlist1");
  for (i = 0, k = 0; i < s1sizetemp && k < list1->n; i++, k++) {
    do {
      tlist1.s[i] = list1->s[k];
      /*  Check if this star is close to some other star  */
      for (j = i - 1; j >= 0; j--) {
	d = hypot(tlist1.s[j].x - tlist1.s[i].x, tlist1.s[j].y - tlist1.s[i].y);
	/*  This star is too close to some other star, so skip it */
	/*  Also skip the star if it is saturated  */
	if (d < match_options.mindist || tlist1.s[j].intensity< 0) {
	  break;
	}
      }
      if (j >= 0) {
	/*  Try with the next star  */
	k++;
      }
    } while (j >= 0 && k < list1->n);
  }
  s1sizetemp = i;
  /*  Now sort first match_options.firstbright stars on y coordinates  */
  qsort((void *) tlist1.s, (size_t) s1sizetemp, sizeof(STAR), compare_star);
  kf = 3.0 / log(list1->s[0].intensity);
  for (j = 0; j < s1sizetemp; j++) {
    append_marker(frame, n, tlist1.s[j].x, tlist1.s[j].y, 
		  1.5 * kf * log(tlist1.s[j].intensity), strdup(""),
		  FL_BLUE, OBJ_BOX);
  }
  tlist1.n = s1sizetemp;


  /*  Sort second list on magnitudes */
  qsort((void *) list2->s, (size_t) list2->n, sizeof(STAR), compare_starmag);
  s2sizetemp  =  min(fsqr(frame[n].map.extarea + 1) * match_options.firstbright, list2->n);

  /*  Copy first s1sizetemp items into a temporary array  */
  /*  Reserve space for the temporary array  */
  tlist2.s = (STAR *) myalloc(sizeof(STAR) * s2sizetemp, 
			    "constellations", "tlist2");
  for (i = 0, k = 0; i < s2sizetemp && k < list2->n; i++, k++) {
    do {
      tlist2.s[i] = list2->s[k];
      /*  Check if this star is close to some other star  */
      for (j = i - 1; j >= 0; j--) {
	d = hypot(tlist2.s[j].x - tlist2.s[i].x, tlist2.s[j].y - tlist2.s[i].y);
	/*  This star is too close to some other star, so skip it */
	if (d < match_options.mindist) {
	  break;
	}
      }
      if (j >= 0) {
	/*  Try with the next star  */
	k++;
      }
    } while (j >= 0 && k < list2->n);
  }
  s2sizetemp = i;
  tlist2.n = s2sizetemp;

  /*  Now sort first match_options.firstbright stars on y coordinates  */
  qsort((void *) tlist2.s, (size_t) s2sizetemp, sizeof(STAR), compare_star);
  /*  Do constellation matching on bright stars only  */
  if (tlist1.n < tlist2.n) {
    constellations(&tlist1, &tlist2, 0, &tr, match_options);
  }
  else {
    constellations(&tlist2, &tlist1, 0, &tr1, match_options);
    det = tr1.a * tr1.e - tr1.b * tr1.d;
    tr.a = tr1.e / det;
    tr.b = -tr1.b / det;
    tr.c = (tr1.f * tr1.b - tr1.e * tr1.c) / det;
    tr.d = -tr1.d / det;
    tr.e = tr1.a / det;
    tr.f = (tr1.c * tr1.d - tr1.a * tr1.f) / det;
    tr.hi = tr1.hi;
    tr.sma = 0;
    tr.smb = 0;
    tr.mb = 1.0 / tr1.mb;
    tr.ma = 0.0;
  }
  free(tlist2.s);
  free(tlist1.s);
  if (tr.hi < 0) {
    return -1;
  }
  /*  Reset pointer values  */
  for (i = 0; i < list1->n; i++) list1->s[i].m = -1;
  for (i = 0; i < list2->n; i++) list2->s[i].m = -1;
  fprintf(state.logfp, "    Coordinate system transformation:\n");
  fprintf(state.logfp, "       x2 = %f x1 %+f y1 %+f\n", tr.a, tr.b, tr.c);
  fprintf(state.logfp, "       y2 = %f x1 %+f y1 %+f\n", tr.d, tr.e, tr.f);
  fprintf(state.logfp, "       Chi squared / point=%.2f\n", tr.hi);

  fprintf(state.logfp, "    Magnitude transformation:\n");
  fprintf(state.logfp, "       mag2 = %f %+f mag1\n", tr.ma, tr.mb);
  fprintf(state.logfp, "       sigma_a = %f, sigma_b = %f\n", tr.sma, tr.smb);
  /*  Now sort both arrays entirely on y coordinate  */
  qsort((void *) list1->s, (size_t) list1->n, sizeof(STAR), compare_star);
  qsort((void *) list2->s, (size_t) list2->n, sizeof(STAR), compare_star);
  /*  Do constellation matching on all stars */
  constellations(list1, list2, 1, &tr, match_options);
  /*  Check how many stars are successfully matched  */
  nmatched = 0;
  for (i = 0; i < list1->n; i++) {
    if (list1->s[i].m != -1) nmatched++;
  }
  fprintf(state.logfp, "    Number of matched stars:%d\n", nmatched);
  frame[n].match.nmatched = nmatched;
  /*  Reserve space for pairs of stars  */
  gs = (STAR_PAIR *) myalloc(sizeof(STAR_PAIR) * nmatched, "field_compare", "gs");
  frefit = 0;
  /*  Sort first list on magnitudes  */
  qsort((void *) list1->s, (size_t) list1->n, sizeof(STAR), compare_starmag);
  /*  Correct pointers from the second list  */
  for (i = 0; i < list1->n; i++) {
    int mm = list1->s[i].m;
    if (mm != -1) {
      list2->s[mm].m = i;
    }
  }
  do {
    /*  Search only up to the median intensity */
    indmed = list1->n / 2;

    refit = 0;
    /*  Fill the gs list with matched pairs of stars  */
    j = 0;
    for (i = 0; i < indmed && j < match_options.firstbright; i++) {
      int mm = list1->s[i].m;
      if (mm != -1 && 
	  !list1->s[i].saturated && /* Exclude saturated stars  */
	  !list2->s[mm].q /*  Exclude bad stars from the catalog */) {
	gs[j].n1 = i;
	gs[j].x1 = list1->s[i].x;
	gs[j].y1 = list1->s[i].y;
	gs[j].mag1 = list1->s[i].intensity;
	gs[j].n2 = mm;
	gs[j].x2 = list2->s[mm].x;
	gs[j].y2 = list2->s[mm].y;
	gs[j].mag2 = pow(10, -0.4 * list2->s[mm].mag);
	j++;
      }
      else if (list1->s[i].saturated) indmed++;
    }
    nmatched = j;
    /*  Find coordinate transformation  */
    tr.hi = transrot(gs, &nmatched, &tr, &match_options);
    if (tr.hi < 0 || tr.hi > match_options.poserrmax) {
      return -1;
    }
    /*  Mark stars which were used for coordinate fit */
    for (i = 0; i < nmatched; i++) {
      list1->s[gs[i].n1].q = 1;
    }
    /*  Now check if errors are small enough  */
    nmatched = 0;
    for (i = 0; i < list1->n; i++) {
      mm = list1->s[i].m;
      if (mm != -1) {
	/*  Transform from image pixel coordinates to catalog pixel coordinates */
	x1 = tr.a * list1->s[i].x + tr.b * list1->s[i].y + tr.c;
	y1 = tr.d * list1->s[i].x + tr.e * list1->s[i].y + tr.f;
	/*  Transform from pixel coordinates to RA-DEC  */
	err = 0;
	fits_pix_to_world(x1 + 1, y1 + 1, frame[n].map.ra, frame[n].map.dec, 
			  xc, yc, xinc, yinc, 
			  frame[n].map.fi, "-TAN",
			  &list1->s[i].ra, &list1->s[i].dec, &err);
	if (list1->s[i].ra >= 360.0) {
	  list1->s[i].ra -= 360.0;
	}
	/*  Calculate distance between matched stars */
	res = sqrt(fsqr(cos(list1->s[i].dec * DEGRAD) * (list1->s[i].ra - list2->s[mm].ra)) +
		   fsqr(list1->s[i].dec - list2->s[mm].dec)) * 3600.0;
	nmatched++;
	/*  Check if this is too big  */
	if (res > match_options.maxres) {
	  list1->s[i].m = -1;  /*  remove matched pair  */
	  refit = 1;
	}
      }
    }
    if (frefit == 0 && refit == 0) {
      frefit = 1;
      refit = 1;
    }
  } while (refit);


  fprintf(state.logfp, "    Second coordinate system transformation:\n");
  fprintf(state.logfp, "       Number of points:%d\n", nmatched);
  fprintf(state.logfp, "       x2 = %f x1 %+f y1 %+f\n", tr.a, tr.b, tr.c);
  fprintf(state.logfp, "       y2 = %f x1 %+f y1 %+f\n", tr.d, tr.e, tr.f);
  fprintf(state.logfp, "       Chi squared / point=%.2f\n", tr.hi);

  fprintf(state.logfp, "    Magnitude transformation:\n");
  fprintf(state.logfp, "       mag2 = %f %+f mag1\n", tr.ma, tr.mb);
  fprintf(state.logfp, "       sigma_a = %f, sigma_b = %f\n", tr.sma, tr.smb);
  frame[n].match.magfac = log10(1 / tr.mb);
  free(gs);
  /*  Calculate inverse coordinate transformation  */
  det = tr.a * tr.e - tr.b * tr.d;
  tr1.a = tr.e / det;
  tr1.b = -tr.b / det;
  tr1.c = (tr.f * tr.b - tr.e * tr.c) / det;
  tr1.d = -tr.d / det;
  tr1.e = tr.a / det;
  tr1.f = (tr.c * tr.d - tr.a * tr.f) / det;

#ifdef DO_HIGHER_ORDER
  fprintf(state.logfp, "Requested order of fit: %d\n", frame[n].match.fitorder);
  if (do_higher_order(nmatched, list1, list2, &tr, &frame[n].match)) {
    fprintf(state.logfp, "WARNING: High order fitting failed\n");
  } 
  else {
    fprintf(state.logfp, "Performed order of fit: %d\n", frame[n].match.fitorder_used);
  }
  w = frame[n].fits.width;
  h = frame[n].fits.height;
  w2 = 0.5 * w;
  h2 = 0.5 * h;
#endif

  nmatched = 0;
  for (i = 0; i < list1->n; i++) {
    int i1, i2, k1;

    mm = list1->s[i].m;
    /*  Transform from image pixel coordinates to catalog pixel coordinates */
#ifdef DO_HIGHER_ORDER
    x = (list1->s[i].x - w2) / w;
    y = (list1->s[i].y - h2) / h;
    i1 = 0;  /*  Array index  */
    x1 = 0;
    y1 = 0;
    for (k1 = 0; k1 <= frame[n].match.fitorder_used; k1++) {
      for (j = 0; j <= k1; j++) {
	i2 = k1 - j;
	x1 += frame[n].match.xcoef[i1] * pow(x, i2) * pow(y, j);
	y1 += frame[n].match.ycoef[i1] * pow(x, i2) * pow(y, j);
	i1++;
      }
    }
#else
    x1 = tr.a * list1->s[i].x + tr.b * list1->s[i].y + tr.c;
    y1 = tr.d * list1->s[i].x + tr.e * list1->s[i].y + tr.f;
#endif
    if (mm != -1) {
      nmatched++;
      x = tr1.a * list2->s[mm].x + tr1.b * list2->s[mm].y + tr1.c;
      y = tr1.d * list2->s[mm].x + tr1.e * list2->s[mm].y + tr1.f;
      xx = x1 - list2->s[mm].x;
      yy = y1 - list2->s[mm].y;
      list2->s[mm].x = x;
      list2->s[mm].y = y;
      /*  Transform from pixel coordinates to RA-DEC  */
      err = 0;
      fits_pix_to_world(x1 + 1, y1 + 1, frame[n].map.ra, frame[n].map.dec, 
			xc, yc, xinc, yinc, 
			frame[n].map.fi, "-TAN",
			&list1->s[i].ra, &list1->s[i].dec, &err);
      if (list1->s[i].ra >= 360.0) {
	list1->s[i].ra -= 360.0;
      }
      /*  Calculate distance between stars  */
      list1->s[i].err = list2->s[mm].err =
	sqrt(fsqr(cos(list1->s[i].dec * DEGRAD) * (list1->s[i].ra - list2->s[mm].ra)) +
	     fsqr(list1->s[i].dec - list2->s[mm].dec)) * 3600.0;
      list1->s[i].catalog = list2->s[mm].catalog;
      list1->s[i].catnum = list2->s[mm].catnum;
      strcpy(list1->s[i].catid, list2->s[mm].catid);
    }
    else {
      /*  Transform from pixel coordinates to RA-DEC  */
      err = 0;
      fits_pix_to_world(x1 + 1, y1 + 1, frame[n].map.ra, frame[n].map.dec, 
			xc, yc, xinc, yinc, 
			frame[n].map.fi, "-TAN",
			&list1->s[i].ra, &list1->s[i].dec, &err);
      if (list1->s[i].ra >= 360.0) {
	list1->s[i].ra -= 360.0;
      }      
      list1->s[i].catalog = '0';
      list1->s[i].catnum = 0;
    }
  }
  fprintf(state.logfp, "    RESULTS:\n");
  fprintf(state.logfp, "      Number of successfully matched stars:%d\n", nmatched);

  t = time(NULL);
  fprintf(state.logfp, "Fitsblink %s field_compare %s - END\n", VERSION, asctime(gmtime(&t)));
  fprintf(state.logfp, "++++++++++++++++++++++++++++++++++++++++++++++++++\n");

  if (state.interactive) {
    newwcs(frame, n, tr);
  }
  trglob = tr;
  return nmatched;
}


