/*  imageacc.c  for Fitsblink 1.2  */
/*  Functions for access and modification of image elements  */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <forms.h>
#include "formblink.h"
#include "functs.h"

extern FITS_IMAGE count_options;
extern STATE state;

/*   Set basic image parameters */
void
set_image_parameters(FITS_IMAGE *slika, int xmax, int ymax, PIXEL *image)
     
{
  int i;

  slika->width = xmax;
  slika->height = ymax;
  slika->x1 = 1;
  slika->y1 = 1;
  slika->x2 = slika->width - 2;
  slika->y2 = slika->height - 2;
  slika->image = image;
  slika->pi = (PIXEL **) myalloc(ymax * sizeof(PIXEL *), 
				 "set_image_parameters", "pi");
  /*  Set array of pointers to rows */
  for (i = 0; i < ymax; i++) {
    slika->pi[i] = image + i * xmax;
  }    
  /*slika->maxsize = DEFAULT_MAXSIZE;
  slika->sg_num = DEFAULT_SGNUM;
  slika->minbright = DEFAULT_MINBRIGHT;
  slika->minstar = DEFAULT_MINSTAR;
  slika->border = DEFAULT_BORDER;
  slika->satlevel = DEFAULT_SATLEVEL;
  slika->resolve = DEFAULT_RESOLVE; */
  if (count_options.maxsize != -999) {
    slika->maxsize = count_options.maxsize;
  }
  if (count_options.sg_num != -999) {
    slika->sg_num = count_options.sg_num;
  }
  if (count_options.minstar != -999) {
    slika->minstar = count_options.minstar;
  }
  if (count_options.minbright != -999) {
    slika->minbright = count_options.minbright;
  }
  if (count_options.fitmeasure != -999) {
    slika->fitmeasure = count_options.fitmeasure;
  }
  if (count_options.border != -999) {
    slika->border = count_options.border;
  }
  if (count_options.satlevel != -999) {
    slika->satlevel = count_options.satlevel;
  }
  if (count_options.inner != -999) {
    slika->inner = count_options.inner;
  }
  if (count_options.outer != -999) {
    slika->outer = count_options.outer;
  }
  if (count_options.resolve != -999) {
    slika->resolve = count_options.resolve;
  }
  if (count_options.aperture != -999) {
    slika->aperture = count_options.aperture;
  }
}


/*  Create a new image with dimensions and other parameters of the image */
/*  given as a template */
int
create_image_from_template(BLINK_FRAME *frame, int m, int ref, char *name)

{
  int n;
  int xmax, ymax;
  FITS_IMAGE *new, *old;
  PIXEL *image;
  double xpix, ypix, x, y;
  double ra, dec;
  int i, j;
  int low, high, g, wh;
  float dg;

  next_image();
  n = state.frame_used - 1;
  frame[n].fits.name = strdup(name);
  frame[n].name = frame[n].fits.name;
  frame[n].imagelist.s = NULL;
  frame[n].imagelist.n = 0;
  frame[n].cataloglist.s = NULL;
  frame[n].cataloglist.n = 0;
  frame[n].detect = 0;
  frame[n].matched = frame[n].fitsfile.matched;
  frame[n].match.magfac = 0;
  frame[n].map.minmag = DEFAULT_MINMAG;
  frame[n].map.maxmag = DEFAULT_MAXMAG;
  frame[n].map.extarea = MAP_EXTAREA;
  /*  Enable the Set parameters option in the astrometry menu  */
  if (state.interactive) {
    fl_set_menu_item_mode(state.blinker->astrometryW, EDIT_PARAMETERS, FL_PUP_NONE);
    fl_set_menu_item_mode(state.blinker->astrometryW, BLUR_IMAGE, FL_PUP_NONE);
    fl_set_menu_item_mode(state.blinker->astrometryW, LOAD_STAR_LIST, FL_PUP_NONE);
  }
  /*  Copy WCS values from the template image */
  frame[n].fitsfile.crval1 = frame[m].fitsfile.crval1; 
  frame[n].fitsfile.crval2 = frame[m].fitsfile.crval2;
  frame[n].fitsfile.crpix1 = frame[m].fitsfile.crpix1 - (frame[m].fits.x1 - 1);
  frame[n].fitsfile.crpix2 = frame[m].fitsfile.crpix2 - (frame[m].fits.y1 - 1);
  frame[n].fitsfile.cdelt1 = frame[m].fitsfile.cdelt1;
  frame[n].fitsfile.cdelt2 = frame[m].fitsfile.cdelt2;
  frame[n].fitsfile.crot = frame[m].fitsfile.crot;
  strcpy(frame[n].fitsfile.ctype, frame[m].fitsfile.ctype);
  frame[n].fitsfile.simple = frame[m].fitsfile.simple;
  frame[n].fitsfile.bitpix = frame[m].fitsfile.bitpix;
  frame[n].fitsfile.naxis = frame[m].fitsfile.naxis;
  frame[n].fitsfile.wcs = frame[m].fitsfile.wcs;
  frame[n].fitsfile.matched = frame[m].fitsfile.matched;
  /*  Width and height of the image  */
  frame[n].fits.width = xmax = frame[m].fits.x2 - frame[m].fits.x1 + 3;
  frame[n].fits.height = ymax = frame[m].fits.y2 - frame[m].fits.y1 + 3;
  frame[n].fits.x1 = 1;
  frame[n].fits.y1 = 1;
  frame[n].fits.x2 = xmax - 1;
  frame[n].fits.y2 = ymax - 1;
  frame[n].fitsfile.nax[0] = xmax;
  frame[n].fitsfile.nax[1] = ymax;
  if (frame[n].fitsfile.wcs) {
    int status = 0;
    fits_pix_to_world(0.5 * xmax, 0.5 * ymax,
	     frame[n].fitsfile.crval1, frame[n].fitsfile.crval2,
	     frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2,
	     frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2,
	     frame[n].fitsfile.crot,
	     frame[n].fitsfile.ctype,
	     &(frame[n].ra), &(frame[n].dec), &status);
    if (frame[n].ra >= 360.0) {
      frame[n].ra -= 360.0;
    }
    frame[n].fi = frame[n].fitsfile.crot;
    frame[n].xsize = fabs(frame[n].fitsfile.cdelt1 * 3600.0);
    frame[n].ysize = fabs(frame[n].fitsfile.cdelt2 * 3600.0);
  }
  else {
    frame[n].ra = 0.0;
    frame[n].dec = 0.0;
    frame[n].fi = 0.0;
    frame[n].xsize = 1.92;
    frame[n].ysize = 1.92;
  }
  if (state.interactive) {
    fl_set_menu_item_mode(state.blinker->astrometryW, DETECT_STARS, FL_PUP_NONE);
    fl_set_menu_item_mode(state.blinker->astrometryW, OPTIONS, FL_PUP_NONE);
  }
  /*  If possible, calculate time and date of mid-exposure  */
  calculate_midexposure(&frame[n].fitsfile, &frame[n].year, &frame[n].month,
			&frame[n].day, &frame[n].hour, 
			&frame[n].minute, &frame[n].second);
  /*  Also calculate the Julian day  */
  frame[n].fitsfile.jd = julian_day(frame[n].year, frame[n].month, frame[n].day + 
				(frame[n].hour + 
				 (frame[n].minute + frame[n].second / 60.0) 
				 / 60.0) / 24.0);
  /*  Set default options for star matching  */
  set_match_defaults(frame, n);
  image = (PIXEL *) myalloc(sizeof(PIXEL) * xmax * ymax, "create_image_from_template", "image");
  set_image_parameters(&frame[n].fits, xmax, ymax, image);
  /* awx = state.blinker->zoom1W.x / state.zoom_width;
  awy = state.blinker->zoom1W.y / state.zoom_width;
  frame[n].astrwin.image = (PIXEL *) myalloc(sizeof(PIXEL) * awx * awy, 
					     "load_frame", "astrwin");
  set_image_parameters(&frame[n].astrwin, awx, awy,
  state.blinker->zoom1W.y / state.zoom_width, );  */
  /*  Translation and rotation of the image  */
  frame[n].dx = frame[n].dy = frame[n].alfa = 0.0;

  new = &frame[n].fits;
  old = &frame[ref].fits;
  /*  Copy the content of reference (not template!) image into this image */
  for (x = 0; x < xmax; x++) {
    int status = 0;
    for (y = 0; y < ymax; y++) {
      /*  Calculate celestial coordinates of the pixel x, y  */
      fits_pix_to_world(x, y,
			frame[n].fitsfile.crval1, frame[n].fitsfile.crval2,
			frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2,
			frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2,
			frame[n].fitsfile.crot,
			frame[n].fitsfile.ctype,
			&ra, &dec, &status);
      fits_world_to_pix(ra, dec, 
			frame[ref].fitsfile.crval1, frame[ref].fitsfile.crval2,
			frame[ref].fitsfile.crpix1, frame[ref].fitsfile.crpix2,
			frame[ref].fitsfile.cdelt1, frame[ref].fitsfile.cdelt2,
			frame[ref].fitsfile.crot,
			frame[ref].fitsfile.ctype,
			&xpix, &ypix, &status);
      set_point(new, (int) x, (int) y, video_point(old, (int) xpix, (int) ypix));
    }
    fits_report_error(stderr, status);
  }
  /*===========================================*/
  /*  Copy FITS header from the template file  */
  /*===========================================*/
  /*  reserve space for the header  */
  frame[n].fitsfile.nkeys = frame[ref].fitsfile.nkeys;
  frame[n].fitsfile.key = (char **) myalloc(sizeof(char *) * 
					     frame[n].fitsfile.nkeys,
					     "read_fits", "f->key");
  /*  Read all header lines  */
  for (i = 0; i < frame[n].fitsfile.nkeys; i++) {
    /* Reserve space for a line  */
    frame[n].fitsfile.key[i] = (char *) myalloc(sizeof(char *) * FLEN_CARD, 
					"create_image_from_template", 
					"frame[n].fitsfile.key[i]");
    strcpy(frame[n].fitsfile.key[i], frame[ref].fitsfile.key[i]);
  }
  if (state.interactive) {
    /*  Create Pixmap for image  */
    frame[n].pixmap = 
      XCreatePixmap(state.disp, 
		    FL_ObjWin(state.blinker->blinkW), 
		    state.bw, state.bh, 
		    fl_state[fl_get_form_vclass(state.blinker->blinker)].depth);

    /*  Find the lowest and the highest grey level  */
    low = 65536;
    high = -65536;
    for (i = 0; i < xmax * ymax; i++) {
      g = image[i];
      if (low > g) low = g;
      if (high < g) high = g;
    }
    frame[n].xmin = frame[n].low = low;
    frame[n].xmax = frame[n].high = high;
    /*  Set number of points for grey level correction  */
    frame[n].ngam = NGAMMA;
    frame[n].gamx = myalloc(frame[n].ngam * sizeof(float), 
			    "load_frame", "frame[n].gamx");
    frame[n].gamy = myalloc(frame[n].ngam * sizeof(float), 
			    "load_frame", "frame[n].gamy");
    frame[n].kg = myalloc(frame[n].ngam * sizeof(float), 
			  "load_frame", "frame[n].kg");
    /*  Width of the histogram box  */
    frame[n].nhisto = wh = HISTOGRAM_WIDTH;
    /* reserve space for histogram  */
    frame[n].histx = myalloc(wh * sizeof(float), 
			     "load_frame", "frame[n].histx");
    frame[n].histy = myalloc(wh * sizeof(float), 
			     "load_frame", "frame[n].histy");
    for (i = 0; i < wh; i++) {
      frame[n].histx[i] = i / (double) (wh - 1) * (high - low);
      frame[n].histy[i] = 0;
    }
    /*  Calculate histogram  */
    dg = (high - low) / (wh - 1.0);
    for (i = 0; i < xmax * ymax; i++) {
      j = (image[i] - low) / dg;
      frame[n].histy[j]++;
    }
    for (i = 0; i < wh; i++) {
      frame[n].histy[i] = log(frame[n].histy[i] + 1);
    }
    /*  Number of markers  */
    frame[n].objnum = 0;
    frame[n].objshow = 1;
    /*  Make automatic grey level correction  */
    autolevels(n);
    state.control[state.control_num] = n;
    set_image(frame, n);
    reserve_marker_space(frame, n, OBJECT_NUM);
    show_frame_info(frame, n, state.control_num);
  }
  return state.frame_used - 1;
  
}



/*   Return value of pixel at coordinates (x, y)   */
PIXEL
video_point(FITS_IMAGE *slika, int x, int y)
    
{
  if (x >= 0 && x < slika->width &&
      y >= 0 && y < slika->height)
    return *(slika->pi[y] + x);
  else return 0;
}


/*   Set pixel with coordinates x, y to value  v  */
void
set_point(FITS_IMAGE *slika, int x, int y, PIXEL v)
{
  PIXEL *p;
  if (x >= 0 && x < slika->width &&
      y >= 0 && y < slika->height) {
    p = slika->pi[y] + x;
    *p = v;
  }
}

void
xor_point(FITS_IMAGE *slika, int x, int y)
{
  PIXEL *p;
  if (x >= 0 && x < slika->width &&
      y >= 0 && y < slika->height) {
    p = slika->pi[y] + x;
    *p ^= 65535;
  }
}




/*  Make a convolution in point (x, y) with a function, defined in structure g  */
float
convolve(GAUSSF *g, FITS_IMAGE *sl, int x, int y)

{
  int i, j, i1, j1;
  int f2n = g->n / 2;
  float s = 0;
  int v, vall = 0;

  for (i1 = 0, i = x - f2n; i1 < g->n; i1++, i++) {
    if (i > sl->x1 && i < sl->x2) {
      for (j1 = 0, j = y - f2n; j1 < g->n; j1++, j++) {
	if (j > sl->y1 && j < sl->y2) {
	  v = video_point(sl, i, j);
	  s += g->g[i1][j1] * v;
	  vall += v;
	}
      }
    }
  }
  return s / vall;
}

/*  Make a simple filtering of the image  */
void
blur(FITS_IMAGE *sl) 

{
  int x, y;
  PIXEL *row, *row1;

  row = myalloc(sizeof(PIXEL) * sl->width, "blur", "row");
  row1 = myalloc(sizeof(PIXEL) * sl->width, "blur", "row1");
  /*  Copy the first row */
  for (x = 0; x < sl->width; x++) row[x] = video_point(sl, x, 0);
  for (y = 1; y < sl->height - 1; y++) {
    row1[0] = video_point(sl, 0, y);
    for (x = 1; x < sl->width - 1; x++) {
      row1[x] = video_point(sl, x, y);
      set_point(sl, x, y, (row1[x - 1] + video_point(sl, x + 1, y) +
			   row[x] + video_point(sl, x, y + 1)) / 4);
    }
    row1[sl->width - 1] = video_point(sl, sl->width - 1, y);
    for (x = 0; x < sl->width; x++) row[x] = row1[x];
  }
  free(row1);
  free(row);
}





/*  Calculate the coefficients of the Gaussian filter  */
void
set_gaussian_filter(GAUSSF *f, float sigma, float r)

{
  float x, y, x2, y2, r2, s2;
  int i, j;
  int fn2;

  f->n = 2 * (r + 1) + 1;  /*  Array size  */
  f->r = r;              /*  Radius inside which the coefficients are valid */
  f->sigma = sigma;      /*  Half-width of the Gaussian  */
  /*  Reserve space for the coefficient array  */
  f->g = myalloc(sizeof(float *) * f->n, "set_gaussian_filter", "f->g");
  for (i = 0; i < f->n; i++) {
    f->g[i] = myalloc(sizeof(float) * f->n, "set_gaussian_filter", "f->g[i]");
  }
  fn2 = f->n / 2;
  r2 = r * r;
  s2 = sigma * sigma;
  for (i = 0; i < f->n; i++) {
    x = (i - fn2);
    x2 = x * x;
    for (j = 0; j < f->n; j++) {
      y = (j - fn2);
      y2 = y * y;
      if (x2 + y2 <= r2) {
	/*  Laplace of the gaussian  */
	f->g[i][j] = (x2 + y2 - 2 * s2) / (s2 * s2) * exp(-(x2 + y2) / (2 * s2));
      }
      else {
	f->g[i][j] = 0;
      }
    }
  }
}


