/*  Main.c for fitsblink */
/*  Jure Skvarc          */
/*  User interface       */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include <forms.h>
#include <fitsio.h>

#include "formblink.h"
#include "functs.h"
#include "consts.h"

extern STATE state;
extern BLINK_FRAME *frame;
extern OBSERVATORY observatory;

extern char *optarg;
extern int optind, opterr, optopt;


/*  Show offsets for the left image  */
void
show_left_offsets(BLINK_FRAME *frame, int n)

{
  char temp[64];

  sprintf(temp, "%.2f", frame[n].dx);
  fl_set_input(state.gui->firstdeltaxW, temp);
  sprintf(temp, "%.2f", frame[n].dy);
  fl_set_input(state.gui->firstdeltayW, temp);
}

/*  Show offsets for the right image  */
void
show_right_offsets(BLINK_FRAME *frame, int n)

{
  char temp[64];

  sprintf(temp, "%.2f", frame[n].dx);
  fl_set_input(state.gui->seconddeltaxW, temp);
  sprintf(temp, "%.2f", frame[n].dy);
  fl_set_input(state.gui->seconddeltayW, temp);
}

/*  Show information about frame n in the control c  */
void
show_frame_info(BLINK_FRAME *frame, int n, int c)

{
  switch (c) {
  case 0:
    fl_set_choice(state.gui->firstnameW, n + 1);
    fl_set_xyplot_data(state.gui->firstgreyW, 
		       frame[n].histx, frame[n].histy, frame[n].nhisto, 
		       "", "", "");
    fl_set_xyplot_data(state.gui->firstgammaW, 
		       frame[n].gamx, frame[n].gamy, frame[n].ngam, 
		       "", "", "");
    fl_set_xyplot_xbounds(state.gui->firstgammaW, frame[n].xmin, frame[n].xmax);
    fl_set_xyplot_xbounds(state.gui->firstgreyW, frame[n].xmin, frame[n].xmax);
    show_left_offsets(frame, n);
    if (state.control_num == 0) {
      draw_image(frame, n);
      show_pan(frame, n);
    }
    break;
  case 1:
    fl_set_choice(state.gui->secondnameW, n + 1);
    fl_set_xyplot_data(state.gui->secondgreyW, 
		       frame[n].histx, frame[n].histy, frame[n].nhisto, 
		       "", "", "");
    fl_set_xyplot_data(state.gui->secondgammaW, 
		       frame[n].gamx, frame[n].gamy, frame[n].ngam, 
		       "", "", "");
    fl_set_xyplot_xbounds(state.gui->secondgammaW, frame[n].xmin, frame[n].xmax);
    fl_set_xyplot_xbounds(state.gui->secondgreyW, frame[n].xmin, frame[n].xmax);
    show_right_offsets(frame, n);
    if (state.control_num == 1) {
      draw_image(frame, n);
      show_pan(frame, n);
    }
    break;
  }
  fl_set_menu_item_mode(state.blinker->astrometryW, MATCH_STARS, (frame[n].detect) ? FL_PUP_NONE : FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, SAVE_STAR_LIST, (frame[n].detect) ? FL_PUP_NONE : FL_PUP_GREY);
  fl_set_menu_item_mode(state.blinker->astrometryW, DO_ASTROMETRY, (frame[n].matched) ? FL_PUP_NONE : FL_PUP_GREY);
  if (state.parameters && fl_form_is_visible(state.parameters->Parameters)) {
    edit_parameters(state.blinker->astrometryW, EDIT_PARAMETERS);
  }
}

/*************************************/
/*  Callbacks for the control window */
/*************************************/
void
turnimageC(FL_OBJECT *obj, long val)

{
  if (state.frame_used > 0) {
    int n = state.control[val];
    state.control_num = val;
    draw_image(frame, n);
    show_pan(frame, n);
    if (state.parameters && fl_form_is_visible(state.parameters->Parameters)) {
      edit_parameters(state.blinker->astrometryW, EDIT_PARAMETERS);
    }
  }
}


/*  Load a FITS image, calculate a histogram and set some parameters in a
 control  */
void 
load(int leftright)

{
  const char *fn;
  static char *curdir = "";
  static char *pattern = "*.[fF]*[tT][sS]*";
  static char *name = "";
  int n;

  fl_use_fselector(1);
  fl_invalidate_fselector_cache();
  fn = fl_show_fselector("Choose input file", curdir, pattern, name);
  if (fn == NULL) return;
  /*  Save the current directory  */
  curdir = (char *) fl_get_directory();
  /*  Save the current pattern */
  pattern = (char *) fl_get_pattern();
  /*  Get index of the current image  */
  n = state.frame_used;
  /*  Release space for the previous name  */
  if (frame[n].name) free(frame[n].name);

  /*  Load the image into the structure */
  load_image((char *)fn, leftright);
}


/* Save an image */
void 
save(int leftright)

{
  const char *fn;
  static char *curdir = "";
  static char *pattern = "*.[fF]*[tT][sS]*";
  static char *name = "";
  int errnum;
  char *errors[ERROR_MAX];
  int n = state.control[leftright];

  fl_use_fselector(1);
  fl_invalidate_fselector_cache();
  fn = fl_show_fselector("Choose name for the output file", curdir, pattern, name);
  if (fn == NULL) return;
  /*  Save the current directory  */
  curdir = (char *) fl_get_directory();
  /*  Save the current pattern */
  pattern = (char *) fl_get_pattern();
  if (frame[n].name) free(frame[n].name);
  frame[n].name = strdup(fn);
  frame[n].fits.name = frame[n].fitsfile.name = frame[n].name;
  errnum = write_fits(frame, n, errors);
  if (errnum != 0) {
    show_errors(errnum, errors);
  }
}



void
eraseC(FL_OBJECT *obj, long val)

{
  erase_image(state.control[val]);
}

/*  Change control to another image  */
void
nameC(FL_OBJECT *obj, long val)

{
  if (state.frame_used > 0) {
    state.control[val] = fl_get_choice(obj) - 1;
    show_frame_info(frame, state.control[val], val);
  }
}

/******************************************/
/*  Show the contents of the FITS header  */
/******************************************/
void
fitsC(FL_OBJECT *obj, long val)

{
  int i;
  int n = state.control[val];

  if (state.frame_used == 0) return;
  if (state.browser[val] == NULL) {
    state.browser[val] = create_form_Edit();
    fl_set_browser_fontstyle(state.browser[val]->browseW, FL_FIXED_STYLE);
  }
  /*  Deactivate FITS button */
  fl_deactivate_object(obj);
  /*  Show the form  */
  fl_show_form(state.browser[val]->Edit, FL_PLACE_FREE_CENTER, FL_FULLBORDER, frame[n].name);
  fl_freeze_form(state.browser[val]->browseW->form);
  fl_clear_browser(state.browser[val]->browseW);
  for (i = 0; i < frame[n].fitsfile.nkeys; i++) {
    fl_addto_browser(state.browser[val]->browseW, frame[n].fitsfile.key[i]);
  }
  fl_set_browser_topline(state.browser[val]->browseW, 1);
  fl_unfreeze_form(state.browser[val]->browseW->form);
  state.b[val] = 1;
}  


/***********************/
/*  Histogram          */
/***********************/

/*  This should probably remain empty */
void
greyC(FL_OBJECT *obj, long val)

{
}

void
emptyC(FL_OBJECT *obj, long val)

{
}

/**************************/
/* Grey level correction  */
/**************************/
void
gammaC(FL_OBJECT *obj, long val)

{
  float x, y;
  int i;
  int n, m;

  /*  Get the value which has changed  */
  fl_get_xyplot(obj, &x, &y, &i);
  if (i >= 0) {
    n = state.control[val];
    frame[n].gamx[i] = x;
    frame[n].gamy[i] = y;
    if (i > 0) {
      frame[n].kg[i] = (frame[n].gamy[i] - frame[n].gamy[i - 1]) / 
	(frame[n].gamx[i] - frame[n].gamx[i - 1]);
    }
    if (i < frame[n].ngam - 1) {
      frame[n].kg[i + 1] = (frame[n].gamy[i + 1] - frame[n].gamy[i]) / 
	(frame[n].gamx[i + 1] - frame[n].gamx[i]);
    }
    m = state.control[1 - val];
    if (n == m) {
      switch (val) {
      case 0:
	fl_set_xyplot_data(state.gui->secondgammaW, 
		       frame[m].gamx, frame[m].gamy, frame[m].ngam, 
		       "", "", "");
	break;
      case 1:
	fl_set_xyplot_data(state.gui->firstgammaW, 
		       frame[m].gamx, frame[m].gamy, frame[m].ngam, 
		       "", "", "");
	break;
      }
    }
    set_image(frame, n);
    if (state.control_num == val || state.control[0] == state.control[1]) {
      draw_image(frame, n);
    }
  }
}

void
histadjC(FL_OBJECT *obj, long val)

{
  float gmin, gmax;
  int c, step  = 0;
  
  c = fl_mouse_button();
  switch(c) {
  case FL_LEFT_MOUSE:
    step = 100;
    break;
  case FL_MIDDLE_MOUSE:
    step = 1000;
    break;
  case FL_RIGHT_MOUSE:
    step = 10000;
    break;
  }
  switch (val) {
  case 1:
    fl_get_xyplot_xbounds(state.gui->firstgammaW, &gmin, &gmax);
    gmin += step;
    break;
  case 2:
    fl_get_xyplot_xbounds(state.gui->firstgammaW, &gmin, &gmax);
    gmin -= step;
    break;
  case 3:
    fl_get_xyplot_xbounds(state.gui->firstgammaW, &gmin, &gmax);
    gmax -= step;
    break;
  case 4:
    fl_get_xyplot_xbounds(state.gui->firstgammaW, &gmin, &gmax);
    gmax += step;
    break;
  case 5:
    fl_get_xyplot_xbounds(state.gui->secondgammaW, &gmin, &gmax);
    gmin += step;
    break;
  case 6:
    fl_get_xyplot_xbounds(state.gui->secondgammaW, &gmin, &gmax);
    gmin -= step;
    break;
  case 7:
    fl_get_xyplot_xbounds(state.gui->secondgammaW, &gmin, &gmax);
    gmax -= step;
    break;
  case 8:
    fl_get_xyplot_xbounds(state.gui->secondgammaW, &gmin, &gmax);
    gmax += step;
    break;
  }
  if (val < 5 || state.control[0] == state.control[1]) {
    fl_set_xyplot_xbounds(state.gui->firstgammaW, gmin, gmax);
    fl_set_xyplot_xbounds(state.gui->firstgreyW, gmin, gmax);
    frame[state.control[0]].xmin = gmin;
    frame[state.control[0]].xmax = gmax;
  }
  if (val > 4 || state.control[0] == state.control[1]) {
    frame[state.control[1]].xmin = gmin;
    frame[state.control[1]].xmax = gmax;
    fl_set_xyplot_xbounds(state.gui->secondgammaW, gmin, gmax);
    fl_set_xyplot_xbounds(state.gui->secondgreyW, gmin, gmax);
  }
}


int
cmppixel (const void *a1, const void *b1)

{
  int t = (int) *((PIXEL *) a1) - (int) *((PIXEL *) b1);
  if (t > 0)
    return 1;
  else if (t < 0)
    return -1;
  else
    return 0;
}

void
invert_display(BLINK_FRAME *frame, int n)

{
  int i;
  for (i = 0; i < frame[n].ngam; i++) {
    frame[n].gamy[i] = state.grey_max - frame[n].gamy[i];
    if (i > 0) 
      frame[n].kg[i] = (frame[n].gamy[i] - frame[n].gamy[i - 1]) / 
	(frame[n].gamx[i] - frame[n].gamx[i - 1]);
  }
}

/*===========================================================*/
/*  Do automatic adjustment of grey levels for the image n   */
/*===========================================================*/
void
autolevels(int n)

{
  int i, l, p, ng;
#if 1
  int j, m;
  double min, max, d;
#endif
  PIXEL *temp, *p1, *p2;
  int step;

  /*  Reserve space for a temporary image   */
  l = 50000;
  step = (frame[n].fits.x2 - frame[n].fits.x1 + 3) * (frame[n].fits.y2 - frame[n].fits.y1 + 3) / l;
  if (step == 0) {
    step = 1;
    l = (frame[n].fits.x2 - frame[n].fits.x1 + 3) * (frame[n].fits.y2 - frame[n].fits.y1 + 3);
  }
  temp = (PIXEL *) myalloc(sizeof(PIXEL) * l, "autoC", "temp");
  /*  Copy the image  */
  p1 = frame[n].fits.image;
  p2 = temp;
  for (i = 0; i < l; i++) {
    *p2 = *p1;
    p1 += step;
    p2++;
  }


  /*  Sort values */
  qsort(temp, l, sizeof(PIXEL), cmppixel);
  ng = frame[n].ngam; 
  p = ng / 2 - 1;
#if 0
  frame[n].gamx[p] = temp[(int) (0.03 * l)];
  frame[n].gamy[p] = 0;
  frame[n].gamx[p + 1] = temp[(int) ((1.0 - state.autolevel_high) * l)];
  frame[n].gamy[p + 1] = state.grey_max;
  for(i = 0; i < p; i++) {
    frame[n].gamx[i] = i * (frame[n].gamx[p] - frame[n].low) / p;
    frame[n].gamy[i] = 0;
  }
  p += 2;
  for(i = p; i < ng; i++) {
    frame[n].gamx[i] = frame[n].gamx[p - 1] + (i + 1 - p) * 
      (frame[n].high - frame[n].gamx[p - 1]) / (ng - p);
    frame[n].gamy[i] = state.grey_max;
  }
#else
  /*  Median value  */
  m = temp[l / 2];
  d = 0;
  j = 0;
  for (i = 0; i < l / 2; i++) {
    min = (temp[i] - m);
    d += min * min;
    j++;
  }
  min = sqrt(d / j); 
  d = 0;
  j = 0;
  for (i = l / 2; i < l; i++) {
    max = (temp[i] - m);
    d += max * max;
    j++;
  }
  max = sqrt(d / j); 
  frame[n].gamx[p - 1] = m - 3 * min;
  frame[n].gamy[p - 1] = 0;
  frame[n].gamx[p] = m;
  frame[n].gamy[p] = 0.2 * state.grey_max;
  frame[n].gamx[p + 1] = m + 10 * min;
  frame[n].gamy[p + 1] = state.grey_max;
  for(i = 0; i < p - 1; i++) {
    frame[n].gamx[i] = i * (frame[n].gamx[p - 1] - frame[n].low) / p;
    frame[n].gamy[i] = 0;
  }
  p += 2;
  for(i = p; i < ng; i++) {
    frame[n].gamx[i] = frame[n].gamx[p - 1] + (i + 1 - p) * 
      (frame[n].high - frame[n].gamx[p - 1]) / (ng - p);
    frame[n].gamy[i] = state.grey_max;
  }
#endif
  for (i = 1; i < ng; i++) {
    frame[n].kg[i] = (frame[n].gamy[i] - frame[n].gamy[i - 1]) / 
      (frame[n].gamx[i] - frame[n].gamx[i - 1]);
  }
  free(temp);
  /*  frame[n].xmin = frame[n].gamx[p - 2] * 0.8;
      frame[n].xmax = frame[n].gamx[p  - 1] * 1.2; */

  if (state.inverse) invert_display(frame, n);
}

void
autoC(FL_OBJECT *obj, long val)

{
  if (state.frame_used > 0) {
    int n = state.control[val];
    
    fl_set_oneliner_font(0, 20); 
    fl_show_oneliner("Adjusting grey levels", 100, 100);
    XFlush(state.disp);
    autolevels(n);
    fl_hide_oneliner();
    set_image(frame, n);
    show_frame_info(frame, n, val);
    if (state.control[0] == state.control[1]) {
      show_frame_info(frame, n, 1 - val);
    }
  }
}

/*  Invert the grey level transformation  */
void
invertC(FL_OBJECT *obj, long val)

{
  if (state.frame_used > 0) {
    int n = state.control[val];
    invert_display(frame, n);
    set_image(frame, n);
    show_frame_info(frame, n, val);
    if (state.control[0] == state.control[1]) {
      show_frame_info(frame, n, 1 - val);
    }
  }
}

/**************************************/
/*   Image translation and rotation  */
/**************************************/
void
deltaxC(FL_OBJECT *obj, long val)

{
  if (state.frame_used > 0) {
    int n = state.control[val];
    frame[n].dx = strtod(fl_get_input(obj), NULL);
    /*frame[n].x1 = dx;*/
    show_left_offsets(frame, n);
    show_right_offsets(frame, n);
    set_image(frame, n);
    if (state.control_num == val) {
      draw_image(frame, n);
    }
  }
}

void
deltayC(FL_OBJECT *obj, long val)

{
  if (state.frame_used > 0) {
    int n = state.control[val];
    frame[n].dy = strtod(fl_get_input(obj), NULL);
    show_left_offsets(frame, n);
    show_right_offsets(frame, n);
    set_image(frame, n);
    if (state.control_num == val) {
      draw_image(frame, n);
    }
  }
}

/***********************/
/*  Ending the program */
/***********************/

void
quitC(FL_OBJECT *obj, long val)

{
  if (fl_show_question("Do you really want to quit?", 1)) {
    fl_finish();
    exit(0);
  }
}


void
helpC(FL_OBJECT *obj, long val)

{
  int m;
  char *a;

  m = fl_get_menu(obj);
  switch (m) {
  case 1:
    info();
    break;
  case 2:
    /* readme(obj, 2); */
    if (state.wwwbrowser && state.helpfile) {
      a = (char *) myalloc(sizeof(char) * (strlen(state.wwwbrowser) + 
					   strlen(state.helpfile) + 2), "helpC", "a");
      strcpy(a, state.wwwbrowser);
      strcat(a, " ");
      strcat(a, state.helpfile);
      fl_exe_command(a, 0);
    }
    break;
  }
}


void
info(void)

{
  static FD_info *info;
  char a[30];

  if (info == NULL) {
    info = create_form_info();
    sprintf(a, "Fistblink %s", VERSION);
    fl_set_object_label(info->versionW, a);
  }
  /*  Deactivate FITS button */
  fl_deactivate_all_forms();
  /*  Show the form  */
  fl_show_form(info->info, FL_PLACE_MOUSE | FL_FIX_SIZE, FL_FULLBORDER, "");
  fl_do_forms();
  fl_hide_form(info->info);
  fl_activate_all_forms();
}  

void
readme(FL_OBJECT *obj, int n)

{

  /*  The window in not created yet, do it now  */
  if (state.readme == NULL) {
    state.readme = create_form_help();
    fl_set_form_minsize(state.readme->help, 
		  state.readme->helpW->w, state.readme->helpW->h);
    /*  Read README file  */
    fl_load_browser(state.readme->readmeW, BLINK_README_FILE);
  }
  /*  Deactivate README item in the menu */
  fl_set_menu_item_mode(obj, n, FL_PUP_GREY);
  /*  Show the form  */
  fl_show_form(state.readme->help, FL_PLACE_FREE_CENTER, FL_FULLBORDER, "README");
}

/*  These two remain empty  */
void
zoom1C(FL_OBJECT *obj, long val)

{
}

void
zoom2C(FL_OBJECT *obj, long val)

{
}



void
fileC(FL_OBJECT *obj, long val)

{
  int m;
  int i;
  const char *fn;
  static char *curdir = "";
  static char *pattern = "*.inp";
  static char *name = "";

  m = fl_get_menu(obj);
  switch (m) {
  case LOAD_TO_LEFT:
    load(0);
    break;
  case LOAD_TO_RIGHT:
    load(1);
    break;
  case LOAD_INSTRUCTION_FILE:
    if (state.frame_used > 0) {
      if (!fl_show_question("All images currently in memory will be removed\n"
			   "If you don't want this, press No, "
			   "otherwise press Yes", 1)) {
	return;
      }
      /*  Erase images from the memory  */
      for (i = state.frame_used - 1; i >= 0; i--) 
	erase_image(i);
    }
    fl_use_fselector(1);
    fl_invalidate_fselector_cache();
    fn = fl_show_fselector("Choose instruction file", curdir, pattern, name);
    if (fn == NULL) return;
    /*  Save the current directory  */
    curdir = (char *) fl_get_directory();
    /*  Save the current pattern */
    pattern = (char *) fl_get_pattern();
    chdir(curdir);
    /*  Load the image into the structure */
    load_file((char *)fn);
    break;
  case SAVE_DISPLAYED:
    save(state.control_num);
    break;
  case BROWSE_FITS_TABLE:
    show_table();
    break;
  }
}

