/*  markers.c  */
/*  Part of the fitsblink program  */
/*  Jure Skvarc, February 1999  */
/*  Routines for manipulation with markers  */
#include <stdlib.h>
#include "functs.h"




/*  Find color index from the string  */
int
find_marker_color(char *color)

{
  char *c[] = {"red", "yellow", "blue", "green", "magenta", "cyan"};
  int cl[] = {FL_RED, FL_YELLOW, FL_BLUE, FL_GREEN, FL_MAGENTA, FL_CYAN, FL_RED};
  int i;

  if (strlen(color) == 0) return FL_RED;
  strlwr(color);
  for (i = 0; i < sizeof(c); i++) {
    if (strcmp(color, c[i]) == 0) break;
  }
  return cl[i];
}


/*  Find shape index from the string  */
int
find_marker_shape(char *shape)

{
  char *c[] = {"circle", "box", "diamond", "uptriangle", "downtriangle"};
  int cl[] = {OBJ_CIRCLE, OBJ_BOX, OBJ_DIAMOND, OBJ_UPTRIANGLE, OBJ_DOWNTRIANGLE,
	      OBJ_CIRCLE};
  int i;

  if (strlen(shape) == 0) return OBJ_CIRCLE;
  strlwr(shape);
  for (i = 0; i < sizeof(c); i++) {
    if (strcmp(shape, c[i]) == 0) break;
  }
  return cl[i];
}


/*==================================*/
/*  Reserve some space for markers  */
/*  Assign pointer to frame[n].obj  */
/*  Also return this pointer        */
/*==================================*/
OBJECT *
reserve_marker_space(BLINK_FRAME *frame, int n, int m)

{
  frame[n].objmax = m;
  frame[n].objnum = 0;
  frame[n].obj = (OBJECT *) myalloc(sizeof(OBJECT) * frame[n].objmax, 
					      "reserve_marker_space", "obj");
  return frame[n].obj;
}



/*  Append new marker to the end of list.  If necessary, reallocate the memory */
/*  and increase the frame[n].objmax value  */
/*  Increment the frame[n].objnum counter   */
void
append_marker(BLINK_FRAME *frame, int n, float x, float y, float r, char *s, 
	      int color, int shape)

{
  int i;

  /*  If necessary, reallocate memory  */
  if (frame[n].objnum == frame[n].objmax) {
    frame[n].objmax += OBJECT_NUM;
    if ((frame[n].obj = (OBJECT *)
	 realloc(frame[n].obj, sizeof(OBJECT) * frame[n].objmax)) == NULL) {
      fprintf(stderr, "Out of memory, function append_marker, variable frame[n].obj\n");
      exit(1);
    }
    frame[n].imagelist.s = (STAR *)realloc(frame[n].imagelist.s, 
					   sizeof(STAR) * frame[n].objmax);
    if (frame[n].imagelist.s == NULL) {
      fprintf(stderr, "Out of memory, function append_marker, variable frame[i1].imagelist.s\n");
      exit(1);
    }
  }
  /*  Assign values  */
  i = frame[n].objnum;
  frame[n].obj[i].x = x;
  frame[n].obj[i].y = y;
  frame[n].obj[i].r = r;
  frame[n].obj[i].s = strdup(s);
  frame[n].obj[i].color = color;
  frame[n].obj[i].shape = shape;
  /*  Increment the counter  */
  frame[n].objnum++;
}


/*  Remove the k-th marker from the list  */
void
remove_marker(BLINK_FRAME *frame, int n, int k)

{
  int i;

  if (k < frame[n].objnum) {
    /*  Free the string  */
    free(frame[n].obj[k].s);
    for (i = k + 1; i < frame[n].objnum; i++) {
      frame[n].obj[i - 1] = frame[n].obj[i];
      if (i < frame[n].imagelist.n) 
	frame[n].imagelist.s[i - 1] = frame[n].imagelist.s[i];
    }
    frame[n].objnum--;
    frame[n].imagelist.n--;
  }

}

/*****************************************/
/*  remove all markers from the frame n  */
/*****************************************/
void
remove_all_markers(BLINK_FRAME *frame, int n)

{
  int i;

  if (frame[n].obj) {
    /*  Free memory occupied by strings */
    for (i = frame[n].objnum - 1; i >= 0; i--) {
      remove_marker(frame, n, i);
    }
    free(frame[n].obj);
  }
}


