/*  match_stars.c  */
/*  part of the fitsblink program  */
/*  routines for matching of stars from the catalog and stars from an image  */
/*  A lot of previously independent programs meet here  */
/*  Jure Skvarc                                 */
#include <stdio.h> 
#include <math.h>
#include <forms.h>
#include <malloc.h>
#include <stdlib.h>
#include <unistd.h>

#include "formblink.h"
#include "functs.h"

extern STATE state;
extern BLINK_FRAME *frame;
extern CATALOG catalog[];

/*  Detect stars in the image  */
/*=============================*/
int
detect_stars(BLINK_FRAME *frame, int n)

{
  int i;
  int nstar;
  float kf;

  /*  Free the previous image list, if it exists */
  if (frame[n].imagelist.s != NULL) {
    remove_all_markers(frame, n);
    free(frame[n].imagelist.s);
    if (state.interactive) {
      set_image(frame, n);
      draw_image(frame, n);
    }
  }
  /*  Reserve some space for stars  */
  nstar = NSTAR;
  frame[n].imagelist.s = (STAR *) myalloc(sizeof(STAR) * nstar, "detect_stars", "frame[n].imagelist.s");
  /*  Count the stars  */
  frame[n].imagelist.n = stetje(frame[n].fits, &nstar, &(frame[n].imagelist.s));
  /*  Save last parameters used for counting  */
  frame[n].last_count_options = frame[n].fits;
  if (frame[n].fitsfile.matched) {
    /*  The WCS values are assumed to be correct, so transform the coordinates */
    for (i = 0; i < frame[n].imagelist.n; i++) {
      double ra, dec;
      int status = 0;
      fits_pix_to_world(frame[n].imagelist.s[i].x + 1, 
			frame[n].imagelist.s[i].y + 1, 
			frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
			frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2, 
			frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2, 
			frame[n].fitsfile.crot,
			frame[n].fitsfile.ctype,
			&ra, &dec, &status);
      if (ra >= 360.0) {
	ra -= 360.0;
      }
      if (!status) {
	frame[n].imagelist.s[i].ra = ra;
	frame[n].imagelist.s[i].dec = dec;
      }
    }
  }
  if (state.interactive && frame[n].imagelist.n > 0) {
  /*  Reserve space for the list of objects  */
    reserve_marker_space(frame, n, frame[n].imagelist.n);
    /*  Find the brightest star  */
    kf = 0;
    for (i = 0; i < frame[n].imagelist.n; i++) {
      if (frame[n].imagelist.s[i].intensity > kf) 
	kf = frame[n].imagelist.s[i].intensity;
    }
    kf = 6.0 / log(kf);
    for (i = 0; i < frame[n].imagelist.n; i++) {
      append_marker(frame, n, frame[n].imagelist.s[i].x, frame[n].imagelist.s[i].y,
		    kf * log(fabs(frame[n].imagelist.s[i].intensity)),
		    strdup(""), FL_RED, OBJ_CIRCLE);
    }
    /*  Draw star outlines  */
    draw_objects(frame, n);
    draw_image(frame, n);
    /*  Allow star matching and output of star list  */
    if (state.interactive) {
      fl_set_menu_item_mode(state.blinker->astrometryW, MATCH_STARS, FL_PUP_NONE);
      fl_set_menu_item_mode(state.blinker->astrometryW, SAVE_STAR_LIST, FL_PUP_NONE);
      if (frame[n].matched) {
	fl_set_menu_item_mode(state.blinker->astrometryW, DO_ASTROMETRY, FL_PUP_NONE);
      }
    }
    frame[n].detect = 1;
  }
  return frame[n].imagelist.n;
}



/*  Match stars from a star list to the stars from a catalogue */
/*=============================================================*/
int
match_stars(BLINK_FRAME *frame, int n)

{
  int nm, i, k;
  char r1[15], r2[15], d2[15], d1[15];
  char a[80];
  float errp;
  int status = 0;

  frame[n].map.w = frame[n].fits.x2 - frame[n].fits.x1 + 3;
  frame[n].map.h = frame[n].fits.y2 - frame[n].fits.y1 + 3;
  fits_pix_to_world(0.5 * (frame[n].fits.x1 + frame[n].fits.x2) + 1,
		    0.5 * (frame[n].fits.y1 + frame[n].fits.y2) + 1, 
		    frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
		    frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2, 
		    frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2, 
		    frame[n].fitsfile.crot,
		    frame[n].fitsfile.ctype,
		    &frame[n].map.ra, &frame[n].map.dec, &status);
  if (frame[n].map.ra >= 360.0) {
    frame[n].map.ra -= 360.0;
  }
  if (status != 0) {
    /*  Copy input image parameters into MAP_IMAGE structure  */
    frame[n].map.ra = frame[n].ra;    /* R.A. of the map center  */
    frame[n].map.dec = frame[n].dec;  /* Declination of the map center  */
    frame[n].map.w = frame[n].fitsfile.nax[0];
    frame[n].map.h = frame[n].fitsfile.nax[1];
  }
  frame[n].map.width = fabs(frame[n].xsize) * frame[n].map.w  / 3600.0;  /*  Map width in decimal degrees */
  frame[n].map.height = fabs(frame[n].ysize) * frame[n].map.h / 3600.0;  /*  Map height in decimal degrees */
  frame[n].map.fi = frame[n].fi;   /* Map rotation              */
  
  /*  Reserve space for the star list  */
  if (!frame[n].cataloglist.s) {
    frame[n].cataloglist.s = (STAR *) myalloc(sizeof(STAR) * MAXSTARNUM,
					      "match_stars", "cataloglist");
  }
  if (!catalog[CAT_USNO].use && !catalog[CAT_ACT].use && !catalog[CAT_GUIDE].use
      && !catalog[CAT_GUIDE_SOUTH].use && !catalog[CAT_TYCHO2].use) {
    if (state.interactive) 
      fl_show_alert("Please, select at least one catalog in Astrometry menu!", "", "", 1);
    else return 2;
  }
  else {
    frame[n].map.nm = 0;
    if (catalog[CAT_TYCHO2].use) {
      get_tycho2_stars(&(frame[n].map), frame[n].cataloglist.s, catalog[CAT_TYCHO2].path, frame[n].fitsfile.jd);
    }
    if (catalog[CAT_ACT].use) {
      get_act_stars(&(frame[n].map), frame[n].cataloglist.s, catalog[CAT_ACT].path, frame[n].fitsfile.jd);
    }
    if (catalog[CAT_GUIDE].use || catalog[CAT_GUIDE_SOUTH].use) {
      get_guide_stars(&(frame[n].map), frame[n].cataloglist.s, catalog[CAT_GUIDE].path, catalog[CAT_GUIDE_SOUTH].path);
    }
    if (catalog[CAT_USNO].use) {
      get_usno_stars(&(frame[n].map), frame[n].cataloglist.s, catalog[CAT_USNO].path);
    }
    frame[n].cataloglist.n = frame[n].map.nm;
    if (frame[n].cataloglist.n == 0) {
      if (state.interactive) {
	fl_show_alert("No stars from the catalog found!", "", "", 1);
      }
      return 1;
    }
    fprintf(state.logfp, "    Number of stars from the catalog:%d\n", frame[n].map.nm);
    frame[n].last_match_options.matched = 0;
    nm = field_compare(&(frame[n].imagelist), &(frame[n].cataloglist), 
		  frame[n].match, frame, n);
    frame[n].last_match_options = frame[n].match;
    if (nm > 3) {
      errp = 0;
      k = 0;
      frame[n].matched = frame[n].match.matched = frame[n].last_match_options.matched = 1;
      if (state.interactive) {
	fl_set_menu_item_mode(state.blinker->astrometryW, DO_ASTROMETRY, FL_PUP_NONE);
	/*  Show coordinates of the matched stars */
	if (!(state.report && fl_form_is_visible(state.report->Report))) 
	  show_report();
	fl_clear_browser(state.report->reportW);
	fl_freeze_form(state.report->Report);
	fl_addto_browser(state.report->reportW, 
			 "Image no.     RA       DEC          Cat. no.       RA        DEC        err");
	for (i = 0; i < frame[n].imagelist.n; i++) {
	  int m = frame[n].imagelist.s[i].m;
	  if (m != -1 && frame[n].imagelist.s[i].q) {
	    strcpy(r1, format_ra(frame[n].imagelist.s[i].ra));
	    strcpy(d1, format_dec(frame[n].imagelist.s[i].dec));
	    strcpy(r2, format_ra(frame[n].cataloglist.s[m].ra));
	    strcpy(d2, format_dec(frame[n].cataloglist.s[m].dec));
	    sprintf(a, "%08d %s %s %c%15s %s %s %6.2f", 
		    frame[n].imagelist.s[i].n, r1, d1, 
		    frame[n].cataloglist.s[m].catalog, 
		    frame[n].cataloglist.s[m].catid, r2, d2, 
		    frame[n].imagelist.s[i].err);
	    fl_addto_browser(state.report->reportW, a);
	    append_marker(frame, n, frame[n].cataloglist.s[m].x, 
			  frame[n].cataloglist.s[m].y, 6.0,
			  strdup(""), FL_YELLOW, OBJ_DIAMOND);
	    errp += frame[n].imagelist.s[i].err;
	    k++;
	  }
	}
	errp /= k;
	fl_addto_browser(state.report->reportW, 
			 "-----------------------------------------------------------------------------");
	sprintf(a, "Number of reference stars: %d", k);
	fl_addto_browser(state.report->reportW, a);
	sprintf(a, "Average residual: %.2f\"", errp);
	fl_addto_browser(state.report->reportW, a);
	fl_unfreeze_form(state.report->Report);
	fprintf(state.logfp, "Number of reference stars: %d\n", k);
	fprintf(state.logfp, "Average residual: %.2f\"\n", errp);

	/*  Draw star outlines  */
	set_image(frame, n);
	draw_objects(frame, n);
	draw_image(frame, n);
      }
      else {
	for (i = 0; i < frame[n].imagelist.n; i++) {
	  int m = frame[n].imagelist.s[i].m;
	  if (m != -1 && frame[n].imagelist.s[i].q) {
	    errp += frame[n].imagelist.s[i].err;
	    k++;
	  }
	}
	errp /= k;
	fprintf(state.logfp, "Number of reference stars: %d\n", k);
	fprintf(state.logfp, "Average residual: %.2f\"\n", errp);
      }
    }
    else {
      if (state.interactive) {
	fl_show_alert("Matching failed!", "", "", 1);
      }
      return 1;
    }
  }
  return 0;
}



/*  Output the detected stars to a file  */
int
star_list(BLINK_FRAME *frame, int n)

{
  const char *fn;
  static char *curdir = "";
  static char *pattern = "*.*dat";
  static char *name = "";
  FILE *fp;
  int i;
  char time[64];
  int timeref = 0;

  make_output_filename(frame[n].fits.name, &name, "dat");
  if (state.interactive) {
    fl_use_fselector(3);
    fl_invalidate_fselector_cache();
    fn = fl_show_fselector("Choose output file", curdir, pattern, name);
    if (fn == NULL) {
      free(name);
      return 1;
    }
    /*  Save the current directory  */
    curdir = (char *) fl_get_directory();
    /*  Save the current pattern */
    pattern = (char *) fl_get_pattern();
  }
  else {
    fn = name;
  }
  if (state.starlistformat == FITS_STARLIST) {
    int status;
    fitsfile *fptr;
    double jd;

    /*  Write star list in the FITS format  */
    /*  List of field names for objects  */
    char *ttype[] = {"ID", "XT", "YT", "SXT", "SYT", "AREA", "INTENSITY",
		     "SATURATE", "BACKGRND", "RA", "DEC", "SRA", "SDEC",
		     "MAG", "SMAG", "CATNAME", "CATID", "COLOR", "SHAPE", "SYMMETRY", "SINTENSITY"};
    /* Types of parameters: J - int, E - float, D - double, I - short */
    /*                      L - boolean, A15 - char[15]  */
    char *tform[] = {"J", "E", "E", "E", "E", "J", "J",
		     "I", "E", "D", "D", "E", "E",
		     "E", "E", "I", "15A15", "B", "B", "E", "E"};
    /*  Units of the parameters  */
    char *tunit[] = {"", "pixel", "pixel", "pixel", "pixel", "pixel", "ADU",
		     "", "ADU", "degrees", "degrees", "arc sec", "arc sec",
		     "", "", "", "", "", "", "", ""};
    status = 0;
    /*  Create a file  */
    fits_create_file(&fptr, fn, &status);
    /*  In interactive mode, ask to overwrite existing file  */
    if (state.interactive && status == FILE_NOT_CREATED) {
      int ans = fl_show_question("File already exists!\nOverwrite?\n", 2);
      if (ans == 0) return 1;
      unlink(fn);
      status = 0;
      fits_create_file(&fptr, fn, &status);
      if (status != 0) {
	fl_show_alert("Still can't create a file", "Aborting", "", 1);
	return 1;
      }
    }
    else if (!state.interactive) {
      fprintf(stderr, "WARNING: Overwriting file %s!\n", fn);
      status = 0;
      unlink(fn);
      fits_create_file(&fptr, fn, &status);
      if (status) {
	fprintf(stderr, "File creation not successful!  Exiting...\n");
	exit(1);
      }
    }
    fits_create_tbl(fptr, BINARY_TBL, 0, sizeof(ttype) / sizeof(char *), 
		    ttype, tform, tunit, NULL, &status);

    fits_write_key(fptr, TSTRING, "EXTNAME", "Objects", "Extension name", &status);
    fits_write_key(fptr, TSTRING, "IMGNAME", frame[n].fits.name, "Original image name", &status);
    /*  Insert creation date  */
    fits_get_system_time(time, &timeref, &status);
    fits_write_key(fptr, TSTRING, "DATE", time, "Creation date", &status);
    jd = julian_day(frame[n].year, frame[n].month, frame[n].day + 
		    (frame[n].hour + (frame[n].minute + 
				      frame[n].second / 60.0) / 60.0) / 24.0);
    fits_write_key(fptr, TDOUBLE, "JULIAN", &jd, "Observation time", &status);
    /*  Insert the factor between the magnitude and intensity  */
    fits_write_key(fptr, TFLOAT, "MAGFAC", &frame[n].match.magfac, 
		   "Intensity to magnitude", &status);

    /*=================================================*/
    /*  Insert to the header last counting parameters  */
    /*=================================================*/
    fits_write_key(fptr, TINT, "X1", &frame[n].last_count_options.x1, 
		   "Active rectangle", &status);
    fits_write_key(fptr, TINT, "Y1", &frame[n].last_count_options.y1, 
		   "Active rectangle", &status);
    fits_write_key(fptr, TINT, "X2", &frame[n].last_count_options.x2, 
		   "Active rectangle", &status);
    fits_write_key(fptr, TINT, "Y2", &frame[n].last_count_options.y2, 
		   "Active rectangle", &status);
    fits_write_key(fptr, TFLOAT, "MINBRITE", &frame[n].last_count_options.minbright, 
		   "Minimal accepted brightness", &status);
    fits_write_key(fptr, TINT, "MINSTAR", &frame[n].last_count_options.minstar, 
		   "Minimal star size (pixels)", &status);
    fits_write_key(fptr, TINT, "BCKSIZE", &frame[n].last_count_options.maxsize, 
		   "Background cell size", &status);
    fits_write_key(fptr, TFLOAT, "SG_NUM", &frame[n].last_count_options.sg_num, 
		   "Threshold in sigma units", &status);
    fits_write_key(fptr, TSHORT, "BORDER", &frame[n].last_count_options.border, 
		   "Inactive border", &status);
    fits_write_key(fptr, TINT, "SATLEVEL", &frame[n].last_count_options.satlevel, 
		   "Saturation level", &status);
    fits_write_key(fptr, TLOGICAL, "APERTURE", &frame[n].last_count_options.aperture, 
		   "T - aperture astrometry was used", &status);
    fits_write_key(fptr, TFLOAT, "AP_INNER", &frame[n].last_count_options.inner, 
		   "Inner circle size in aperture astrometry", &status);
    fits_write_key(fptr, TFLOAT, "AP_OUTER", &frame[n].last_count_options.outer, 
		   "Outer circle size in aperture astrometry", &status);

    fits_write_key(fptr, TFLOAT, "RESOLVE", &frame[n].last_count_options.resolve, 
		   "How strongly try to resolve close stars", &status);
    fits_write_key(fptr, TFLOAT, "GAIN", &frame[n].last_count_options.gain, 
		   "Camera gain", &status);
    fits_write_key(fptr, TFLOAT, "RDNOISE", &frame[n].last_count_options.rdnoise, 
		   "Camera read out noise", &status);
    if (status) {
      free(name);
      if (state.interactive) {
	fl_show_alert("Error in writing counting parameters to header!", "", "", 1);
	return 1;
      }
      else {
	fprintf(stderr, "Error in writing counting parameters to header. Exiting...\n");
	exit(1);
      }
    }
    /*=================================================*/
    /*  Insert to the header last matching parameters  */
    /*=================================================*/
    if (frame[n].last_match_options.matched) {
      fits_write_key(fptr, TFLOAT, "MINCONST", &frame[n].last_match_options.min, 
		     "Minimal constellation value", &status);
      fits_write_key(fptr, TINT, "FIRSTBRT", &frame[n].last_match_options.firstbright, 
		     "Number of stars used for initial matching", &status);
      fits_write_key(fptr, TFLOAT, "POSERRMX", &frame[n].last_match_options.poserrmax, 
		     "Maximal positional error in pixels", &status);
      fits_write_key(fptr, TFLOAT, "MAXRES", &frame[n].last_match_options.maxres, 
		     "[arcseconds] Maximal allowed residual", &status);
      fits_write_key(fptr, TFLOAT, "MINDIST", &frame[n].last_match_options.mindist, 
		     "Minimal distance between stars in constellation", &status);
      fits_write_key(fptr, TINT, "PLTORDER", &frame[n].last_match_options.fitorder_used, 
		     "Order of plate constants", &status);
      fits_write_key(fptr, TSTRING, "FILTER", frame[n].last_match_options.filter, 
		     "Filter", &status);
      fits_write_key(fptr, TINT, "NUMSTARS", &frame[n].last_match_options.nmatched, 
		     "Number of successfully matched stars", &status);
    }
    if (status) {
      free(name);
      if (state.interactive) {
	fl_show_alert("Error in writing matching parameters to header!", "", "", 1);
	return 1;
      }
      else {
	fprintf(stderr, "Error in writing matching parameters to header. Exiting...\n");
	exit(1);
      }
    }
    for (i = 0; i < frame[n].imagelist.n; i++) {
      int j = i + 1;
      char *catid[1];
      char color = 0, shape = 0;
      
      float x = frame[n].imagelist.s[i].x, y = frame[n].imagelist.s[i].y;
      catid[0] = frame[n].imagelist.s[i].catid;
      status = 0;
      fits_write_col(fptr, TINT, 1, j, 1, 1, &i, &status);
      fits_write_col(fptr, TFLOAT, 2, j, 1, 1, &x, &status);
      fits_write_col(fptr, TFLOAT, 3, j, 1, 1, &y, &status);
      fits_write_col(fptr, TFLOAT, 4, j, 1, 1, 
		     &frame[n].imagelist.s[i].sxt, &status);
      fits_write_col(fptr, TFLOAT, 5, j, 1, 1, 
		     &frame[n].imagelist.s[i].syt, &status);
      fits_write_col(fptr, TINT, 6, j, 1, 1, 
		     &frame[n].imagelist.s[i].area, &status);
      fits_write_col(fptr, TINT, 7, j, 1, 1, 
		     &frame[n].imagelist.s[i].intensity, &status);
      fits_write_col(fptr, TSHORT, 8, j, 1, 1, 
		     &frame[n].imagelist.s[i].saturated, &status);
      fits_write_col(fptr, TFLOAT, 9, j, 1, 1, 
		     &frame[n].imagelist.s[i].bg, &status);
      fits_write_col(fptr, TDOUBLE, 10, j, 1, 1, 
		     &frame[n].imagelist.s[i].ra, &status);
      fits_write_col(fptr, TDOUBLE, 11, j, 1, 1, 
		     &frame[n].imagelist.s[i].dec, &status);
      fits_write_col(fptr, TFLOAT, 12, j, 1, 1, 
		     &frame[n].imagelist.s[i].sra, &status);
      fits_write_col(fptr, TFLOAT, 13, j, 1, 1, 
		     &frame[n].imagelist.s[i].sdec, &status);
      fits_write_col(fptr, TFLOAT, 14, j, 1, 1, 
		     &frame[n].imagelist.s[i].mag, &status);
      fits_write_col(fptr, TFLOAT, 15, j, 1, 1, 
		     &frame[n].imagelist.s[i].smag, &status);
      fits_write_col(fptr, TSHORT, 16, j, 1, 1, 
		     &frame[n].imagelist.s[i].catalog, &status);
      /*sprintf(frame[n].imagelist.s[i].catid, "%12.0f", 
	frame[n].imagelist.s[i].catnum);*/
      fits_write_col(fptr, TSTRING, 17, j, 1, 1, catid, &status);
      fits_write_col(fptr, TBYTE, 18, j, 1, 1, &color, &status);
      fits_write_col(fptr, TBYTE, 19, j, 1, 1, &shape, &status);
      fits_write_col(fptr, TFLOAT, 20, j, 1, 1, &frame[n].imagelist.s[i].sym, &status);
      fits_write_col(fptr, TFLOAT, 21, j, 1, 1, &frame[n].imagelist.s[i].sintensity, &status);
      fits_report_error(stderr, status);
    }
#if 0
    /*  Make a copy of the HDU of the original file  */
    fits_create_hdu(fptr, &status);
    fits_open_file(&orfptr, frame[n].fits.name, READONLY, &status);
    i = 1;
    do {
      char line[81];
      /*  Read fits header line */
      fits_read_record(orfptr, i, line, &status);
      if (!strncmp("END", line, 3)) break;
      if (!strncmp("SIMPLE", line, 6)) {
	fits_write_record
      fits_write_record(fptr, line, &status);
      i++;
    } while (status == 0);
#endif
    fits_close_file(fptr, &status);
    fits_report_error(stderr, status);
    if (status != 0) {
      if (state.interactive) {
	fl_show_alert("Error in writing FITS star list!", "", "", 1);
      }
      else {
	fprintf(stderr, "Error in writing FITS star list!\n");
      }
    }
  }
  else {
    /*====================================================*/
    /*  Plain and limited ASCII data output               */
    /*====================================================*/
    if (!(fp = fopen(fn, "w"))) {
      if (state.interactive) {
	fl_show_alert("Can not open file", fn, "for writing!", 0);
	free(name);
	return 1;
      }
      else {
	fprintf(stderr, "Can not open file %s for writing!\n", fn);
	exit(1);
      }
    }
    fprintf(fp, "Input file name: %s time:%.8f magfac: %e\n", frame[n].fits.name, 
	    julian_day(frame[n].year, frame[n].month, frame[n].day + 
		       (frame[n].hour + (frame[n].minute + frame[n].second / 60.0) 
			/ 60.0) / 24.0), frame[n].match.magfac);
    for (i = 0; i < frame[n].imagelist.n; i++) {
      fprintf(fp, "%5d %7.2f %7.2f %11.2f", i, frame[n].imagelist.s[i].x,
	      frame[n].imagelist.s[i].y, (double) frame[n].imagelist.s[i].intensity);
      if (frame[n].matched) {
	int m;
	fprintf(fp, " %9.6f %9.6f", 
		frame[n].imagelist.s[i].ra, frame[n].imagelist.s[i].dec);
	m = frame[n].imagelist.s[i].m;
	if (m != -1) {
	  fprintf(fp, " %c %15s %6.2f %6.2f", frame[n].cataloglist.s[m].catalog,
		  frame[n].cataloglist.s[m].catid, frame[n].cataloglist.s[m].err,
		  frame[n].cataloglist.s[m].mag);
	}
      }
      fprintf(fp, "\n");
    }
    fprintf(state.logfp, "Star list output written to: %s\n", fn);
    fclose(fp);
    free(name);
  }
  return 0;
}


/*  Read star positions and intensities from a file  */
int
load_star_list(BLINK_FRAME *frame, int n)

{
  const char *fn;
  static char *curdir = "";
  static char *pattern = "*.dat";
  static char *name = "";
  char line[256];
  char *p;
  FILE *fp;
  long nstar;
  int i, m;
  float kf;
  int was = 0;
  char ftype[7];
  int inputformat = state.starlistformat;
  int colid, colxt, colyt, colsxt, colsyt, colarea;
  int colintensity, colsintensity, colsaturate, colbackgrnd;
  int colra, coldec, colsra, colsdec, colmag, colsmag, colcatname;
  int colcatid, colcolor, colshape, colsymmetry;

  make_output_filename(frame[n].fits.name, &name, (state.starlistformat == OLD_ASCII_LIST) ? "dat" : "dat");
  fn = name;
  if (state.interactive) {
    fl_use_fselector(3);
    fl_invalidate_fselector_cache();
    fn = fl_show_fselector("Choose input file", curdir, pattern, name);
    if (fn == NULL) {
      free(name);
      return 1;
    }
    /*  Save the current directory  */
    curdir = (char *) fl_get_directory();
    /*  Save the current pattern */
    pattern = (char *) fl_get_pattern();
  }

  /* Remove old markers, if they exist  */
  if (frame[n].imagelist.s != NULL) {
    remove_all_markers(frame, n);
    free(frame[n].imagelist.s);
    was = 1;
  }
  if ((fp = fopen(fn, "r"))) {
    /*  Check if file is FITS  */
    for (i = 0; i < 6; i++) {
      ftype[i] = fgetc(fp);
    }
    ftype[6] = '\0';
    if (strcmp(ftype, "SIMPLE") == 0) {
      inputformat = FITS_STARLIST;
    }
    else {
      inputformat = OLD_ASCII_LIST;
    }
    fclose(fp);
  }
  else {
    if (state.interactive) {
      fl_show_alert("Can not open file", fn, "for reading!", 0);
      free(name);
      return 1;
    }
    else {
      fprintf(stderr, "Can not open file %s for reading!\n", fn);
      free(name);
      return 1;
    }	
  }
  if (inputformat == FITS_STARLIST) {
    char comment[81];
    int status  = 0;
    fitsfile *fptr;
    /*  Open the file  */
    fits_open_file(&fptr, fn, READONLY, &status);
    if (status) {
      if (state.interactive) {
	fl_show_alert("Can not open file", fn, "for reading!", 0);
	free(name);
	return 1;
      }
      else {
	fprintf(stderr, "Can not open file %s for reading!\n", fn);
	free(name);
	return 1;
      }	
    }
    /*  Move to the table extension  */
    fits_movnam_hdu(fptr, BINARY_TBL, "Objects", 0, &status);
    fits_report_error(stderr, status);
    /*  Read the number of stars in the array  */
    fits_get_num_rows(fptr, &nstar, &status);
    fits_report_error(stderr, status);
    /*  reserve space for stars  */
    frame[n].imagelist.s = (STAR *) myalloc(sizeof(STAR) * nstar, "load_star_list", "frame[n].imagelist.s");
    /*  Set star number  */
    frame[n].imagelist.n = nstar;
    
    /*  Read the magnitude factor  */
    fits_read_key_flt(fptr, "MAGFAC", &frame[n].match.magfac, 
		      comment, &status);

    status = 0;
    /*====================================================*/
    /*  Read counting parameters used for star detection  */
    /*====================================================*/
    fits_read_key(fptr, TINT, "X1", &frame[n].last_count_options.x1, 
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "Y1", &frame[n].last_count_options.y1, 
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "X2", &frame[n].last_count_options.x2,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "Y2", &frame[n].last_count_options.y2,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "MINBRITE", &frame[n].last_count_options.minbright,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "MINSTAR", &frame[n].last_count_options.minstar,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "BCKSIZE", &frame[n].last_count_options.maxsize,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "SG_NUM", &frame[n].last_count_options.sg_num,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TSHORT, "BORDER", &frame[n].last_count_options.border,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "SATLEVEL", &frame[n].last_count_options.satlevel,
		      comment, &status);
    status = 0; 
    fits_read_key_log(fptr, "APERTURE", &frame[n].last_count_options.aperture,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "AP_INNER", &frame[n].last_count_options.inner,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "AP_OUTER", &frame[n].last_count_options.outer,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "RESOLVE", &frame[n].last_count_options.resolve,
		      comment, &status);
    status = 0; 
    /*====================================================*/
    /*  Read matching parameters used for star detection  */
    /*====================================================*/
    fits_read_key_flt(fptr, "MINCONST", &frame[n].last_match_options.min,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT,  "FIRSTBRT", 
		  &frame[n].last_match_options.firstbright, comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "POSERRMX", &frame[n].last_match_options.poserrmax,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "MAXRES", &frame[n].last_match_options.maxres,
		      comment, &status);
    status = 0; 
    fits_read_key_flt(fptr, "MINDIST", &frame[n].last_match_options.mindist,
		      comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT,  "PLTORDER", 
		  &frame[n].last_match_options.fitorder_used, comment, &status);
    status = 0; 
    fits_read_key_str(fptr, "FILTER", 
		      frame[n].last_match_options.filter, comment, &status);
    status = 0; 
    fits_read_key(fptr, TINT, "NUMSTARS", 
		  &frame[n].last_match_options.nmatched, comment, &status);
    if (status == 0) {
      frame[n].last_match_options.matched = 1;
    }
    status = 0; 

    /*  Read the original image name  */
    frame[n].name1 = (char *) myalloc(sizeof(char) * 1024, "load_star_list",
				     "frame[n].name1");
    fits_read_key_str(fptr, "IMGNAME", frame[n].name1, comment, &status);
    frame[n].matched = (frame[n].match.magfac > 0);

    /* Find out column numbers  */
    fits_get_colnum(fptr, CASESEN, "ID", &colid, &status);
    fits_get_colnum(fptr, CASESEN, "XT", &colxt, &status);
    fits_get_colnum(fptr, CASESEN, "YT", &colyt, &status);
    fits_get_colnum(fptr, CASESEN, "SXT", &colsxt, &status);
    fits_get_colnum(fptr, CASESEN, "SYT", &colsyt, &status);
    fits_get_colnum(fptr, CASESEN, "AREA", &colarea, &status);
    fits_get_colnum(fptr, CASESEN, "INTENSITY", &colintensity, &status);
    fits_get_colnum(fptr, CASESEN, "SATURATE", &colsaturate, &status);
    fits_get_colnum(fptr, CASESEN, "BACKGRND", &colbackgrnd, &status);
    fits_get_colnum(fptr, CASESEN, "RA", &colra, &status);
    fits_get_colnum(fptr, CASESEN, "DEC", &coldec, &status);
    fits_get_colnum(fptr, CASESEN, "SRA", &colsra, &status);
    fits_get_colnum(fptr, CASESEN, "SDEC", &colsdec, &status);
    fits_get_colnum(fptr, CASESEN, "MAG", &colmag, &status);
    fits_get_colnum(fptr, CASESEN, "SMAG", &colsmag, &status);
    fits_get_colnum(fptr, CASESEN, "CATNAME", &colcatname, &status);
    fits_get_colnum(fptr, CASESEN, "CATID", &colcatid, &status);
    fits_get_colnum(fptr, CASESEN, "COLOR", &colcolor, &status);
    fits_get_colnum(fptr, CASESEN, "SHAPE", &colshape, &status);
    fits_get_colnum(fptr, CASESEN, "SYMMETRY", &colsymmetry, &status);
    fits_get_colnum(fptr, CASESEN, "SINTENSITY", &colsintensity, &status);


    /*  Read all stars  */
    for (i = 0; i < nstar; i++) {
      int j = i + 1;
      int anynul, i1;
      float x, y;
      char *catid[1];
      catid[0] = frame[n].imagelist.s[i].catid;

      fits_read_col(fptr, TINT, colid, j, 1, 1, NULL, &i1, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colxt, j, 1, 1, NULL, &x, &anynul, &status);
      frame[n].imagelist.s[i].x = x;
      fits_read_col(fptr, TFLOAT, colyt, j, 1, 1, NULL, &y, &anynul, &status);
      frame[n].imagelist.s[i].y = y;
      fits_read_col(fptr, TFLOAT, colsxt, j, 1, 1, NULL,
		     &frame[n].imagelist.s[i].sxt, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsyt, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].syt, &anynul, &status);
      fits_read_col(fptr, TINT, colarea, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].area, &anynul, &status);
      fits_read_col(fptr, TINT, colintensity, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].intensity, &anynul, &status);
      fits_read_col(fptr, TSHORT, colsaturate, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].saturated, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colbackgrnd, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].bg, &anynul, &status);
      fits_read_col(fptr, TDOUBLE, colra, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].ra, &anynul, &status);
      fits_read_col(fptr, TDOUBLE, coldec, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].dec, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsra, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].sra, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsdec, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].sdec, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colmag, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].mag, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsmag, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].smag, &anynul, &status);
      fits_read_col(fptr, TSHORT, colcatname, j, 1, 1, NULL, 
		     &frame[n].imagelist.s[i].catalog, &anynul, &status);
      fits_read_col(fptr, TSTRING, colcatid, j, 1, 1, NULL, catid, &anynul, &status);
      frame[n].imagelist.s[i].catid[15] = '\0';
      frame[n].imagelist.s[i].catnum = strtod(frame[n].imagelist.s[i].catid, NULL);
      fits_read_col(fptr, TFLOAT, colsymmetry, j, 1, 1, NULL, &frame[n].imagelist.s[i].sym, &anynul, &status);
      fits_read_col(fptr, TFLOAT, colsintensity, j, 1, 1, NULL, &frame[n].imagelist.s[i].sintensity, &anynul, &status);
    }
    fits_close_file(fptr, &status);
    fits_report_error(stderr, status);
  }
  else {
    if (!(fp = fopen(fn, "r"))) {
      if (state.interactive) {
	fl_show_alert("Can not open file", fn, "for reading!", 0);
	free(name);
	return 1;
      }
      else {
	fprintf(stderr, "Can not open file %s for reading!\n", fn);
	free(name);
	return 1;
      }	
    }
    /*  reserve space for some stars  */
    nstar = NSTAR;
    frame[n].imagelist.s = (STAR *) myalloc(sizeof(STAR) * nstar, "load_star_list", "frame[n].imagelist.s");
    /*  Set star counter to zero  */
    i = frame[n].imagelist.n = 0;
    /*  Get the first line in the input file  */
    fgets(line, 255, fp);
    /*  Read the magnitude factor  */
    p = strstr(line, "magfac: ");
    if (p) {
      frame[n].match.magfac = strtod(p + strlen("magfac:"), NULL);
      frame[n].matched = 1;
    }
    if (frame[n].match.magfac == 0) {
      frame[n].matched = 0;
    }
    /*  Read all stars  */
    while (!feof(fp)) {
      /*  Get a line  */
      fgets(line, 255, fp);
      if (!feof(fp)) {
	char c;
	float intensity;
	frame[n].imagelist.s[i].ra = 0.0;
	frame[n].imagelist.s[i].dec = 0.0;
	frame[n].imagelist.s[i].catnum = 0;
	m = sscanf(line, "%d %lf %lf %f %lf %lf %c %lf %f", &frame[n].imagelist.s[i].n, 
		   &frame[n].imagelist.s[i].x, &frame[n].imagelist.s[i].y, 
		   &intensity, &frame[n].imagelist.s[i].ra, 
		   &frame[n].imagelist.s[i].dec,  &c,
		   &frame[n].imagelist.s[i].catnum, &frame[n].imagelist.s[i].err);
	frame[n].imagelist.s[i].catalog = c;
	frame[n].imagelist.s[i].intensity = (int) intensity;
	if (m >= 4) {
	  if (frame[n].imagelist.s[i].ra == 0.0 && 
	      frame[n].imagelist.s[i].dec == 0.0 && frame[n].fitsfile.matched) {
	    if (frame[n].fitsfile.wcs) {
	      double ra, dec;
	      int status = 0;
	      fits_pix_to_world(frame[n].imagelist.s[i].x + 1, 
				frame[n].imagelist.s[i].y + 1, 
				frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
				frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2, 
				frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2, 
				frame[n].fitsfile.crot,
				frame[n].fitsfile.ctype,
				&ra, &dec, &status);
	      if (ra >= 360.0) {
		ra -= 360.0;
	      }
	      if (!status) {
		frame[n].imagelist.s[i].ra = ra;
		frame[n].imagelist.s[i].dec = dec;
	      }
	    }
	  }
	  frame[n].imagelist.n++;
	  i++;
	  if (i == nstar) {
	    nstar += NSTAR;
	    frame[n].imagelist.s = (STAR *) realloc(frame[n].imagelist.s,
						    sizeof(STAR) * nstar);
	    if (frame[n].imagelist.s == NULL) {
	      fprintf(stderr, "Out of memory, function load_star_list, frame[%d].imagelist.s", n);
	      exit(1);
	    }
	  }
	}
      }
    }
    fclose(fp);
  }
  /*  Return space  */
  frame[n].imagelist.s = (STAR *) realloc(frame[n].imagelist.s,
					  sizeof(STAR) * frame[n].imagelist.n);
  /*   Create markers   */
  reserve_marker_space(frame, n, frame[n].imagelist.n);
  /*  Find the brightest star  */
  kf = 0;
  for (i = 0; i < frame[n].imagelist.n; i++) {
    if (fabs(frame[n].imagelist.s[i].intensity) > kf) 
      kf = fabs(frame[n].imagelist.s[i].intensity);
  }
  kf = 6.0 / log(kf);
  for (i = 0; i < frame[n].imagelist.n; i++) {
    append_marker(frame, n, frame[n].imagelist.s[i].x, frame[n].imagelist.s[i].y,
		  kf * log(fabs(frame[n].imagelist.s[i].intensity)), strdup(""),
		  FL_RED, OBJ_CIRCLE);
  }
  if (frame[n].objnum > 0) {
    if (state.interactive) {
      fl_set_menu_item_mode(state.blinker->astrometryW, MATCH_STARS, FL_PUP_NONE);
      fl_set_menu_item_mode(state.blinker->astrometryW, SAVE_STAR_LIST, FL_PUP_NONE);
      if (frame[n].matched) {
	fl_set_menu_item_mode(state.blinker->astrometryW, DO_ASTROMETRY, FL_PUP_NONE);
      }
    }
    frame[n].detect = 1;
  }
  if (state.interactive) {
    if (was) {
      set_image(frame, n);
    }
    else {
      draw_objects(frame, n);
    }      
    draw_image(frame, n);
  }
  free(name);
  return 0;
}

