#!/bin/sh

# Replace the right side of the following line with
# a path your catalogs
gscn="/home/jure/astro/katalogi/gsc"
gscs="/home/jure/astro/katalogi/gsc"


# erases the result file
>results.dat

for a in *.fts; do 

# Create a name of the star list file
b=`basename $a .fts`.dat
# Count the stars using "aperture" mode
# -s 5    ---  threshold
# -n 3.0  ---  radius of the inner circle
# -o 12.0 ---  radius of the outer circle
# -m 30   ---  size of the unit square inside which background is determined
starcount -l log.dat -s 5 -n 3.0 -o 12.0 -m 30 $a

# Make a match with the GSC catalog
# -f $b   ---  $b contains the name of the star list file
# -o 4    ---  4th order polynoms used for fitting
# -e 3    ---  allowed difference in star positions in initial fitting (in pixels)
# -m 12   ---  a minimal constellation value to be accepted
# -D 10   ---  a minimal distance between stars in constellation
# -b 200  ---  number of bright stars used for initial catalog matching
# -n 15   ---  number of stars in constellation
# -r 4    ---  maximal residual for the final matching (in arc seconds)
# -i 100  ---  maximal allowed ratio of intensities (not magnitudes)
#              between the stars from catalog and image
# -c      ---  two -c switches supply a path to the catalog
catalog -f $b -o 4 -e 3 -m 12 -D 10 -b 200 -n 15 -r 4 -i 100 \
    -c gscn:$gscn -c gscs:$gscs 

#  From the output file extract only lines which contain identification
#  with a catalog stars.  Also, calculate magnitudes from the intensities
awk 'NR==1{split($0, a, ":");mf=a[4];l10=1/log(10)}NR>1&&NF==10{printf "%s %5.2f\n", $0, 2.5*(mf-log($4)*l10)}' $b >>results.dat
done



