/*  showstar.c  */
/*  part of the fitsblink program  */
/*  Jure Skvarc                    */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
 
#include <forms.h>
#include <fitsio.h>

#include "formblink.h"
#include "functs.h"

extern STATE state;
extern BLINK_FRAME *frame;
extern OBSERVATORY observatory;

STAR *cstar;

void
objcometC(FL_OBJECT *obj, long val)

{
  fl_trigger_object(state.showcomet->acceptcometW);
  fl_trigger_object(state.showcomet->okcometW);
}


void
draw_object(FL_OBJECT *o, STAR *z, int zoom, FITS_IMAGE *im, BLINK_FRAME *frame, int n)

{
  int xq, yq, yt, xt, r;
  int v, h, w;
  int i, j, x, y;
  GC gc = fl_state[fl_get_form_vclass(state.blinker->blinker)].gc[0];
  Window win = FL_ObjWin(o);

  x = z->x;
  y = z->y;
  /*  Number of pixels in x and y direction  */
  w = o->w / zoom;
  h = o->h / zoom;
  /*  Do the drawing  */
  yq = o->y;
  yt = y - h / 2 + 0.5;
  for (j = 0; j < h; j++) {
    xq = o->x;
    xt = x - w / 2 + 0.5; 
    for (i = 0; i < w; i++) {
      v = video_point(im, xt++, yt);
      XSetForeground(state.disp, gc, state.pixel[correct(frame, n, v)]);
      XFillRectangle(state.disp, win, gc, xq, yq, zoom, zoom);
      xq += zoom;
    }
    yq += zoom;
    yt++;
  }
  /*  Draw a cross  */
  xt = z->x - w / 2;
  xt = (z->x - (float) xt + 0.5) * zoom + o->x;
  yt = z->y - h / 2;
  yt = (z->y - (float) yt + 0.5) * zoom + o->y;
  w = o->w / 4;
  h = o->h / 4;
  fl_line(xt, o->y, xt, o->y + o->h - 1, FL_RED);
  fl_line(o->x, yt, o->x + o->w - 1, yt, FL_RED);
  if (im->aperture) {
    r = (int) (frame[n].fits.inner * zoom + 0.5);
    fl_ovall(xt - r, yt - r, 2 * r, 2 * r, FL_WHITE);
    fl_ovall(xt - r - 1, yt - r - 1, 2 * (r + 1), 2 * (r + 1), FL_BLACK);
    r = (int) (frame[n].fits.outer * zoom + 0.5);
    fl_ovall(xt - r, yt - r, 2 * r, 2 * r, FL_WHITE);
    fl_ovall(xt - r - 1, yt - r - 1, 2 * (r + 1), 2 * (r + 1), FL_BLACK);
  }
}

int
freeobj_windstarW_handle(FL_OBJECT *d, int event, FL_Coord mx, FL_Coord my, int key, void *xev)

{
  int n;
  int zoom;
  FITS_IMAGE *im;

  n = state.control[state.control_num];
  switch(event) {
  case FL_DRAW:
    zoom = 5;
    im = &(frame[n].fits);
    draw_object(d, cstar, zoom, im, frame, n);
  }
  return 0;
}

int
freeobj_windcometW_handle(FL_OBJECT *d, int event, FL_Coord mx, FL_Coord my, int key, void *xev)

{
  int n;
  int zoom;
  FITS_IMAGE *im;

  n = state.control[state.control_num];
  switch(event) {
  case FL_DRAW:
    zoom = 5;
    im = &(frame[n].fits);
    draw_object(d, cstar, zoom, im, frame, n);
  }
  return 0;
}



void
distanceC(FL_OBJECT *obj, long val)

{
  float dist;
  char a[10];

  dist = strtod(fl_get_input(obj), NULL);
  if (dist > 300) {
    dist = 300;
  }
  if (dist <= 0) {
    dist = 0.01;
  }
  sprintf(a, "%.2f", dist);
  fl_set_input(obj, a);
}



void
show_nearby(void)

{
  /*  The window in not created yet, do it now  */
  if (state.matchedobjects == NULL) {
    state.matchedobjects = create_form_Matched_objects();
    fl_set_form_minsize(state.matchedobjects->Matched_objects, 
			state.matchedobjects->Matched_objects->w, 
			state.matchedobjects->Matched_objects->h);
    fl_set_browser_fontsize(state.matchedobjects->matchchoicesW, 14);
    fl_set_browser_fontstyle(state.matchedobjects->matchchoicesW, FL_FIXED_STYLE);
  }
  if (!fl_form_is_visible(state.matchedobjects->Matched_objects)) {
    fl_show_form(state.matchedobjects->Matched_objects,  
		 FL_FIX_SIZE, FL_FULLBORDER | FL_PLACE_FREE_CENTER, 
		 "Astrometry report");
  }
}


void
hide_nearby(void)

{
  fl_hide_form(state.matchedobjects->Matched_objects);
}


/*===========================*/
/*  Identify a minor planet  */
/*===========================*/
void
identifyC(FL_OBJECT *obj, long val)

{
  ASTEROID asteroid[REZMAXNUM];
  double dist;
  int n = state.control[state.control_num];
  int i, k;
  int out;

  dist = strtod(fl_get_input(state.showcomet->cometdistW), NULL);
  /*===============================================================*/
  /*  Check which asteroids are within dist of the current object  */
  /*===============================================================*/
  k = check_asteroid(asteroid, frame[n].fitsfile.jd, cstar->ra, cstar->dec, dist, 
		 state.asteroid_server, state.asteroid_id, state.asteroid_maxmag);
  if (k <= 0) {
    fl_show_alert("Nothing found!", "", "", 1);
  }
  else {
    char a[70];
    float errmin = 1e10, imin = -1;
    fl_deactivate_all_forms();
    show_nearby();
    fl_clear_browser(state.matchedobjects->matchchoicesW);
    for (i = 0; i < k; i++) {
      sprintf(a, "%5d %19s %s %s %5.2f %8.2f", asteroid[i].num, asteroid[i].name,
	      format_ra(asteroid[i].ra), 
	      format_dec(asteroid[i].dec), asteroid[i].magnitude, asteroid[i].err);
      if (asteroid[i].err < errmin) {
	errmin = asteroid[i].err;
	imin = i;
      }
      fl_addto_browser(state.matchedobjects->matchchoicesW, a);
    }
    fl_select_browser_line(state.matchedobjects->matchchoicesW, imin + 1);

    out = 0;
    do {
      XEvent event;
      FL_OBJECT *o = fl_check_forms();
      if (o == FL_EVENT) {
	fl_XNextEvent(&event);
      }
      else if (o == state.matchedobjects->acceptobjectW) {
	/*  Find the selected object  */
	/*============================*/
	for (i = 0; i < k; i++) {
	  if (fl_isselected_browser_line(state.matchedobjects->matchchoicesW, i + 1)) {
	    char number[8];
	    if (asteroid[i].num == 0) {
	      designation_to_packed(asteroid[i].name, number);
	    }
	    else {
	      sprintf(number, "%d", asteroid[i].num);
	    }
	    fl_set_input(state.showcomet->objcometW, number);
	  }
	}
	out = 2;
      }
      else if (o == state.matchedobjects->cancelobjectW) {
	out = 1;
      }
    } while(!out);
    fl_activate_all_forms();
    hide_nearby();
  }
}


/*  Given a FITS file name and x, y coordinates create a .dat file name */
/*  and read records until a match with x1, y1 coordinate is found  */
/*  Return the read ra and dec  */

int
find_object_from_file(char *name, int n, double x, double y, STAR *obj)

{
  const char *fn;
  static char *curdir = "";
  static char *pattern = "*.dat";
  char *datname;
  char line[256];
  char *p;
  int k, m;
  FILE *fp;
  int rvalue = 0;

  make_output_filename(name, &datname, "dat");
  fn = datname;
  while (!(fp = fopen(fn, "r"))) {
    if (state.interactive) {
      fl_show_alert("Can not open file", fn, "for reading!", 0);
      fl_use_fselector(3);
      fl_invalidate_fselector_cache();
      fn = fl_show_fselector("Choose input file", curdir, pattern, datname);
      if (fn == NULL) {
	free(datname);
	return 0;
      }
      /*  Save the current directory  */
      curdir = (char *) fl_get_directory();
      /*  Save the current pattern */
      pattern = (char *) fl_get_pattern();
    }
  }

  fgets(line, 255, fp);
  /*  Read the magnitude factor  */
  p = strstr(line, "magfac: ");
  if (p) {
    frame[n].match.magfac = strtod(p + strlen("magfac:"), NULL);
    frame[n].matched = 1;
  }
  if (frame[n].match.magfac == 0) {
    frame[n].matched = 0;
  }
  /*  Read all stars  */
  while (!feof(fp)) {
    /*  Get a line  */
    fgets(line, 255, fp);
  
    if (!feof(fp)) {
      char c = 0;
      double intens;
      obj->ra = 0.0;
      obj->dec = 0.0;
      obj->catnum = 0;
      m = sscanf(line, "%d %lf %lf %lf %lf %lf %c %s %f", &k, 
		 &obj->x, &obj->y, &intens, &obj->ra, &obj->dec, 
		 &c, obj->catid, &obj->err);
      obj->intensity = intens;
      obj->catalog = c;
      if (m >= 3) {
	/*  See if the coordinates match  */
	if (fabs(x - obj->x) < 1.0 && fabs(y - obj->y) < 1.0) {
	  if (m < 6) {
	    fl_show_alert("No astrometry|", 
			  "Object was found, but no astrometry was done.", "",
			  1);
	    rvalue = 0;
	  }
	  else {
	    rvalue = 1;
	  }
	  break;
	}
      }
    }
  }
  fclose(fp);

  free(datname);
  return rvalue;
}


/*  Read astrometry data from the .dat files on the basis of currently */
/*  chosen marker  */

void
autoastrC(FL_OBJECT *obj, long val)

{
  double x, y;
  int i;
  int imarker = -1;
  int n = state.control[state.control_num];
  char *name;
  STAR star;

  name = strdup(fl_get_input(state.showcomet->objcometW));
  trim_spaces(name);
  if (strlen(name) == 0) {
    fl_show_alert("Please enter object designation!", "", "", 1);
    free(name);
    return;
  }

  /*  Coordinates to search for in the current image  */
  x = cstar->x;
  y = cstar->y;
  /*  Find the marker with coordinates x, y  */
  for (i = 0; i < frame[n].objnum; i++) {
    if (fabs(frame[n].obj[i].x - x) < 0.01 &&
	fabs(frame[n].obj[i].y - y) < 0.01) {
      /*  The marker is found, remember its index and exit loop */
      imarker = i;
      break;
    }
  }
  if (imarker == -1) {
    fl_show_alert("Marker not found!", "This should never happen", "", 1);
    free(name);
    return;
  }
  else {
    char a[256], b[10];
    /*  Find imarker-th marker in each image  */
    for (i = 0; i < state.frame_used; i++) {
      if (imarker < frame[i].objnum) {
	x = frame[i].obj[imarker].x;
	y = frame[i].obj[imarker].y;

	if (find_object_from_file(frame[i].fits.name, i, x, y, &star)) {
	  if (strlen(name) > 7) 
	    name[8] = '\0';
	  if (strlen(name) > 0 && isalpha(name[0])) {
	    if (i == 0 && strncmp(name, state.desig_prefix, strlen(state.desig_prefix)) == 0&&
		state.desig == strtol(name + strlen(state.desig_prefix), NULL, 10)) {
	      sprintf(b, "%-7s*", name);
	    }
	    else {
	      sprintf(b, "%-7s ", name);
	    }
	    sprintf(a, "     %-8s %c%4d %02d %08.5f %11s %11s          %4.1f %s      %3s", 
		    b, 'C', frame[i].year, frame[i].month,
		    frame[i].day +
		    (frame[i].hour + 
		     (frame[i].minute + frame[i].second / 60.0) / 60.0) / 24.0,
		    format_ra_mpc(star.ra),
		    format_dec_mpc(star.dec), 2.5 * (frame[i].match.magfac - log10(fabs(star.intensity))),
		    frame[i].match.filter, observatory.code);
	  }
	  else {
	    int number = strtol(name, NULL, 10);
	    sprintf(a, "%05d         %c%4d %02d %08.5f %11s %11s          %4.1f %s      %3s",
		    number, 'C', frame[i].year, frame[i].month,
		    frame[i].day + 
		    (frame[i].hour + 
		     (frame[i].minute + frame[i].second / 60.0) / 60.0) / 24.0,
		    format_ra_mpc(star.ra),
		    format_dec_mpc(star.dec), 2.5 * (frame[i].match.magfac - log10(fabs(star.intensity))),
		    frame[i].match.filter, observatory.code);
	  }
	  if (!state.mail) {
	    edit_mail(state.blinker->astrometryW, SEND_MAIL);
	    fl_set_menu_item_mode(state.blinker->astrometryW, SEND_MAIL, FL_PUP_NONE);
	    fl_hide_form(state.mail->Mail);
	  }
	  fl_addto_browser(state.mail->messageW, a);
	}
	else {
	  sprintf(a, "Object with coordinates x=%.2f, y=%.2f is not found in image", 
		  x, y);
	  fl_show_alert("Object not found!", a, frame[i].fits.name, 1);
	}
      }
      else {
	sprintf(a, "Image %s does not have marker no. %d!", 
		frame[i].fits.name, imarker);
	fl_show_alert(a, "", "", 1);
      }
    }
  }    
  free(name);
  return;
}




void
innercircleC(FL_OBJECT *obj, long val)

{
  int zoom = 5;
  FL_OBJECT *o;
  int n = state.control[state.control_num];
  char a[20];
  double ac, dc;
  int err = 0;
  OBJEKT obj1;

  frame[n].fits.inner = fl_get_counter_value(obj);
  o = state.showcomet->windcometW;
  obj1.xf = cstar->x;
  obj1.yf = cstar->y;
  fl_set_counter_bounds(state.showcomet->outercircleW, frame[n].fits.inner + 1.0, 50.0);
  cstar->intensity = aperture(&frame[n].fits, &obj1, 
	   frame[n].fits.inner, frame[n].fits.outer, FIND_PEAK);
  cstar->x = obj1.xt;
  cstar->y = obj1.yt;
  draw_object(o, cstar, zoom, &frame[n].fits, frame, n);
  sprintf(a, "x = %.2f", cstar->x);
  fl_set_object_label(state.showcomet->xcometW, a);
  sprintf(a, "y = %.2f", cstar->y);
  fl_set_object_label(state.showcomet->ycometW, a);
  sprintf(a, "v = %d", cstar->intensity);
  fl_set_object_label(state.showcomet->vcometW, a);
  if (frame[n].fitsfile.wcs) {
    fits_pix_to_world(cstar->x + 1, cstar->y + 1, 
		      frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
		      frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2, 
		      frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2, 
		      frame[n].fitsfile.crot,
		      frame[n].fitsfile.ctype,
		      &ac, &dc, &err);
    if (ac >= 360.0) {
      ac -= 360.0;
    }
    cstar->ra = ac; 
    cstar->dec = dc; 
    sprintf(a, "RA = %s", format_ra(ac));
    fl_set_object_label(state.showcomet->racometW, a);
    sprintf(a, "DEC = %s", format_dec(dc));
    fl_set_object_label(state.showcomet->deccometW, a);
  }
  if (frame[n].matched) {
    fl_show_object(state.showcomet->magcometW);
    sprintf(a, "m = %.2f", 2.5 * (frame[n].match.magfac - log10(fabs(cstar->intensity))));
    fl_set_object_label(state.showcomet->magcometW, a);
  }
}


void
outercircleC(FL_OBJECT *obj, long val)

{
  int zoom = 5;
  FL_OBJECT *o;
  int n = state.control[state.control_num];
  char a[20];
  double ac, dc;
  int err;
  OBJEKT obj1;

  frame[n].fits.outer = fl_get_counter_value(obj);
  o = state.showcomet->windcometW;

  obj1.xf = cstar->x;
  obj1.yf = cstar->y;
  fl_set_counter_bounds(state.showcomet->innercircleW, 0.1, frame[n].fits.outer - 1.0);
  cstar->intensity = aperture(&frame[n].fits, &obj1, 
	   frame[n].fits.inner, frame[n].fits.outer, FIND_PEAK);
  cstar->x = obj1.xt;
  cstar->y = obj1.yt;
  draw_object(o, cstar, zoom, &frame[n].fits, frame, n);
  sprintf(a, "x = %.2f", cstar->x);
  fl_set_object_label(state.showcomet->xcometW, a);
  sprintf(a, "y = %.2f", cstar->y);
  fl_set_object_label(state.showcomet->ycometW, a);
  sprintf(a, "v = %d", cstar->intensity);
  fl_set_object_label(state.showcomet->vcometW, a);
  if (frame[n].fitsfile.wcs) {
    err = 0;
    fits_pix_to_world(cstar->x + 1, cstar->y + 1, 
		      frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
		      frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2, 
		      frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2, 
		      frame[n].fitsfile.crot,
		      frame[n].fitsfile.ctype,
		      &ac, &dc, &err);
    if (ac >= 360.0) {
      ac -= 360.0;
    }
    cstar->ra = ac; 
    cstar->dec = dc; 
    sprintf(a, "RA = %s", format_ra(ac));
    fl_set_object_label(state.showcomet->racometW, a);
    sprintf(a, "DEC = %s", format_dec(dc));
    fl_set_object_label(state.showcomet->deccometW, a);
  }
  if (frame[n].matched) {
    fl_show_object(state.showcomet->magcometW);
    sprintf(a, "m = %.2f", 2.5 * (frame[n].match.magfac - log10(fabs(cstar->intensity))));
    fl_set_object_label(state.showcomet->magcometW, a);
  }
}


void
cometdesigC(FL_OBJECT *obj, long val)

{
  char a[10];
  state.desig++;
  if (state.desig == 10000) state.desig = 0;
  sprintf(a, "%s%04d", state.desig_prefix, state.desig);
  fl_set_input(state.showcomet->objcometW, a);
  save_designation();
}



/*  Take a designation and convert it to the MPC format */
/*  This function assumes that name has at least 14 bytes of space allooated  */
void
adjust_name_to_mpc(char *name)

{
  int i, first, last, discovery;
  int num = 1;
  char b[15];

  last = strlen(name) - 1;
  /*  First check if all of the characters are numbers  */
  for (i = 0; i <= last; i++) {
    if (!isdigit(name[i])) {
      num = 0;
      break;
    }
  }
  if (num && i < 5) {
    /* All characters are numbers, format as a number  */
    sprintf(b, "%05d        ", (int) strtol(name, NULL, 10));
  }
  else {
    first = 0;
    /*  Read the string up to the first alphanumerice character */
    while (first <= last && !isalnum(name[first])) first++;
    while (last >= first && !isalnum(name[last])) last--;
    discovery = (name[last + 1] == '*');
    name[last + 1] = '\0';
    /*  Now check if we have a comet designation  */
    /*  This is defined either by the last character beiing P, C, D, X or A */
    /*  and all other characters being numbers  */
    /*  or by the first character being one of the same and the total string */
    /*  length more than 7 */
    if (name[last] == 'P' || name[last] == 'C' || name[last] == 'D' || name[last] == 'X' ||
	name[last] == 'A') {
      /* Check if other characters are numbers  */
      if (last - first <= 3) {
	num = 1;
	for (i = first; i < last; i++) {
	  if (!isdigit(name[i])) {
	    num = 0;
	    break;
	  }
	}
	if (num) {
	  sprintf(b, "%04d%c        ", (int) strtol(name + first, NULL, 10), name[last]);
	}
	else {
	  sprintf(b, "     %s", name + first);
	}
      }
      else {
	sprintf(b, "     %s", name + first);
      }
    }
    else if (last - first == 7 && (name[first] == 'P' || name[first] == 'C' || name[first] == 'D' 
				  || name[first] == 'X' || name[first] == 'X')) {
      sprintf(b, "    %-9s", name + first);
    }
    else {
      name[first + 8] = '\0';
      sprintf(b, "     %-7s%c", name + first, (discovery) ? '*' : ' ');
    }
  }
  strcpy(name, b);
}


/*  Show an object */
int
showcomet(BLINK_FRAME *frame, int n, STAR *z, int ok, char *name, int aperture)

{
  double ac, dc;
  char a[25];
  int out, err = 0;
  XEvent event;
  FL_OBJECT *o;

  cstar = z;
  /*  The window in not created yet, do it now  */
  if (state.showcomet == NULL) {
    state.showcomet = create_form_Comet();
    fl_set_object_shortcut(state.showcomet->acceptcometW, "^m", 1);
    fl_set_object_shortcut(state.showcomet->okcometW, "^m", 1);
    fl_set_input(state.showcomet->cometdistW, "1.00");
  }
  if (ok) {
    fl_hide_object(state.showcomet->acceptcometW);
    fl_hide_object(state.showcomet->rejectcometW);
    fl_show_object(state.showcomet->okcometW);
    fl_show_object(state.showcomet->autoastrW);
  }
  else {
    fl_show_object(state.showcomet->acceptcometW);
    fl_show_object(state.showcomet->rejectcometW);
    fl_hide_object(state.showcomet->okcometW);
    fl_hide_object(state.showcomet->autoastrW);
    fl_activate_object(state.showcomet->objcometW);
  }
  if (frame[n].matched) {
    fl_show_object(state.showcomet->magcometW);
    sprintf(a, "m = %.2f", 2.5 * (frame[n].match.magfac - log10(fabs(z->intensity))));
    fl_set_object_label(state.showcomet->magcometW, a);
  }
  else {
    fl_hide_object(state.showcomet->magcometW);
  }
  if (aperture) {
    fl_show_object(state.showcomet->innercircleW);
    fl_show_object(state.showcomet->outercircleW);
    fl_set_counter_bounds(state.showcomet->innercircleW, 0.1, frame[n].fits.outer - 1.0);
    fl_set_counter_step(state.showcomet->innercircleW, 0.1, 1);
    fl_set_counter_bounds(state.showcomet->outercircleW, frame[n].fits.inner + 1.0, 30.0);
    fl_set_counter_step(state.showcomet->outercircleW, 0.1, 1);
    fl_set_counter_value(state.showcomet->innercircleW, frame[n].fits.inner);
    fl_set_counter_value(state.showcomet->outercircleW, frame[n].fits.outer);
  }
  else {
    fl_hide_object(state.showcomet->innercircleW);
    fl_hide_object(state.showcomet->outercircleW);
  }
  fl_show_form(state.showcomet->Comet,  FL_FIX_SIZE, FL_FULLBORDER | FL_PLACE_FREE_CENTER, "Clicked object");
  /*  Write object coordinates in the window  */
  sprintf(a, "x = %.2f", z->x);
  fl_set_object_label(state.showcomet->xcometW, a);
  sprintf(a, "y = %.2f", z->y);
  fl_set_object_label(state.showcomet->ycometW, a);
  sprintf(a, "v = %d", z->intensity);
  fl_set_object_label(state.showcomet->vcometW, a);
  if (frame[n].matched) {
    sprintf(a, "RA = %s", format_ra(z->ra));
    fl_set_object_label(state.showcomet->racometW, a);
    sprintf(a, "DEC = %s", format_dec(z->dec));
    fl_set_object_label(state.showcomet->deccometW, a);
  }
  else if (frame[n].fitsfile.wcs) {
    fits_pix_to_world(z->x + 1, z->y + 1, 
		      frame[n].fitsfile.crval1, frame[n].fitsfile.crval2, 
		      frame[n].fitsfile.crpix1, frame[n].fitsfile.crpix2, 
		      frame[n].fitsfile.cdelt1, frame[n].fitsfile.cdelt2, 
		      frame[n].fitsfile.crot,
		      frame[n].fitsfile.ctype,
		      &ac, &dc, &err);
    if (ac >= 360.0) {
      ac -= 360.0;
    }
    sprintf(a, "RA = %s", format_ra(ac));
    fl_set_object_label(state.showcomet->racometW, a);
    sprintf(a, "DEC = %s", format_dec(dc));
    fl_set_object_label(state.showcomet->deccometW, a);
    cstar->ra = ac;
    cstar->dec = dc;
  }
  else {
    fl_set_object_label(state.showcomet->racometW, " ");
    fl_set_object_label(state.showcomet->deccometW, " ");
  }
  if (name != NULL) {
    fl_set_input(state.showcomet->objcometW, name);
  }
  out = 0;
  fl_deactivate_all_forms();
  fl_activate_form(state.showcomet->Comet);
  do {
    o = fl_check_forms();
    if (o == FL_EVENT) {
      fl_XNextEvent(&event);
    }
    else if (o == state.showcomet->acceptcometW) {
      out = 2;
      strncpy(name, fl_get_input(state.showcomet->objcometW), 30);
    }
    else if (o == state.showcomet->rejectcometW) {
      out = 1;
    }
    else if (o == state.showcomet->okcometW) {
      out = 3;
    }
  } while(!out);
  fl_hide_form(state.showcomet->Comet);
  fl_activate_all_forms();
  /*  Turn the designation in a form suggested by MPC  */
  if (out == 2) adjust_name_to_mpc(name);
  return out - 1;
}

