/* structs.h for fitsblink 2,0   */
/* Data structures  */
#ifndef _blink_structs
#define _blink_structs
#include <forms.h>
#include <fitsio.h>
#include "consts.h"
#include "formblink.h"

typedef unsigned short int PIXEL;
typedef unsigned char BYTE;

typedef
struct {
  int simple;
  long bitpix;
  long naxis, nax[2];
  fitsfile *fptr;
  char ctype1[9];
  char ctype2[9];
  double crval1, crval2;  /* Celestial coordinates of the reference pixel (in decimal degrees) */
  double crpix1, crpix2;  /* Image coordinates of the reference pixel  */
  double cdelt1, cdelt2;  /* Coordinate incremement at reference pixel  */
  double crot;            /* Rotation angle (should be the same for both axis */
  char ctype[5];          /* Projection type (like -TAN)  */
  short wcs;              /* Flag which is set if wcs coordinates are present  */
  int matched;          /* This is set if WCS coordinates were matched to catalog */
  float sizex, sizey;
  char telescope[FLEN_VALUE];
  char instrument[FLEN_VALUE];
  char observer[FLEN_VALUE];
  char date_obs[FLEN_VALUE];
  char filter[2];
  char ut_start[20];
  char ut_cent[20];
  char ut_end[20];
  char time_beg[20];
  char time_end[20];
  char object[FLEN_VALUE];     /*  Name of the object  */
  /*  Flags which tell whether specific keyword was read  */
  short is_date_obs;
  short is_exposure;
  short is_ut_start;
  short is_ut_cent;
  short is_ut_end;
  short is_time_beg;
  short is_time_end;
  short is_gain;
  short is_rdnoise;
  char time[9];
  float exposure;
  float gain, rdnoise;
  float ccd_temp;
  float temperat, pressure, humidity;
  float equinox;
  char *name;
  double jd;
  char **key;
  int nkeys;
} FITSFILE;


  
/*  Structure which contains complete information about a FITS table  */
typedef 
struct {
  char *name;  /*  Name of the file  */
  int hdu;     /*  Number of the HDU */
  int ncol;   /*  Number of columns */
  int nrows;  /*  Number of rows    */
  char **colnames;  /*  Array of column names  */
  short *selected;  /* Array of selected columns */
  char **lines; /*  Array of ASCII lines with selected table content */
  short *width; /*  Widths of columns in file */
  short *swidth; /*  Widths of columns in browser */
  short *type;  /*  Column types  */
  char **key;
  int nkeys;
  short selections_only;
} FITS_TABLE;

typedef
struct slika {
  int width, height;  /* Width and height of the image */
  int  x1, y1, x2, y2;/* Coordinates of the active rectangle inside the image */
  PIXEL *image;       /* Pointer to image */
  PIXEL **pi;         /* Array of pointers to image rows  */
  float fitmeasure;   /*  Ratio between fitted and measured brightness 
			  (and vice versa) of star which is still accepted  */
  float minbright;      /*  Minimum still accepted brightness  */
  int minstar;
  int maxsize;      /*  Size of the window inside which the average background is calculated  */
  float sg_num;     /*  Sensitivity threshold expressed in standard deviations of the background */
  short border;       /* The width of the region on the image edge where no star is reported  */
  int satlevel;   /* Saturation level  */
  int aperture;   /* Flag which tells if we do aperture astrometry  */
  float inner;      /* radius of the inner circle for the aperture photometry  */
  float outer;      /* radius of the outer circle for the aperture photometry  */
  float resolve;    /* Coefficient used is resolving close stars  */
  short blur;         /* Do we use bluring */
  short median;     /* Do we use median filtering  */
  float gain;       /*  Camera gain  */
  float rdnoise;    /*  Readout noise  */
  char *name;       /*  name od the image  */
} FITS_IMAGE;


/*  Structure with coordinates highlighted in the image */
typedef struct {
  float x, y;  /* Coordinates of the center  */
  float r;     /* radius of the circle  */
  char *s;     /* Name of the object  */
  short color; /* Object color  */
  short shape; /* Object shape  */
}  OBJECT;

/*  Data describing window showing stars  */
typedef
struct {
  double ra, dec;  /*  RA and rec in decimal degrees  */
  double width, height;  /*  width and height in decimal degrees */
  double fi;
  double w, h;        /* width and height in pixels  */
  float minmag, maxmag;  /*  Minimum and maximum acceptable magnitudes */
  int minsize, maxsize;  /* Minimum and maximum size of star  */
  float extarea; /*  Extension in direction used for matching  */
  int nm;       /*  Number of stars in memory */
} MAP_IMAGE;

/*  A structure which describes a single star from a catalog  */
typedef
struct {
  short catalog;  /*  Catalog code: ('u'=USNO, 'g' = GSC, ...)  */
  double catnum; /*  Catalog number  */
  char catid[16];  /* String representation of object's id  */
  int n;     /*  Star number  */
  int m;     /*  Index of the matched star from the image  */
  double x, y;  /*  Star screen coordinates */
  float sxt, syt;  /* Error estimates for object's image coordinates */
  int area;        /* Area of the object in pixels  */
  double ra, dec;    /*  Star space coordinates  */
  float sra, sdec;  /* Error estimates for coordinates  */
  float bluemag, redmag, mag;  /*  Star magnitudes  */
  float smag;    /*  Error estimate for the magnitude */
  float sigma;   /*  Star width  */
  PIXEL maxval;  /*  Intensity of the brightest pixel  */
  int intensity; /*  Intensity of the object */
  float sintensity;  /*  Estimated arror in intensity */
  float bg;      /*  Background/pixel estimate at the objects position  */
  int q;     /*  Indicator whether magnitudes can be trusted  */
  short saturated;  /* Indicator whether the object contains saturated pixels */
  float fitmag;  /*  Fitted star intensity  */
  float sym;     /* Symmetry indicator  */
  float err; /*  Distance between the imaged in catalogued star  */
}  STAR;

typedef
struct {
  int n;
  STAR *s;
  float sp;
} STARS;

/*  Structure which hold values needed for matching subroutine  */
typedef
struct {
  float min;        /* Minimum acceptable value of constellation to accepted as a match  */
  int nstar;        /* Number of neighbouring stars in constellation  */
  int firstbright;  /* Number of bright stars used for initial matching  */
  float poserrmax;     /* Maximum positional error allowed for acceptance of a star in costellation  */
  float magerrmax;     /* Maximum magnitude error (factor) allowed for acceptance a a star in constellation  */
  float maxres;        /* Maximal allowed residual  */
  float mindist;    /*  minimal allowed distance between stars for initial */
  /*                    matching (in pixels)   */
  int fitorder;        /* order of plate fitting */
  int fitorder_used;   /* Actually used order of plate fitting */
  double xcoef[21], ycoef[21];  /* Coefficients of the plate solution  */
  float magfac;     /* Ratio between instrumental and catalog magnitude  */
  char filter[2];
  short matched;    /*  Was the matching successfully performed?  */
  int nmatched;     /*  Number of matched stars  */
} MATCH_OPTIONS;


/*  Structure carrying information about individual images  */
typedef struct {
  float dx, dy;  /*  Translation of the image relative to a reference image */
  float alfa;    /*  Angle of rotation */
  float a, b, d, e;   /*  Koeficients relating to angle od rotation and extension */
  double ra, dec; /* right ascension and declination of the image  */
  double fi;     /* Angle of rotation from north to east  */
  double xsize, ysize;  /* Sizes of pixel in x and y direction  */
  int year, month, day; /* Time when the image was taken  */
  int hour, minute;
  float second;
  float *gamx, *gamy;  /*  Lists of values for grey level correction */
  float *kg;    /* list of coefficients for grey level correction */
  int ngam;     /*  Number of points for grey level correction */
  int low, high;  /*  The lowest and the highest value in the image  */
  float *histx, *histy;  /*  Lists of values for the histogram  */
  float xmin, xmax;   /*  Histogram range  */
  int nhisto;   /*  Number of values in the histogram */
  char *name;   /*  The name of the image  */
  char *name1;  /*  Name of the image read by load_star_list  */
  Pixmap  pixmap;  /*  Image as shown in X window  */
  FITS_IMAGE fits;      /*  Structure with information about FITS image */
  FITS_IMAGE astrwin;   /*  Small image inside which an object is located */
  FITSFILE fitsfile;
  OBJECT *obj;
  int objnum;       /*  Number of markers loaded to memory  */
  int objmax;       /*  maximum number of markers  */
  short objshow;    /*  A flag which tells whether the markers are shown  */
  int grey_max;     /*  Maximum grey level value for the screen  */
  short detect;     /*  Set to 1 if any stars were detected  */
  short matched;      /*  Set to 1 if star matching was successful  */
  MATCH_OPTIONS match;
  MAP_IMAGE map;
  STARS imagelist;
  STARS cataloglist;   /*  List of stars from the catalog  */
  FITS_IMAGE last_count_options;
  MATCH_OPTIONS last_match_options;
}  BLINK_FRAME;

/*=========================================================================*/
/*  Structure with the information about the current state of the program  */
/*=========================================================================*/

typedef struct {
  int control_num;  /*  Number of the currently selected control (0 or 1)  */
  int control[2];  /*  Numbers of images belonging to the first and second control */
  int frame_max;   /*  Number reserved spaces for frames  */
  int frame_used;  /*  Number of actually used frames  */
  Display *disp;   /*  Pointer to X11 display structure */
  int bw, bh;      /*  Dimensions of the blinking area in pixels  */
  FD_setting *gui; /*  Pointer to structure of control window  */
  FD_blinker *blinker;  /*  Pointer to blinker structure  */
  FL_COORD blinkwidth;  /*  Width  */
  FL_COORD blinkheight; /*  and height of the blinker window  */
  FD_Edit *browser[2];  /* Pair of pointers to browsers  */
  FD_Error *error;  /*  Browser for the output of errors */
  FD_help *readme;  /*  Window with the README file  */
  FD_Parameters *parameters;  /*  Image parameters  */
  FD_Report *report;/*  Report window  */
  FD_Options *options; /* astrometry options  */
  FD_Catalog *catalogs; /* catalog options  */
  FD_Mail *mail;
  FD_Wcs *newwcs; 
  FD_Settings *settings;
  FD_Set_catalogs *set_catalogs;
  FD_Set_observatory *set_observatory;
  FD_Set_display *set_display;
  FD_Set_mail *set_mail;
  FD_Set_help *set_help;
  FD_Commands *set_commands;
  FD_Set_asteroid *set_asteroid;
  FD_Set_ccd *set_ccd;
  FD_Set_count *set_count;
  FD_Set_match *set_match;
  FD_Set_misc *set_misc;
  FD_Comet *showcomet;
  FD_Telescope *telescope;
  FD_Message *message;
  FD_Matched_objects *matchedobjects;
  FD_Dss *dss;
  FD_Table *table;
  FD_Tablebrowser *tablebrowser;
  FD_Tableheader *tableheader;
  FD_Tablefields *tablefields;
  FD_Tablegraph *tablegraph;
  
  int b[2];         /* */
  short zoom_width;   /*  Number of magnification (zoom) */
  short freeze;       /*  Flag which is set if the zoom windows are locked  */
  int xf, yf;       /*  Coordinates of the zoom window when the zoom is locked */
  short blink;        /*  Am I in a blink mode ?  */
  int blinknum;     /*  Number of the image shown in the blink mode  */
  int grey_max;
  short color;      /* Is display in color mode (1) or grey level (0) ? */
  short inverse;    /* Is display inversed after autoleveling?  */
  unsigned long pixel[GREY_WANTED];  /*  Index for X colors  */
  char *logfile;
  FILE *logfp;
  short interactive;
  short verbose;
  short script;
  short doastr;
  float delay;
  float autolevel_high;
  int timeout;
  char *wwwbrowser;
  char *helpfile;
  char *asteroid_server;
  char *asteroid_id;
  float asteroid_maxmag;
  short starlistformat;  /*  Format of the star list file  */
  float dss_big;
  char *dss_server;
  char desig_prefix[4];
  int  desig;
} STATE;


/*======================================================*/
/* Structure with the information about the observatory */
/*======================================================*/
typedef
struct {
  char code[4];
  char observer[CARDLENGTH];
  char telescope[CARDLENGTH];
  char instrument[CARDLENGTH];
  double longitude;
  double latitude;
  double height;        /* Height above the sea level  */
  double sinfi, cosfi;  /*  Parallactic coefficients */
} OBSERVATORY;


/*======================================================*/
/* Current telescope and camera settings                */
/*======================================================*/
typedef
struct {
  double ra, dec; /*  right ascension and declination  */
  double alt, az; /*  altitude and azimuth  */
  double jd, sr;  /*  Julian day and sidereal time  */
  double exptime; /*  exposure time  */
  double temp;    /*  Camera temperature  */
  char *filename; /*  output file name  */
  short dark;     /*  dark image indicator  */
  char start[256];
  char point[256];
  char park[256];
  char where[256];
  char zero[256];
  char center[256];
  char gettemp[256];
  char settemp[256];
  char expose[256];
} TELESCOPE;


/*================================================*/
/* Structure with information about email report  */
/*================================================*/
typedef
struct {
  char *to;    /* Address to which we send  */
  char *cc;    /* A list of cc addresses    */
  char *title; /* Title of the message      */
  char *file;  /* Name of the file which contains the body of message  */
} MAIL;


/*  Structures for catalogs  */
/*===========================*/

/*  Description of catalogs  */
typedef
struct {
  int use;      /* Set to TRUE if the catalog is used  */
  int lpath;    /* Length of the string, reserved for path to the catalog  */
  int passband; /* Passband which has to be used  */
  char *filter[FILTERNUM];  /*  Filter names  */
  char *name;   /* Catalog name  */
  char *path;   /* full path to the catalog  */
} CATALOG;

/**********/
/*  USNO  */
/**********/
/*  A list of bytes definining one record in USNO catalog  */
typedef
struct {
  unsigned char a[USNO_RECORDLENGTH];
} USNO_STAR;

/*  One entry from an accelerator file  */
typedef
struct {
  float ra;
  int index, length;
} ACCELERATOR;

/*********  USNO end  ***********/


/**********************/
/*     GSC            */
/**********************/
typedef
struct {
  int id, plate;
  double ra, dec, poserr;
  float mag, magerr;
  short magband;
  short class;
} GUIDE_STAR;

typedef
struct {
  double center;  /* center of declination band  */
  short numlarge;   /* Number of large regions in declination band  */
  short firstlarge; /* First large region in declination band */
} LARGE_REGION;

typedef
struct {
  short largeid;    /*  Id of the large region  */
  short firstsmall; /* id of first small region in large region  */
  short depth;      /*  Depth of subdivision of large region  */
} SMALL_REGION;

/****** GSC end  *********/

/**********************/
/*  ACT star catalog  */
/**********************/
typedef
struct {
  double id;  /*  TYCHO ID  */
  double ra;        /* Right ascension in decimal degrees  */
  double dec;       /* Declination in decimal degrees  */
  float pmra;      /* proper motion in ra (sec of time / year)  */
  float pmdec;     /* proper motion in dec (arcsec / year)  */
  float sra, sdec; /* standard deviation in ra and dec (mas)  */
  float spmra, spmdec;  /*  Standard deviations of proper motion mas/yr */
  float bmag;      /*  B magnitude  */
  float vmag;      /*  V magnitude  */
  int q;
}  ACT_STAR;

typedef
struct {
  float limit;
  int num;
  double reallimit;
} ACT_ACCEL;



/*********** ACT end  ************/

/************  TYCHO 2 ***************/
typedef
struct {
  char catid[14];  /* String representation of object's id  */
  char meanposflag;  /* Contains ' ' for normal mean position, 
			'P' for B_T photo-centre of two Tycho-2 entries,
			'X' no mean position, no proper motion  */
  double ra, dec;   /*  mean position at epoch J2000 */
  float pmra;      /* proper motion in ra (marcsec / year)  */
  float pmdec;     /* proper motion in dec (marcsec / year)  */
  float sra, sdec; /* Model based positional uncertainties at mean epoch */
  float spmra, spmdec; /* Model based uncertainties of proper motion at mean epoch */
  float mepra, mepdec;  /*  Mean epochs of ra and dec  */
  int n;               /*  Number of positions used  */
  float g_mra, g_mdec, g_pmra, g_pmdec;  /*  Goodness of fit parameters  */
  float bt, sbt;     /*  Tycho-2 BT magnitude and uncertainty */
  float vt, svt;     /*  Tycho-2 BT magnitude and uncertainty */
  float prox;        /* Proximity indicator  */
  char tycho1flag;   /*  Tycho 1 star flag  */
  int hip;           /* Hipparcos number  */
  char ccdm[4];      /* CCDM component identifier for HIP stars  */
  double raobs, decobs;  /* Observed TYCHO-2 positions, ICRS  */
  float epra, epdec;  /*  Epochs-1990 of raobs and decobs  */
  float e_ra, e_dec;  /* Model-based uncertainty of observed position  */
  char  posflg;       /* Type of TYCHO-2 solution  */
  float corr;         /* Correlation of observed position  */
} TYCHO2_STAR;


/************  TYCHO 2 end ***********/

/*======================================*/
/*  Structures for starcount functions  */
/*======================================*/

typedef 
struct {
  int x, y;     /*      Koordinati in vrednost tocke  */
  PIXEL v;
} POINT;

typedef 
struct {
  int xf, yf;	        /* First point in object found by the program  */
  double xt, yt;	/* Centroid coordinates */
  float sxt, syt;       /* Uncertainty estimates of the coordinates  */
  int x1, y1, x2, y2;	/* Limiting coordinates  */
  long area;	        /* Number of pixels belonging to the object  */
  long perim;	        /* Number of pixels in object which lie on the edge */
  long p_out;	        /* Number of points touching the object from outside  */
  long grey;            /* Sum of grey level values inside the object  */
  float sgrey;          /* Error estimate of the grey level  */
  float  bg;            /* Average background value inside the object  */
  PIXEL maxval;
  POINT *outer;	        /* Pointer to the list of outside edge points  */
  POINT *inner;	        /* Pointer to the list of inside points    */ 
  int xu;               /* Coordinate of a point on the upper edge of the object */
  short saturated;      /* Set if the object contains saturated pixel(s)  */
  float sym;            /* Symmetry indicator  */
  int n;
} OBJEKT;

typedef struct {
  BYTE *s, **p;	/*	Kazalec na sliko  */
  int width;  /*  Sirina slike v BYTE-ih  * 8  (torej v pixlih) */
  long size;  /*	Velikost  slike  v bytih  */
  int x0, y0;  /*	Offset glede na koordinate objekta na sliki  */
} SHADOW;

/*  Structure with data about Gaussian filter */
typedef struct {
  float sigma;   /*  Width of the gaussian in pixels  */
  int n;         /*  linear size of the array  */
  float r;       /*  radius of the circle taken into account  */
  float **g;     /*  Pointer to the coefficient array  */
} GAUSSF;


/*==========================================*/
/*  Structures for field_compare functions  */
/*==========================================*/
/*  Pair of star coordinates  */
typedef
struct {
  float x1, y1, x2, y2;
  float mag1, mag2;
  int n1, n2;
  float err;
} STAR_PAIR;


/*  Parameters of the coordinate rotation and translation and magnitude transfor
mation  */
typedef
struct {
  double a, b, c, d, e, f, hi;  /*  Translation and rotation  */
  double ma, mb, sma, smb;      /*  Magnitude transformation  */
} TRANSROT;
  

/*=======================================*/
/* Structures for the asteroid routines  */
/*=======================================*/
typedef
struct {
  double a;     /*    semimajor exis in AU     */
  double e;     /*    eccentricity             */
  double i;     /*    inclination              */
  double per;   /*    argument of perihelion   */
  double asc;   /*    longitude of ascending node  */
  double n;     /*    mean motion in degrees/day   */
  double perdist;/*    perihelion distance          */
  double anomaly;/*    mean anomaly at the epoch    */
  double t;     /*    time of perihelion           */
  double epoch; /*    epoch of osculation     */
  double equinox;/*   equinox of the orbital elements */
  double m;     /*  Mean anomaly  */
  double ecc;   /*  Eccentric anomaly */
  double nu;    /*  True anomaly  */
  double x, y, z;/* Heliocentric equatorial rectangular coordinates  */
  double rv;    /*  radius vector  (distance to Sun) */
  double delta; /*  distance to Earth  */
  float elongation;
  float phase_angle;
  double a1, a2;/*  Constants of motion, */
  double b1, b2;/*  calculated from the  */
  double c1, c2;/*  orbital elements     */
  double jd;    /*  Julian day for which the elements are valid  */
  double ra, dec;/*  Celestial coordinates at the time jd  */
  float magnitude;

  int num;   /* Asteroid number  */
  char name[19];  /* Asteroid name */
  float hmag;    /* absolute magnitude */
  float gmag;    /* slope parameter  */
  float color_index;  /*  Color index  */
  float diameter;
  int arc;     /* orbital arc in days */
  float err;
  int numobs;
}  ASTEROID;

#endif



