%module fitsblink
%{
#include "../functs.h"
#include <fitsio.h>
%}
%include array.i
%include constraints.i

/*  A structure which describes a single star from a catalog  */
typedef
struct {
  short catalog;  /*  Catalog code: ('u'=USNO, 'g' = GSC, ...)  */
  double catnum; /*  Catalog number  */
  char catid[16];  /* String representation of object's id  */
  int n;     /*  Star number  */
  int m;     /*  Index of the matched star from the image  */
  double x, y;  /*  Star screen coordinates */
  float sxt, syt;  /* Error estimates for object's image coordinates */
  int area;        /* Area of the object in pixels  */
  double ra, dec;    /*  Star space coordinates  */
  float sra, sdec;  /* Error estimates for coordinates  */
  float bluemag, redmag, mag;  /*  Star magnitudes  */
  float smag;    /*  Error estimate for the magnitude */
  int intensity; /*  Intensity of the object */
  float bg;      /*  Background/pixel estimate at the objects position  */
  int q;     /*  Indicator whether magnitudes can be trusted  */
  short saturated;  /* Indicator whether the object contains saturated pixels */
  float fitmag;  /*  Fitted star intensity  */
  float err; /*  Distance between the imaged in catalogued star  */
}  STAR;

typedef
struct {
  int n;
  STAR *s;
  float sp;
} STARS;

typedef unsigned short int PIXEL;
typedef struct {
  int control_num;  /*  Number of the currently selected control (0 or 1)  */
  int control[2];  /*  Numbers of images belonging to the first and second control */
  int frame_max;   /*  Number reserved spaces for frames  */
  int frame_used;  /*  Number of actually used frames  */
  Display *disp;   /*  Pointer to X11 display structure */
  int bw, bh;      /*  Dimensions of the blinking area in pixels  */
  FD_setting *gui; /*  Pointer to structure of control window  */
  FD_blinker *blinker;  /*  Pointer to blinker structure  */
  FL_COORD blinkwidth;  /*  Width  */
  FL_COORD blinkheight; /*  and height of the blinker window  */
  FD_Edit *browser[2];  /* Pair of pointers to browsers  */
  FD_Error *error;  /*  Browser for the output of errors */
  FD_help *readme;  /*  Window with the README file  */
  FD_Parameters *parameters;  /*  Image parameters  */
  FD_Report *report;/*  Report window  */
  FD_Options *options; /* astrometry options  */
  FD_Catalog *catalogs; /* catalog options  */
  FD_Mail *mail;
  FD_Wcs *newwcs; 
  FD_Settings *settings;
  FD_Set_catalogs *set_catalogs;
  FD_Set_observatory *set_observatory;
  FD_Set_display *set_display;
  FD_Set_mail *set_mail;
  FD_Set_help *set_help;
  FD_Commands *set_commands;
  FD_Set_asteroid *set_asteroid;
  FD_Set_ccd *set_ccd;
  FD_Set_count *set_count;
  FD_Set_match *set_match;
  FD_Set_misc *set_misc;
  FD_Comet *showcomet;
  FD_Telescope *telescope;
  FD_Message *message;
  FD_Matched_objects *matchedobjects;
  FD_Dss *dss;
  int b[2];         /* */
  short zoom_width;   /*  Number of magnification (zoom) */
  short freeze;       /*  Flag which is set if the zoom windows are locked  */
  int xf, yf;       /*  Coordinates of the zoom window when the zoom is locked */
  short blink;        /*  Am I in a blink mode ?  */
  int blinknum;     /*  Number of the image shown in the blink mode  */
  int grey_max;
  short color;      /* Is display in color mode (1) or grey level (0) ? */
  short inverse;    /* Is display inversed after autoleveling?  */
  unsigned long pixel[GREY_WANTED];  /*  Index for X colors  */
  char *logfile;
  FILE *logfp;
  short interactive;
  short verbose;
  short script;
  short doastr;
  float delay;
  float autolevel_high;
  int timeout;
  char *wwwbrowser;
  char *helpfile;
  char *asteroid_server;
  char *asteroid_id;
  float asteroid_maxmag;
  short starlistformat;  /*  Format of the star list file  */
  float dss_big;
  char *dss_server;
  char desig_prefix[4];
  int  desig;
} STATE;


typedef
struct {
  int simple;
  long bitpix;
  long naxis, nax[2];
  fitsfile *fptr;
  char ctype1[9];
  char ctype2[9];
  double crval1, crval2;  /* Celestial coordinates of the reference pixel (in decimal degrees) */
  double crpix1, crpix2;  /* Image coordinates of the reference pixel  */
  double cdelt1, cdelt2;  /* Coordinate incremement at reference pixel  */
  double crot;            /* Rotation angle (should be the same for both axis */
  char ctype[5];          /* Projection type (like -TAN)  */
  short wcs;              /* Flag which is set if wcs coordinates are present  */
  int matched;          /* This is set if WCS coordinates were matched to catalog */
  float sizex, sizey;
  char telescope[FLEN_VALUE];
  char instrument[FLEN_VALUE];
  char observer[FLEN_VALUE];
  char date_obs[FLEN_VALUE];
  char ut_start[20];
  char ut_cent[20];
  char ut_end[20];
  char time_beg[20];
  char time_end[20];
  char object[FLEN_VALUE];     /*  Name of the object  */
  /*  Flags which tell whether specific keyword was read  */
  short is_date_obs;
  short is_exposure;
  short is_ut_start;
  short is_ut_cent;
  short is_ut_end;
  short is_time_beg;
  short is_time_end;
  char time[9];
  float exposure;
  float ccd_temp;
  float temperat, pressure, humidity;
  float equinox;
  char *name;
  double jd;
} FITSFILE;


typedef
struct slika {
  int width, height;  /* Width and height of the image */
  int  x1, y1, x2, y2;/* Coordinates of the active rectangle inside the image */
  PIXEL *image;       /* Pointer to image */
  PIXEL **pi;         /* Array of pointers to image rows  */
  float fitmeasure;   /*  Ratio between fitted and measured brightness 
			  (and vice versa) of star which is still accepted  */
  float minbright;      /*  Minimum still accepted brightness  */
  int minstar;
  int maxsize;      /*  Size of the window inside which the average background is calculated  */
  float sg_num;     /*  Sensitivity threshold expressed in standard deviations of the background */
  short border;       /* The width of the region on the image edge where no star is reported  */
  int satlevel;   /* Saturation level  */
  short aperture;   /* Flag which tells if we do aperture astrometry  */
  float inner;      /* radius of the inner circle for the aperture photometry  */
  float outer;      /* radius of the outer circle for the aperture photometry  */
  short blur;       /* Do we use bluring */
  short median;     /* Do we use median filtering  */
  char *name;       /*  name od the image  */
} FITS_IMAGE;

/*  Structure carrying information about individual images  */
typedef struct {
  float dx, dy;  /*  Translation of the image relative to a reference image */
  float alfa;    /*  Angle of rotation */
  float a, b, d, e;   /*  Koeficients relating to angle od rotation and extension */
  double ra, dec; /* right ascension and declination of the image  */
  double fi;     /* Angle of rotation from north to east  */
  double xsize, ysize;  /* Sizes of pixel in x and y direction  */
  int year, month, day; /* Time when the image was taken  */
  int hour, minute;
  float second;
  float *gamx, *gamy;  /*  Lists of values for grey level correction */
  float *kg;    /* list of coefficients for grey level correction */
  int ngam;     /*  Number of points for grey level correction */
  int low, high;  /*  The lowest and the highest value in the image  */
  float *histx, *histy;  /*  Lists of values for the histogram  */
  float xmin, xmax;   /*  Histogram range  */
  int nhisto;   /*  Number of values in the histogram */
  char *name;   /*  The name of the image  */
  Pixmap  pixmap;  /*  Image as shown in X window  */
  FITS_IMAGE fits;      /*  Structure with information about FITS image */
  FITS_IMAGE astrwin;   /*  Small image inside which an object is located */
  FITSFILE fitsfile;
  OBJECT *obj;
  int objnum;       /*  Number of markers loaded to memory  */
  int objmax;       /*  maximum number of markers  */
  short objshow;    /*  A flag which tells whether the markers are shown  */
  int grey_max;     /*  Maximum grey level value for the screen  */
  short detect;     /*  Set to 1 if any stars were detected  */
  short matched;      /*  Set to 1 if star matching was successful  */
  MATCH_OPTIONS match;
  MAP_IMAGE map;
  STARS imagelist;
  STARS cataloglist;   /*  List of stars from the catalog  */
}  BLINK_FRAME;


/*  Call of initialization function  */
extern int main0(int, char *[]);

/*  Calculate the Julian day for the given arguments */
extern double julian_day(int, int, double);

/*  Load a FITS image into memory and show it if in interactive mode */
extern int load_image(char *, int);

/* Change display mode to color or grey level */
extern void colormap(long);

/* Resize the blinker window */
extern int resize_blinker(int, int, int, int);


/*  Resize the i-th image inside the blinker window  */
extern int resize_image(int, int, int, int, int);

/*  Show full i-th image inside the blinker window  */
extern int full_image(int);

/*  Switch markers on or off */
extern void markers(int);

/* Start or stop blinking  */
extern int blink(int);

/* Set delay time  */
extern int set_delay(float);

/* GUI loop  */
extern void loop(void);


int find_marker_color(char * NONNULL);
int find_marker_shape(char * NONNULL);



/*  Global structure with information about the current state of the
program */
extern STATE state;

/* Global structure with information about the images  */
extern BLINK_FRAME *frame;

/*  Create accessor function for BLINK_FRAME pointers  */
%addmethods BLINK_FRAME {
     BLINK_FRAME __getitem__(int i) {
         return frame[i];
      }
}

%addmethods STAR {
     STAR __getitem__(int i) {
         return self[i];
      }
}


%include exception.i
%addmethods STARS {
     STAR __getitem__(int i) {
         if (i < self[0].n) 
             return self[0].s[i];
         else {
	    STAR s;
	    /*	    throw_exception("Wrong image index");*/
	    return s;
         }
     }
     int __len__(void) {
         return self[0].n;
     }
}

extern int error_status;
extern char *check_exception(void);
%except(python) {
     char *err;
     error_status = 0;
     $function
     if ((err = check_exception())) {
         SWIG_exception(SWIG_IndexError, err);
     }
}


/*  Access image elements  */
extern PIXEL Video_point(int, int, int);
extern void Set_point(int, int, int, PIXEL);
extern void Xor_point(int, int, int);
/*  Image offset relative to the first image */
extern void Set_offset(int, double, double);
/*  Processing routines  */
extern int Detect_stars(int);
extern int Match_stars(int);
extern int Star_list(int);
extern int Load_star_list(int, char *);
extern void Blur(int);

/*  Market related routines  */
extern void Append_marker(int, float, float, float, char * NONNULL, 
       char * NONNULL, char * NONNULL);
extern void Remove_marker(int, int);


/*  Set the i-th image  */
extern void Set_image(int);
/*  Erase the i-th image  */
extern void Erase_image(int);
/*  Draw the i-th image  */
extern void Draw_image(int);

/*  DSS related routines  */

/*  Load an image and use existing image as an template */
extern int DSSload(int NONNEGATIVE);
/*  Download a DSS image with given coordinates and dimension and give */
/*  it a name  */
extern int get_dss_image(char *, char *, double, double, double, double, char *);

