/* swig_routines.c  */
/*  Jure Skvarc, December 1999  */
/* Wrappers for some functions to used by scripting languages  */
#include "../functs.h"

extern STATE state;
extern BLINK_FRAME *frame;


/*  For wrapper functions which are not normally used in fitsblink */
/*  define exception handler  */

int error_status;
char error_message[256];

void
throw_exception(char *msg)

{
    error_status = 1;
    strncpy(error_message, msg, 255);
}

char *
check_exception(void)

{
    if (error_status) return error_message;
    else return NULL;
}




/*   Return value of pixel at coordinates (x, y)   */
PIXEL
Video_point(int i, int x, int y)
    
{
  if (i < state.frame_used && i >= 0) { 
    return video_point(&frame[i].fits, x, y);
  }
  else throw_exception("Wrong image index"); 
  return 0;
}


/*   Set pixel with coordinates x, y to value  v  */
void
Set_point(int i, int x, int y, PIXEL v)
{
  if (i < state.frame_used && i >= 0) { 
    set_point(&frame[i].fits, x, y, v);
  }
  else throw_exception("Wrong image index");
}

void
Xor_point(int i, int x, int y)
{
  if (i < state.frame_used && i >= 0) { 
    xor_point(&frame[i].fits, x, y);
  }
  else throw_exception("Wrong image index");
}



/* Define x and y offsets of the given image  */
void
Set_offset(int i, double dx, double dy)

{
  if (i < state.frame_used && i >= 0) { 
    frame[i].dx = dx;
    frame[i].dy = dy;
    if (i == state.control[0]) {
      /*  Show offsets */
      show_left_offsets(frame, i);
      draw_zoom(frame, i, state.blinker->zoom1W, state.xf, state.yf);
      if (state.control_num == 0) {
	/*  Redraw the image  */
	set_image(frame, i);
	draw_image(frame, i);
      }
    }
    if (i == state.control[1] && state.control[1] != state.control[0]) {
      draw_zoom(frame, i, state.blinker->zoom2W, state.xf, state.yf);
      show_right_offsets(frame, i);
      if (state.control_num == 1) {
	/*  Redraw the image  */
	set_image(frame, i);
	draw_image(frame, i);
      }
    }
  }
  else throw_exception("Wrong image index");

}



/*  Detect stars in the image i  */
int
Detect_stars(int i)

{
  if (i < state.frame_used && i >= 0) { 
    return detect_stars(frame, i);
  }
  else throw_exception("Wrong image index");
  return 0;

}

/*  Match stars in the image i with the catalog */
int
Match_stars(int i, int write)

{
  int r;
  if (i < state.frame_used && i >= 0) { 
    if ((r = match_stars(frame, i)) == 0) {
      switch(write) {
      case 1:
	/*  Update WCS values in memory */
	break;
      case 2:
	/*  Update WCS in the file as well in the memory  */
	break;
      }
    }
    return r;
  }
  else throw_exception("Wrong image index");
  return 0;

}


/*  Output star list produced to a file  */
int
Star_list(int i)

{
  if (i < state.frame_used && i >= 0) { 
    return star_list(frame, i);
  }
  else throw_exception("Wrong image index");
  return 0;
  
}


/*  Read a star list from a given file */
int
Load_star_list(int i, char *name)

{
  if (i < state.frame_used && i >= 0) { 
    if (strlen(name) > 0) frame[i].fits.name = name;
    return load_star_list(frame, i);
  }
  else throw_exception("Wrong image index");
  return 0;
  
}


/*  Blur the given by a 3x3 kernel  */
void
Blur(int i)

{
  if (i < state.frame_used && i >= 0) { 
    /* Make the filtering  */
    blur(&frame[i].fits);
    /*  Redisplay the image  */
    set_image(frame, i);
    draw_image(frame, i);
  }
  else throw_exception("Wrong image index");
}


/*  Append a marker to the i-th frame  */
void
Append_marker(int i, float x, float y, float r, char *s, char *color, char *shape)

{
  /*  First check validity of values passed to this function  */
  if (i < state.frame_used && i >= 0) { 
    /* x and y do not need to be checked  */
    /* *s, *color and *shape will be checked by constaint in fitsblink.i  */
    append_marker(frame, i, x, y, r, s, find_marker_color(color), 
		  find_marker_shape(shape));
  }
  else throw_exception("Wrong image index");
}

/*  Remove a marker from the i-th frame  */
void
Remove_marker(int i, int k)

{
  if (i < state.frame_used && i >= 0) { 
    /* Check marker index  */
    if (k >= 0 && k < frame[i].objnum) 
      remove_marker(frame, i, k);
    else 
      throw_exception("Wrong marker index");
  }
  else throw_exception("Wrong image index");
}


/*  Set the i-th image  */
void
Set_image(int i)

{
  if (i < state.frame_used && i >= 0) { 
    set_image(frame, i);
  }
  else throw_exception("Wrong image index");
}

/*  Remove i-th image from the memory  */

void
Erase_image(int i)

{
  if (i < state.frame_used && i >= 0) { 
    erase_image(i);
  }
  else throw_exception("Wrong image index");
}

/*  Display the image on the screen  */
void
Draw_image(int i)

{
  if (i < state.frame_used && i >= 0) { 
    draw_image(frame, i);
  }
  else throw_exception("Wrong image index");
}


int
DSSload(int n)

{
  return dssload(frame, n);
}
