/*  table.c for fitsblink  */
/*  Jure Skvarc  */
/*  User interface routines for browsing FITS tables  */

#include <stdlib.h>
#include <math.h>

#include "functs.h"
extern STATE state;
FITS_TABLE table;

/*==============================================================*/
/*  Reserve space for table column names and pointer for lines  */
/*==============================================================*/
void
create_table(FITS_TABLE *table, char *name, int ncol, int nrows)

{
  int i;

  table->name = strdup(name);
  table->ncol = ncol;
  table->nrows = nrows;
  table->colnames = myalloc(sizeof(char *) * table->ncol, "create_table", 
			    "table->colnames");
  memset(table->colnames, 0, sizeof(char *) * table->ncol);
  table->lines = myalloc(sizeof(char *) * table->nrows, "create_table", 
			    "table->lines");
  memset(table->lines, 0, sizeof(char *) * table->nrows);
  table->selected = myalloc(sizeof(short) * table->ncol, "create_table", 
			    "table->selected");
  for (i = 0; i < table->ncol; i++) table->selected[i] = 1;
  table->width = myalloc(sizeof(short) * table->ncol, "create_table", 
			    "table->width");
  table->swidth = myalloc(sizeof(short) * table->ncol, "create_table", 
			    "table->swidth");
  table->type = myalloc(sizeof(short) * table->ncol, "create_table", 
			    "table->type");
}


/*====================================*/
/*  Return space occupied by a table  */
/*====================================*/
void
destroy_table(FITS_TABLE *table)

{
  int i;

  for (i = table->nkeys - 1; i >= 0; i--) {
    if (table->key[i]) free(table->key[i]);
  }
  free(table->key);
  free(table->type);
  free(table->swidth);
  free(table->width);
  free(table->selected);
  for (i = table->nrows - 1; i >= 0; i--) {
    if (table->lines[i]) free(table->lines[i]);
  }
  free(table->lines);
  for (i = table->ncol - 1; i >= 0; i--) {
    if (table->colnames[i]) free(table->colnames[i]);
  }
  free(table->colnames);
}


/*  Change the plot type  */
void
plottypeC(FL_OBJECT *obj, long val)

{
  int type = FL_NORMAL_XYPLOT;

  if (strcmp(fl_get_choice_text(state.tablegraph->plottypeW), "Lines") == 0) {
    type = FL_NORMAL_XYPLOT;
  }
  else if (strcmp(fl_get_choice_text(state.tablegraph->plottypeW), "Points") == 0) {
    type = FL_POINTS_XYPLOT;
  }
  else if (strcmp(fl_get_choice_text(state.tablegraph->plottypeW), "Lines+points") == 0) {
    type = FL_LINEPOINTS_XYPLOT;
  }
  else if (strcmp(fl_get_choice_text(state.tablegraph->plottypeW), "Fill") == 0) {
    type = FL_FILL_XYPLOT;
  }
  fl_set_xyplot_overlay_type(state.tablegraph->tgraphW, 1, type);
}


/* Draw data according to the columns chosen in choice objects */
void
tabledrawC(FL_OBJECT *obj, long val)

{
  float *x, *y;
  int i, j, n;
  int ix, iy;
  int *istart;
  int wmax;
  char *niz;
  int type = FL_NORMAL_XYPLOT;

  /*  Number of points */
  n = table.nrows;
  x = (float *) myalloc(sizeof(float) * n, "tabledrawC", "x");
  y = (float *) myalloc(sizeof(float) * n, "tabledrawC", "y");
  /*  Index of the x axis column  */
  ix = fl_get_choice(state.tablegraph->xaxisW) - 1;
  /*  Index of the x axis column  */
  iy = fl_get_choice(state.tablegraph->yaxisW) - 1;
  /*  Set indices of the starting points  */
  istart = (int *) myalloc(sizeof(int) * table.ncol, "tabledrawC", "istart");
  istart[0] = 0;
  for (i = 1; i < table.ncol; i++) {
    if (table.selected[i - 1]) {
      istart[i] = istart[i - 1] + table.swidth[i - 1] + 1;
    }
    else istart[i] = istart[i - 1];
  }
  wmax = istart[i - 1] + table.swidth[i - 1] + 2;
  niz = (char *) myalloc(sizeof(char) * wmax, "tabledrawC", "niz");
  j = 0;
  for (i = 0; i < n; i++) {
    if (!table.selections_only || 
	fl_isselected_browser_line(state.tablebrowser->tbrowserW, i + 1)) {
      strcpy(niz, fl_get_browser_line(state.tablebrowser->tbrowserW, i + 1));
      x[j] = strtod(niz + istart[ix], NULL);
      y[j] = strtod(niz + istart[iy], NULL);
      if (!(isnan(x[j]) || isnan(y[j]))) j++;
    }
  }
  n = j;
  fl_set_xyplot_data(state.tablegraph->tgraphW, x, y, n, "", 
		     fl_get_choice_text(state.tablegraph->xaxisW), 
		     fl_get_choice_text(state.tablegraph->yaxisW));
  fl_add_xyplot_overlay(state.tablegraph->tgraphW, 1, x, y, n, FL_BLACK);
  if (strcmp(fl_get_choice_text(state.tablegraph->plottypeW), "Lines") == 0) {
    type = FL_NORMAL_XYPLOT;
  }
  else if (strcmp(fl_get_choice_text(state.tablegraph->plottypeW), "Points") == 0) {
    type = FL_POINTS_XYPLOT;
  }
  fl_set_xyplot_overlay_type(state.tablegraph->tgraphW, 1, type);
  free(niz);
  free(istart);
  free(y);
  free(x);
}

void
choosetableC(FL_OBJECT *obj, long val)

{
}


/*  Select/deselect table columns  */
void
tselectallC(FL_OBJECT *obj, long val)

{
  int i;

  switch(val) {
  case 0:
    /*  Select all lines  */
    for (i = 1; i <= fl_get_browser_maxline(state.tablefields->tfieldsW); 
	 i++) {
      fl_select_browser_line(state.tablefields->tfieldsW, i);
    }
    break;
  case 1:
    /* Deselect all lines  */
      fl_deselect_browser(state.tablefields->tfieldsW);
    break;
  case 2:
    /* Reverse selection of all lines  */
    for (i = 1; i <= fl_get_browser_maxline(state.tablefields->tfieldsW); i++) {
      if (fl_isselected_browser_line(state.tablefields->tfieldsW, i)) {
	fl_deselect_browser_line(state.tablefields->tfieldsW, i);
      }
      else {
	fl_select_browser_line(state.tablefields->tfieldsW, i);
      }
    }
    break;
  }
}


/*  Select/deselect table rows  */
void
bselectallC(FL_OBJECT *obj, long val)

{
  int i;

  switch(val) {
  case 0:
    /*  Select all lines  */
    for (i = 1; i <= fl_get_browser_maxline(state.tablebrowser->tbrowserW); 
	 i++) {
      fl_select_browser_line(state.tablebrowser->tbrowserW, i);
    }
    break;
  case 1:
    /* Deselect all lines  */
      fl_deselect_browser(state.tablebrowser->tbrowserW);
    break;
  case 2:
    /* Reverse selection of all lines  */
    for (i = 1; i <= fl_get_browser_maxline(state.tablebrowser->tbrowserW); i++) {
      if (fl_isselected_browser_line(state.tablebrowser->tbrowserW, i)) {
	fl_deselect_browser_line(state.tablebrowser->tbrowserW, i);
      }
      else {
	fl_select_browser_line(state.tablebrowser->tbrowserW, i);
      }
    }
    break;
  }
}



void
tfieldsC(FL_OBJECT *obj, long val)

{
}


void
loadtableC(FL_OBJECT *obj, long val)

{
  const char *fn;
  static char *curdir = "";
  static char *pattern = "*.*";
  static char *name = "";
  fl_use_fselector(1);
  fl_invalidate_fselector_cache();
  fn = fl_show_fselector("Load FITS table", curdir, pattern, name);
  if (fn == NULL) {
    fl_hide_form(state.table->Table);
    return;
  }
  /*  Save the current directory  */
  curdir = (char *) fl_get_directory();
  /*  Save the current pattern */
  pattern = (char *) fl_get_pattern();
  if (table.name) destroy_table(&table);
  if (!read_table_header(fn, &table, state.tableheader->theaderW,
			 state.table->choosetableW, 
			 state.tablefields->tfieldsW,
			 state.tablegraph->xaxisW, state.tablegraph->yaxisW)) {
    read_table_contents(&table, state.tablebrowser->colnamesW,
			state.tablebrowser->tbrowserW, NULL, 
			state.table->choosetableW);
  }
  
}

void
tbrowserC(FL_OBJECT *obj, long val)

{
}

void
tableselC(FL_OBJECT *obj, long val)

{
  table.selections_only = fl_get_button(obj);
}



/*  Adjust x position of column names with the table content */
void
syncbrowsers(int xx, void *t)

{
  FL_Coord x;
  x = fl_get_browser_xoffset(state.tablebrowser->tbrowserW);
  fl_set_browser_xoffset(state.tablebrowser->colnamesW, x);
  fl_add_timeout(100, syncbrowsers, NULL);
}

void
check_selection(void)

{
  /*  Check if any field selection was changed */
  int i, same = 1;
  int ix, iy, first;
  for (i = 0; i < table.ncol; i++) {
    int t1, t2;
    t1 = fl_isselected_browser_line(state.tablefields->tfieldsW, i + 1) 
      != 0;
    t2 = table.selected[i] != 0;
    if (t1 != t2) { 
      same = 0;
    }
    table.selected[i] = t1;
  }
  if (!same) {
    read_table_contents(&table, state.tablebrowser->colnamesW,
			state.tablebrowser->tbrowserW, NULL, 
			state.table->choosetableW);
    /*  Grey out deselected column names in the graph window  */
    first = 0;
    for (i = 0; i < table.ncol; i++) {
      if (table.selected[i]) {
	fl_set_choice_item_mode(state.tablegraph->xaxisW, i + 1, FL_PUP_NONE);
	fl_set_choice_item_mode(state.tablegraph->yaxisW, i + 1, FL_PUP_NONE);
	if (first == 0) first = i;
      }
      else {
	fl_set_choice_item_mode(state.tablegraph->xaxisW, i + 1, FL_PUP_GREY);
	fl_set_choice_item_mode(state.tablegraph->yaxisW, i + 1, FL_PUP_GREY);
      }
    }
    /* If chosen column is deselected, find first selected column  */
    ix = fl_get_choice(state.tablegraph->xaxisW) - 1;
    if (!table.selected[ix]) {
      fl_set_choice(state.tablegraph->xaxisW, first + 1);
    }
    iy = fl_get_choice(state.tablegraph->yaxisW) - 1;
    if (!table.selected[iy]) {
      fl_set_choice(state.tablegraph->yaxisW, first + 1);
    }
  }
}

void
show_table(void)

{
  int out;
  XEvent event;
  FL_OBJECT *o;

  /*  The window in not created yet, do it now  */
  if (state.table == NULL) {
    table.ncol = 0;
    state.table = create_form_Table();
    state.tableheader = create_form_Tableheader();
    state.tablefields = create_form_Tablefields();
    state.tablebrowser = create_form_Tablebrowser();
    state.tablegraph = create_form_Tablegraph();
    fl_set_browser_fontstyle(state.tablebrowser->tbrowserW, FL_FIXED_STYLE);
    fl_set_browser_fontstyle(state.tablebrowser->colnamesW, FL_FIXED_STYLE);
    fl_set_browser_fontstyle(state.tableheader->theaderW, FL_FIXED_STYLE);

    fl_addto_tabfolder(state.table->tableW, "Fields", 
		       state.tablefields->Tablefields);
    fl_addto_tabfolder(state.table->tableW, "Header", 
		       state.tableheader->Tableheader);
    fl_addto_tabfolder(state.table->tableW, "Browser", 
		       state.tablebrowser->Tablebrowser);
    fl_addto_tabfolder(state.table->tableW, "Graph", 
		       state.tablegraph->Tablegraph);
    fl_add_timeout(100, syncbrowsers, NULL);
  }
  fl_show_form(state.table->Table,  FL_FIX_SIZE, FL_FULLBORDER | FL_PLACE_FREE_CENTER, "FITS table browser");

  out = 0;
  do {
    o = fl_check_forms();
    if (o == FL_EVENT) {
      fl_XNextEvent(&event);
    }
    else if (o == state.table->tablecloseW) {
      out = 1;
    }
    else if (o == state.table->exportW) {
      /*  Export table as an ASCII file  */
      const char *fn;
      static char *curdir = "";
      static char *pattern = "*.*";
      static char *name = "";
      fl_use_fselector(1);
      fl_invalidate_fselector_cache();
      fn = fl_show_fselector("Save table as ASCII file", curdir, pattern, name);
      if (fn) {
	FILE *fp = fopen(fn, "w");
	check_selection();
	if (!fp) {
	  fl_show_alert("Can't write table to file", fn, "", 1);
	}
	else {
	  int i;
	  for (i = 1; i <= fl_get_browser_maxline
		 (state.tablebrowser->tbrowserW); i++) {
	    if (!table.selections_only || 
		fl_isselected_browser_line(state.tablebrowser->tbrowserW, i)) {
	      fputs(fl_get_browser_line(state.tablebrowser->tbrowserW, i), fp);
	      fputc('\n', fp);
	    }
	  }
	  fclose(fp);
	}
      }
      /*  Save the current directory  */
      curdir = (char *) fl_get_directory();
      /*  Save the current pattern */
      pattern = (char *) fl_get_pattern();
    }
    if (fl_get_active_folder_number(state.table->tableW) == 3 ||
	fl_get_active_folder_number(state.table->tableW) == 4) {
      check_selection();
    }
  } while(!out);
  fl_hide_form(state.table->Table);
}

