/*  readtycho2.c  */
/*  (c)  Jure Skvarc  March 1998  */
/*  A program which reads stars from the TYCHO 2 catalog, which are  */
/*    contained in image with center coordinates */
/*  (ra, dec), rotation angle fi  and dimensions (width, height)     */
/*  and outputs their pixel coordinates          */
/*  TAN projection is assumed                    */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "../consts.h"
#include "../structs.h"
#include "../functs.h"

extern STATE state;
extern BLINK_FRAME *frame;

/*  Read TYCHO-2 index, pointed to by name  */
/*  If an error occurs, return nonzero value  */
int
read_tycho2_index(int *index, char *name)

{
  FILE *fp;
  int i;
  char line[44];

  fp = fopen(name, "r");
  /* Return if can't open the file  */
  if (fp == NULL) return 1;
  for (i = 0; i <= NUM_GSC_PLATES; i++) {
    /*  If reading of number fails, return with an error flag  */
    if (!fgets(line, 44, fp)) {
      fclose(fp);
      return 1;
    }
    if (sscanf(line, "%d", &index[i]) != 1) {
      fclose(fp);
      return 1;
    }
  }
  fclose(fp);
  return 0;
}

int
read_tycho2_region(char *name, TYCHO2_STAR **star, int *n, int plate, 
		   int *tycho_index)

{
  int first, last;
  int nstar;
  FILE *fp;
  int fields[] = {1, 6, 12, 14, 16, 29, 42, 50, 58, 62, 66, 71, 76, 84, 92, 95,
		99, 103, 107, 111, 118, 124, 131, 137, 141, 143, 149, 153, 166,
		179, 184, 189, 195, 201, 203};
  char line[TYCHO2_RECORDLEN + 1];
  int i;

  for (i = 0; i < sizeof(fields) / sizeof(int); i++) fields[i]--;
  /*  Indeks prve zvezde s plosce plate v katalogu */
  first = tycho_index[plate - 1];
  /*  Indeks prve zvezde s plosce plate v katalogu */
  last = tycho_index[plate];
  nstar = last - first;
  *star = myalloc(sizeof(TYCHO2_STAR) * nstar, "read_tycho2_region", "*star");
  /*  Odprem katalog in se namestim na prvo zvezdo */
  fp = fopen(name, "r");
  if (fp == NULL) return 1;
  fseek(fp, (first - 1) * TYCHO2_RECORDLEN, SEEK_SET);
  for (i = 0; i < nstar; i++) {
    /*  Preberem eno vrstico  */
    fgets(line, TYCHO2_RECORDLEN + 1, fp);
    /*  Kataloska oznaka  */
    strncpy((*star)[i].catid, line, fields[3]);
    /*  Oznaka glede povprecne lege  */
    sscanf(line + fields[3], "%c", &(*star)[i].meanposflag);
    /*  Povprecna RA pri epohi 2000  */
    sscanf(line + fields[4], "%lf", &(*star)[i].ra);
    /*  Povprecna DEC pri epohi 2000  */
    sscanf(line + fields[5], "%lf", &(*star)[i].dec);
    /*  Lastno gibanje v smeri RA  */
    sscanf(line + fields[6], "%f", &(*star)[i].pmra);
    /*  Lastno gibanje v smeri DEC  */
    sscanf(line + fields[7], "%f", &(*star)[i].pmdec);
    /*  Nedolocenost polozaja v smeri RA  */
    sscanf(line + fields[8], "%f", &(*star)[i].sra);
    /*  Nedolocenost polozaja v smeri DEC  */
    sscanf(line + fields[9], "%f", &(*star)[i].sdec);
    /*  Nedolocenost lastnega gibanja v smeri RA  */
    sscanf(line + fields[10], "%f", &(*star)[i].spmra);
    /*  Nedolocenost lastnega gibanja v smeri DEC  */
    sscanf(line + fields[11], "%f", &(*star)[i].spmdec);
    /*  Povprecna epoha polozaja v smeri RA  */
    sscanf(line + fields[12], "%f", &(*star)[i].mepra);
    /*  Povprecna epoha polozaja v smeri DEC  */
    sscanf(line + fields[13], "%f", &(*star)[i].mepdec);
    /*  Stevilo meritev, uporabljenih za dolocanje polozaja  */
    sscanf(line + fields[14], "%d", &(*star)[i].n);
    /*  Kvaliteta fita polozaja v smeri RA  */
    sscanf(line + fields[15], "%f", &(*star)[i].g_mra);
    /*  Kvaliteta fita polozaja v smeri DEC  */
    sscanf(line + fields[16], "%f", &(*star)[i].g_mdec);
    /*  Kvaliteta fita lastnega gibanja v smeri RA  */
    sscanf(line + fields[17], "%f", &(*star)[i].g_pmra);
    /*  Kvaliteta fita lastnega gibanja  v smeri DEC  */
    sscanf(line + fields[18], "%f", &(*star)[i].g_pmdec);
    /* Tycho2 B sij  */
    sscanf(line + fields[19], "%f", &(*star)[i].bt);
    /* Ocena napake za B sij  */
    sscanf(line + fields[20], "%f", &(*star)[i].sbt);
    /* Tycho2 V sij  */
    sscanf(line + fields[21], "%f", &(*star)[i].vt);
    /* Ocena napake za V sij  */
    sscanf(line + fields[22], "%f", &(*star)[i].svt);
    /*  Oznaka razdalje najblizje zvezde  */
    sscanf(line + fields[23], "%f", &(*star)[i].prox);
    /*  Oznaka, ali gre za TYCHO1 zvezdo  */
    sscanf(line + fields[24], "%c", &(*star)[i].tycho1flag);
    /*  Hipparcosovo stevilo  */
    sscanf(line + fields[25], "%d", &(*star)[i].hip);
    /*  CCDM oznaka komponente  */
    strncpy((*star)[i].ccdm, line + fields[26], 4);
    /*  RA v casu opazovanja  */
    sscanf(line + fields[27], "%lf", &(*star)[i].raobs);
    /*  DEC v casu opazovanja  */
    sscanf(line + fields[28], "%lf", &(*star)[i].decobs);
    /*  Epoha polozaja RA  */
    sscanf(line + fields[29], "%f", &(*star)[i].epra);
    /*  Epoha polozaja DEC  */
    sscanf(line + fields[30], "%f", &(*star)[i].epdec);
    /*  Nedolocenost opazovanega polozaja RA  */
    sscanf(line + fields[31], "%f", &(*star)[i].e_ra);
    /*  Nedolocenost opazovanega polozaja RA  */
    sscanf(line + fields[32], "%f", &(*star)[i].e_dec);
    /*  Tip TYCHO resitve  */
    sscanf(line + fields[33], "%c", &(*star)[i].posflg);
    /*  Korelacija opazovane resitve  */
    sscanf(line + fields[34], "%f", &(*star)[i].corr);
  }
  fclose(fp);
  *n = i;
  return 0;
}


/*  transforms s list of stars in TYCHO format into a list in "STAR" format */
/*  it copies only those which fall within image limits  */
/*  returns 1 if it runs out of space, 2 if user presses cancel  */
int
extract_tycho2_stars(char *name, STAR *starlist, MAP_IMAGE *image, 
		     int plate, double jd, int *tycho2_index)

{
  int i, nstar, err, n;
  double xpix, ypix, xinc, yinc;
  TYCHO2_STAR *star;
  int wmin, hmin, wmax, hmax;
  int n1 = state.control[state.control_num];
  float mag;
  double dt;

  nstar = image->nm;
  xinc = image->width / image->w;
  yinc = image->height / image->h;
  /*  Read all of the stars in file name  */
  wmin = -image->extarea * image->w;
  hmin = -image->extarea * image->h;
  wmax =  (1 + image->extarea) * image->w;
  hmax =  (1 + image->extarea) * image->h;
  if (read_tycho2_region(name, &star, &n, plate, tycho2_index)) {
    if (state.interactive) {
      fl_show_alert("Error reading TYCHO-2 catalog!", "", "", 0);
      return 1;
    }
    else {
      fprintf(stderr, "Error reading TYCHO-2 catalog!\n");
      exit(1);
    }
  }
  for (i = 0; i < n; i++) {
    /*  Check if the star falls into the image  */
    err = 0;
    fits_world_to_pix(star[i].ra, star[i].dec, image->ra, image->dec, 
		0.5 * image->w, 0.5 * image->h, 
		xinc, yinc, image->fi, "-TAN", 
		&xpix, &ypix, &err);
    if (!err && xpix >= wmin && xpix < wmax && ypix >= hmin && ypix < hmax) {
      switch(frame[n1].match.filter[0]) {
      case 'B':	
      case 'U':
	if (star[i].bt != 0 && star[i].vt != 0) {
	  mag = 0.760 * star[i].bt + 0.240 * star[i].vt;
	  starlist[nstar].smag = 0.760 * star[i].sbt + 0.240 * star[i].svt;
	}
	else 
	  /*  If the b magnitude can not be calculates, skip this star */
	  continue;
	break;
      case 'V':
      case 'W':
      case ' ':	
      case 'R':
      case 'I':
      default:
	if (star[i].bt != 0 && star[i].vt != 0) {
	  mag = 0.910 * star[i].vt - 0.090 * star[i].bt;
	  starlist[nstar].smag = 0.910 * star[i].svt - 0.090 * star[i].sbt;
	}
	else if (star[i].vt != 0) {
	  mag = star[i].vt;
	  starlist[nstar].smag = 2 * star[i].svt;
	}
	/* If VT magnitude is not given, skip this star  */
	else continue;
      }
      if (mag > image->minmag && mag < image->maxmag ) {
	dt = (jd - 2451545) / 365.25;
	starlist[nstar].catalog = 'T';
	starlist[nstar].n = starlist[nstar].catnum = 
	  100000 * strtol(star[i].catid, NULL, 10) + 10 * strtol(star[i].catid + 5, NULL, 10) + strtol(star[i].catid + 11, NULL, 10);
	sprintf(starlist[nstar].catid, "%04ld_%05ld_%01ld", 
		strtol(star[i].catid, NULL, 10), 
		strtol(star[i].catid + 5, NULL, 10),
		strtol(star[i].catid + 11, NULL, 10));
	starlist[nstar].m = -1;
	/* Do not take into account stars with no proper motion  */
	if (star[i].meanposflag == 'X') continue;
	/*  Make correction for the RA proper motion  */
	starlist[nstar].ra = star[i].ra + star[i].pmra * dt / cos(star[i].dec * DEGRAD) / 3.6e6;
	if (starlist[nstar].ra > 360) starlist[nstar].ra -= 360;
	if (starlist[nstar].ra < 0) starlist[nstar].ra += 360;
	/*  Make correction for the DEC proper motion  */
	starlist[nstar].dec = star[i].dec + star[i].pmdec * dt / 3.6e6;
	/*  Calculate error in coordinates  */
	starlist[nstar].sra = sqrt(fsqr(star[i].sra) + 
				   fsqr(((jd - julian_day((int) star[i].mepra, 1, 1.5)) / 365.25  - fmod(star[i].mepra, 1.0)) * star[i].spmra));
	starlist[nstar].sdec = sqrt(fsqr(star[i].sdec) + 
				    fsqr(((jd - julian_day((int) star[i].mepdec, 1, 1.5)) / 365.25 - fmod(star[i].mepdec, 1.0)) * star[i].spmdec));
	starlist[nstar].mag = mag;
	starlist[nstar].intensity = 1e9 * pow(10.0, -0.4 * mag);
	starlist[nstar].bluemag = star[i].bt;
	starlist[nstar].x = xpix - 1;
	starlist[nstar].y = ypix - 1;
	nstar++;
	if (nstar == MAXSTARNUM) {
	  image->nm = nstar;
	  return 1;
	}
      }
    }
  }
  free(star);
  image->nm = nstar;
  return 0;
}  


int
get_tycho2_stars(MAP_IMAGE *image, STAR *starlist, char *tycho2_path, double jd)
     
{
  STAR corner[4];
  double declow, dechigh;
  double ralow[2], rahigh[2];
  double ramin, ramax, decmin, decmax;
  double rawidth, rastart, rastep;
  int rarange;
  double xpix, ypix;
  int i, j, k;
  int i1, i2, ind;
  int err;
  int zonelown = 0, zonehighn = 0, zonelows = 0, zonehighs = 0;
  int fsmall;
  char *file;
  int lowindex, highindex;
  int cd;
  LARGE_REGION large[24];
  SMALL_REGION small[732];
  char *lpath, *spath, *indexpath;
  double xinc = image->width / image->w;
  double yinc = image->height / image->h;
  int tycho2index[NUM_GSC_PLATES + 1];
  
  
  /*  Create the name of the index file  */
  indexpath = (char *) myalloc(sizeof(char) * (strlen(tycho2_path) + 
					       strlen("/index.dat")) + 1,
			       "readtycho2", "indexpath");
  sprintf(indexpath, "%s/index.dat", tycho2_path);
  
  /*  Read the index file  */
  if (read_tycho2_index(tycho2index, indexpath)) {
    free(indexpath);
    if (state.interactive) {
      fl_show_alert("Error in reading TYCHO-2 index.dat!", "Write the correct path!", "", 0);
    }
    else {
      fprintf(stderr, "Error in reading  TYCHO-2 index.dat!\nWrite the correct path!\n");
    }
    return 1;
  }
  
  /*  image->nm counts stars which fall into our image  */
  calculate_corners(image, corner);
  /*  Find the highest and the lowest declination value  */
  declow = dechigh = corner[0].dec;
  ralow[0] = rahigh[0] = corner[0].ra;
  for (i = 1; i < 4; i++) {
    if (corner[i].dec < declow) declow = corner[i].dec;
    if (corner[i].dec > dechigh) dechigh = corner[i].dec;
    if (corner[i].ra < ralow[0]) ralow[0] = corner[i].ra;
    if (corner[i].ra > rahigh[0]) rahigh[0] = corner[i].ra;
  }
  /* Check if the north pole falls inside the rectangle  */
  err = 0;
  fits_world_to_pix(0.0, 90.0, image->ra, image->dec, 
		    0.5 * image->w, 0.5 * image->h, 
		    xinc, yinc, image->fi, "-TAN", 
		    &xpix, &ypix, &err);
  if (!err && (xpix >= 0.0 && xpix <= image->w && ypix >= 0.0 && ypix <= image->h)) {
    ralow[0] = 0;
    rahigh[0] = 360;
    rarange = 1;
    dechigh = 90.0;
  }
  else {
    /* Check if the south pole falls inside the rectangle  */
    err = 0;
    fits_world_to_pix(0.0, -90.0, image->ra, image->dec, 
		      0.5 * image->w, 0.5 * image->h, 
		      xinc, yinc, image->fi, "-TAN", 
		      &xpix, &ypix, &err);
    if (!err && (xpix >= 0.0 && xpix <= image->w && ypix >= 0.0 && ypix <= image->h)) {
      ralow[0] = 0;
      rahigh[0] = 360;
      rarange = 1;
      declow = -90.0;
    }
    else {
      if (rahigh[0] - ralow[0] >= 180) {
	ralow[1] = rahigh[0];
	rahigh[1] = 360;
	rahigh[0] = ralow[0];
	ralow[0] = 0;
	rarange = 2;
      }
      else {
	rarange = 1;
      }
    }
  }

  if (declow >= 0) {
    zonelown = (int) (declow / GUIDE_DECWIDTH);
    zonehighn = (int) (dechigh / GUIDE_DECWIDTH - 1e-9);
    cd = 1;    /*  Only use north pole cd  */
  }
  else if (declow > -GUIDE_DECWIDTH) {
    zonelown = -1;
    zonehighn = (int) (dechigh / GUIDE_DECWIDTH - 1e-9);
    cd = 1;    /*  Only use north pole cd  */
  } 
  else if (dechigh <= -GUIDE_DECWIDTH) {
    /* South hemisphere */
    zonelows = (int) (fabs(dechigh) / GUIDE_DECWIDTH) + 12;
    zonehighs = (int) (fabs(declow) / GUIDE_DECWIDTH - 1e-9) + 12;
    cd = 2;
  } else {
    /*  Both hemispheres  */
    zonelown = -1;
    zonehighn = (int) (dechigh / GUIDE_DECWIDTH - 1e-9);
    zonelows = 13;
    zonehighs = (int) (fabs(declow) / GUIDE_DECWIDTH - 1e-9) + 12;
    cd = 3;
  }
  /*  first extract stars from the northern sky  */
  /*  Calculate indices for declination zones  */
  /*  North hemisphere first  */
  if (cd & 1) {
    lpath = (char *) myalloc(sizeof(char) * (strlen(tycho2_path) + 
					     strlen("/lg_reg_x.tbl")) + 1,
			     "readtycho2", "lpath");
    spath = (char *) myalloc(sizeof(char) * (strlen(tycho2_path) + 
					     strlen("/sm_reg_x.tbl")) + 1,
			     "readtycho2", "spath");
    sprintf(lpath, "%s/lg_reg_x.tbl", tycho2_path);
    sprintf(spath, "%s/sm_reg_x.tbl", tycho2_path);
    if (read_large_regions(lpath, large)) {
      if (state.interactive) {
	fl_show_alert("Error in reading lg_reg_x.tbl", "Write the correct path!", "", 0);
      }
      else {
	fprintf(stderr, "Error in reading lg_reg_x.tbl!\nWrite the correct path!\n");
      }
      return 1;
    }
    if (read_small_regions(spath, small)) {
      if (state.interactive) {
	fl_show_alert("Error in reading sm_reg_x.tbl", "Write the correct path!", "", 0);
      }
      else {
	fprintf(stderr, "Error in reading sm_reg_x.tbl!\nWrite the correct path!\n");
      }
      return 1;
    }
    /*  Reserve space for the file name  */
    file = (char *) myalloc(sizeof(char) * (strlen(tycho2_path) + 50), 
			  "readtycho2", "file");
    for (i = zonelown; i <= zonehighn; i++) {
      if (i == -1) ind = 12;
      else ind = i;
      /*  ind is an index to large regions table */      
      for (j = 0; j < rarange; j++) {
	/*  Find lower and upper bound for large regions numbers */
	rawidth = 360.0 / large[ind].numlarge;
	lowindex = ralow[j] / rawidth + large[ind].firstlarge - 1;
	highindex = rahigh[j] / rawidth + large[ind].firstlarge - 1 - 1e-9;
	for (k = lowindex; k <= highindex; k++) {
	  fsmall = small[k].firstsmall;
	  rastart = (k + 1 - large[ind].firstlarge) * rawidth;
	  rastep =  rawidth / (double) small[k].depth;
	  /*  For every subdivision (small region),   */
	  /*  check if any star falls inside  */
	  for (i1 = 0; i1 < small[k].depth; i1++) {
	    if (i == -1) {
	      decmin = (i + (small[k].depth - 1 - i1) / (double) small[k].depth) * GUIDE_DECWIDTH;
	      decmax = decmin + GUIDE_DECWIDTH / small[k].depth;
	    }
	    else {
	      decmin = (i + i1 / 
		      (double) small[k].depth) * GUIDE_DECWIDTH;
	      decmax = decmin + GUIDE_DECWIDTH / small[k].depth;
	    }
	    if (!(decmax < declow || decmin > dechigh)) {
	      for (i2 = 0; i2 < small[k].depth; i2++) {
		ramin = rastart + i2 * rastep;
		ramax = ramin + rastep;
		if (!(ramax < ralow[j] || ramin > rahigh[j])) {
		  int plate = fsmall + i2 + i1 * small[k].depth;
		  sprintf(file, "%s/catalog.dat", tycho2_path);
		  if (image->nm < MAXSTARNUM) {
		    int k = extract_tycho2_stars(file, starlist, image, plate, jd, tycho2index);
		    if (k > 0) return 1;
		  }
		}
	      }
	    }
	  }
	}
      }
    }
    free(file);
    free(spath);
    free(lpath);
  }

  /*  Southern sky */
  if (cd & 2) {
    lpath = (char *) myalloc(sizeof(char) * (strlen(tycho2_path) + 
					     strlen("/lg_reg_x.tbl")) + 1,
			     "readtycho2", "lpath");
    spath = (char *) myalloc(sizeof(char) * (strlen(tycho2_path) + 
					     strlen("/sm_reg_x.tbl")) + 1,
			     "readtycho2", "spath");
    sprintf(lpath, "%s/lg_reg_x.tbl", tycho2_path);
    sprintf(spath, "%s/sm_reg_x.tbl", tycho2_path);
    if (read_large_regions(lpath, large)) {
      if (state.interactive) {
	fl_show_alert("Error in reading lg_reg_x.tbl", "Write the correct path!", "", 0);
      }
      else {
	fprintf(stderr, "Error in reading lg_reg_x.tbl!\nWrite the correct path!\n");
      }
      return 1;
    }
    if (read_small_regions(spath, small)) {
      if (state.interactive) {
	fl_show_alert("Error in reading sm_reg_x.tbl", "Write the correct path!", "", 0);
      }
      else {
	fprintf(stderr, "Error in reading sm_reg_x.tbl!\nWrite the correct path!\n");
      }
      return 1;
    }
    /*  Reserve space for the file name  */
    file = (char *) myalloc(sizeof(char) * (strlen(tycho2_path) + 50), 
			  "readtycho2", "file");


    for (i = zonelows; i <= zonehighs; i++) {
      ind = i;
      /*  ind is an index to large regions table */      
      for (j = 0; j < rarange; j++) {
	/*  Find lower and upper bound for large regions numbers */
	rawidth = 360.0 / large[ind].numlarge;
	lowindex = ralow[j] / rawidth + large[ind].firstlarge - 1;
	highindex = rahigh[j] / rawidth + large[ind].firstlarge - 1 - 1e-9;
	for (k = lowindex; k <= highindex; k++) {
	  fsmall = small[k].firstsmall;
	  rastart = (k + 1 - large[ind].firstlarge) * rawidth;
	  rastep =  rawidth / (double) small[k].depth;
	  /*  For every subdivision (small region),   */
	  /*  check if any star falls inside  */
	  for (i1 = 0; i1 < small[k].depth; i1++) {
	    decmin = (11 - i + (small[k].depth - 1 - i1) / (double) small[k].depth) * GUIDE_DECWIDTH;
	    decmax = decmin + GUIDE_DECWIDTH / small[k].depth;
	    if (!(decmax < declow || decmin > dechigh)) {
	      for (i2 = 0; i2 < small[k].depth; i2++) {
		ramin = rastart + i2 * rastep;
		ramax = ramin + rastep;
		if (!(ramax < ralow[j] || ramin > rahigh[j])) {
		  int plate = fsmall + i2 + i1 * small[k].depth;
		  sprintf(file, "%s/catalog.dat", tycho2_path);
		  if (image->nm < MAXSTARNUM) {
		    int k = extract_tycho2_stars(file, starlist, image, plate, jd, tycho2index);
		    if (k > 0) return 1;
		  }
		}
	      }
	    }
	  }
	}
      }
    }
    free(file);
    free(spath);
    free(lpath);
  }
  free(indexpath);
  return 0;
}




